#========================================================================================
# package      DE_EPAGES::AppStore::API::Object::AppsClient
# base         DE_EPAGES::EPagesJ::API::Object::RestClient
# state        public
# description  Client for appstore
#========================================================================================
package DE_EPAGES::AppStore::API::Object::AppsClient;
use base DE_EPAGES::EPagesJ::API::Object::RestClient;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::EPagesJ::API::Object::UriTemplate;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::CSV qw ( ParseCSV );

use Readonly;
use Try::Tiny;

Readonly my %URI_TEMPLATES => (
    apps        => 'appstore/{shopGUID}/apps',
    app         => 'appstore/{shopGUID}/apps/{appId}',
    install     => 'appstore/{shopGUID}/installations/{appId}',
    myApps      => 'appstore/{shopGUID}/installations',
    actions     => 'appstore/{shopGUID}/installations/{appId}/actions',
    scope       => 'appstore/{shopGUID}/installations/scope',
    findApp     => 'appstore/{shopGUID}/apps/app-url',
);

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::AppStore::API::Object::PrivateAppsClient->new( %args );
#----------------------------------------------------------------------------------------
# description  Instanciates a new OAuthTokenClient.
#----------------------------------------------------------------------------------------
# input        %args | Input arguments. Possible keys.<ul>
#                 <li>AppsUriTemplate - </li>
#                 <li>AppUriTemplate - ... - DE_EPAGES::EPagesJ::API::Object::UriTemplate</li>
#                 <li>LogCategory - category for commlogger. Defaults to "AppStore" - string</li>
#               </ul> | hash
# return       OAuthTokenClient | new instance | object
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;
    $args{LogCategory} //= 'AppStore';

    return $class->SUPER::new(%args);
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $TokenClient = DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient->create;
#----------------------------------------------------------------------------------------
# description  Factory method for a OAuthTokenClient object.
#               Initializes based on the current configuration.
#----------------------------------------------------------------------------------------
# input        $hArgs | arguments. TODO: document. | ref.hash
# return       $TokenClient | OAuthTokenClient object | DE_EPAGES::OAuthTokenClient::API::OAuth::OAuthTokenClient
#========================================================================================
sub create {
    my $class = shift;
    my ($hArgs) = @_;

    my $System = LoadRootObject;
    $hArgs->{BaseURL}      //= $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');
    $hArgs->{ShopType}     //= $hArgs->{Shop}->get('ShopTypeRef')->alias;
    $hArgs->{Shop}         //= $hArgs->{Shop};
    $hArgs->{BusinessUnit} //= ParseCSV($System->get('ProviderPath'), { 'sep_char' => '/' })->[-1];
    foreach my $key (keys %URI_TEMPLATES) {
        $hArgs->{UriTemplates}->{$key} = DE_EPAGES::EPagesJ::API::Object::UriTemplate->new($hArgs->{BaseURL}, $URI_TEMPLATES{$key});
    }

    return $class->SUPER::create( $System, $hArgs );

}

sub getApp {
    my $self = shift;
    my ($appId) = @_;

    my $url      = $self->_singleAppUrl($appId);
    $url->query_form(
        'locale'            => $self->_locale,
        'businessUnit'      => $self->_businessUnit,
        'shopType'          => $self->_shopType,
        'preferredCurrency' => $self->_currencyId,
        );
    my $Response;
    try {
            $Response = $self->client->get($url);
            return $self->_decodeFromResponse($Response);
    } catch {
            my $error =  $_;
            die $error->message;
    };

}

sub getAppLinks {
    my $self = shift;
    my ($appId) = @_;

    my $url      = $self->_appLinksUrl($appId);
    $url->query_form(
        'businessUnit'   => $self->_businessUnit,
        'shopType'       => $self->_shopType ,
        'locale'         => $self->_locale,
    );
    my $Response = $self->client->get($url);
    return $self->_decodeFromResponse($Response);
}

sub findAppIdByUrl{
    my $self = shift;
    my ($appType, $appUrl) =@_;

    my $url      = $self->_findAppId;
    $url->query_form(
        'businessUnit'   => $self->_businessUnit,
        'shopType'       => $self->_shopType,
        'locale'         => $self->_locale,
        'page'           => 0,
        'resultsPerPage' => 12,
        'appType'        => $appType,
        'appUrl'         => $appUrl,
    );
    my $Response = $self->client->get($url);
    my $decodedResponse = $self->_decodeFromResponse($Response);

    return undef unless isValid($decodedResponse ->{id});
    return $decodedResponse ->{id};
}

sub submitAppConsent {
    my $self    = shift;
    my ($appId, $scope) = @_;
    my $url     = $self->_installAppUrl($appId);
    $url->query_form(
        'businessUnit' => $self->_businessUnit(),
        'shopType'     => $self->_shopType(),
        'scope'        => $scope,
        'locale'       => $self->_locale()
    );
    my $Response = $self->client->post($url);

    return $self->_decodeFromResponse($Response);
}

sub uninstall {
    my $self = shift;
    my ($appId) = @_;

    my $url      = $self->_installAppUrl($appId);
    $self->client->delete($url);
    return;
}


sub listApps {
    my $self = shift;
    my ($Page, $AppType, $categoryId) = @_;
    $Page //= 1;

    my $hQueryParams = {
            'businessUnit'      => $self->_businessUnit,
            'shopType'          => $self->_shopType,
            'locale'            => $self->_locale,
            'preferredCurrency' => $self->_currencyId,
            'appTypes'          => $AppType,
            'page'              => $Page - 1,
            'resultsPerPage'    => 12,
            'categoryId'        => $categoryId,
        };
    my $url = $self->_appsUrl;
    $url->query_form($hQueryParams);
    my $Response = $self->client->get( $url, undef, undef );

    return $self->_decodeFromResponse($Response);
}

#========================================================================================
# function     myApps
# state        public
#----------------------------------------------------------------------------------------
# syntax       myApps( $Page );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Page | page (starting with 1) | int
# input        $state | installation state (optional) | string
# return       $hMyApps | list of of installed apps | ref.hash
#========================================================================================
sub myApps {
    my $self = shift;
    my ($Page, $state) = @_;
    $Page //= 1;

    $state //= 'INSTALLED';
    my $url = $self->_myAppsUrl($state);
    $url->query_form('state', $state , 'locale' => $self->_locale);

    my $Response = $self->client->get( $url, undef, undef );

    return $self->_decodeFromResponse($Response);
}

#========================================================================================
# function     isScope
# state        public
#----------------------------------------------------------------------------------------
# syntax       isScope( $Scope );
#----------------------------------------------------------------------------------------
# description  Description
#----------------------------------------------------------------------------------------
# input        $Scope | Scope string | String
# return       $IsScope | Whether is scope or not | object
#========================================================================================
sub isScope {
    my $self = shift;
    my ($Scope) = @_;

    return unless isValid($Scope);

    my $Url = $self->_scopeUrl($Scope);
    my $Response = $self->client->get( $Url, undef, undef );

    return $self->_decodeFromResponse($Response);
}

sub _decodeFromResponse {
    my $self = shift;
    my ($Response) = @_;

    return $self->decodeResponse( $Response->decoded_content );
}

sub _shop {
    my $self = shift;
    return $self->{Shop};
}

sub _appsUrl {
    my $self = shift;
    return $self->{UriTemplates}->{apps}->uri({ shopGUID => $self->_shop->guid });
}

sub _findAppId {
    my $self = shift;
    return $self->{UriTemplates}->{findApp}->uri({ shopGUID => $self->_shop->guid });
}

sub _locale {
    my $self = shift;
    return $self->{Locale};
}

sub _currencyId {
    my $self = shift;
    my $Shop = $self->_shop();
    return $Shop->get('CurrencyID');
}

sub _shopType {
    my $self = shift;
    return $self->{ShopType};
}

sub _businessUnit {
    my $self = shift;
    return $self->{BusinessUnit};
}

sub _myAppsUrl {
    my $self = shift;
    return $self->{UriTemplates}->{myApps}->uri( { shopGUID => $self->_shop->guid } );
}

sub _singleAppUrl {
    my $self = shift;
    my ($appId) = @_;
    return $self->{UriTemplates}->{app}->uri( { shopGUID => $self->_shop->guid, appId => $appId } );
}

sub _installAppUrl {
    my $self = shift;
    my ($appId) = @_;
    return $self->{UriTemplates}->{install}->uri( { shopGUID => $self->_shop->guid, appId => $appId }  );
}

sub _appLinksUrl {
    my $self = shift;
    my ($appId) = @_;
    return $self->{UriTemplates}->{actions}->uri( { shopGUID => $self->_shop->guid, appId => $appId }  );
}

sub _scopeUrl {
    my $self = shift;
    my ($Scope) = @_;
    return $self->{UriTemplates}->{scope}->uri( { shopGUID => $self->_shop->guid, scope => $Scope }  );
}

1;
