#========================================================================================
# state        private
#========================================================================================
package DE_EPAGES::AppStore::t::AppsClient;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use Test::MockObject;
use Test::More;

use strict;

use constant APP_ID => '333';

## no critic qw(ProhibitLongLines)

sub init {
    my $self = shift;

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
        GUID => 'shopGUID',
        CurrencyID => 'EUR',
    });

    $self->{'Client'} = DE_EPAGES::AppStore::API::Object::AppsClient->create( {
            BaseURL  => 'http://localhost:8088/rs-test/',
            Shop     => $Shop,
            ShopType => 'eCom100',
            Locale   => 'en_GB',
            BusinessUnit => 'blabla',
        }
    );
    return;
}

sub test {
    my $self = shift;
    $self->testFindAppIdByUrl();
    $self->testPostAppConsentRequestPreparation();
    $self->testPostAppConsentResponse();
    $self->testGetSingleApp();
    $self->testGetAppList();
    $self->testMyApps();
    $self->testGetAppActionsUninstallApp();
}

sub testFindAppIdByUrl() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertFindAppIdByUrlContent, } );

    my $responseId = $Client->findAppIdByUrl('SOAP_APP','onesaas.com');
    is( $responseId, '54e207786a1a63f3019bb67b', "$SubTestName - correct app id found from appUrl" );
}

sub testPostAppConsentRequestPreparation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'post' => \&_assertPostAppConsentContent, } );

    # check url
    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_installAppUrl(APP_ID), qr#appstore/shopGUID/installations/333#, "$SubTestName - post app consent url OK" );
    is( $Client->_businessUnit() , 'blabla', "$SubTestName - business unit OK");
    is( $Client->_shopType() , 'eCom100', "$SubTestName - shop type OK");
    is( $Client->_locale() , 'en_GB', "$SubTestName - locale OK");
}

sub testPostAppConsentResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'post' => \&_assertPostAppConsentContent, } );

    # post app consent
    my $response = $Client->submitAppConsent(APP_ID, 'products_read');

    is( $response->{'shopId'}, 'sID', "$SubTestName - correct shopId");
    is( $response->{'appId'}, 'aID', "$SubTestName - correct appId" );
    is( $response->{'callbackUrl'}, 'http://callback', "$SubTestName - correct callbackUrl" );
}

sub testGetSingleApp {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetAppContent, } );

    # check url
    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_singleAppUrl(APP_ID . 'aaa'), qr#appstore/shopGUID/apps/333aaa#, "$SubTestName - single appstore app url OK" );

    # get single app
    my $privateApp = $Client->getApp(APP_ID);
    is( $privateApp->{'name'}, 'bourneIdentityGET', "$SubTestName - App name is set to: $privateApp->{'name'}" );
    is( $privateApp->{'logoUrl'}, 'http://hostaname/logo.jpg', "$SubTestName - App logo is set to: $privateApp->{'logoUrl'}" );
    is( $privateApp->{'price'}, 'free', "$SubTestName - App price is set to: $privateApp->{'price'}" );
    is( $privateApp->{'developer'}, 'mister developer', "$SubTestName - App price is set to: $privateApp->{'developer'}" );
    my @aRoles = $privateApp->{'scopes'};
    is( $aRoles[0][0], 'products_read', "$SubTestName - App first role is set to: $aRoles[0][0]" );
}

sub testGetAppList {
    my $self   = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetAppListContent, } );

    like( $Client->_appsUrl(), qr#appstore/shopGUID/apps#, "$SubTestName - multiple appstore apps url OK" ); ## no critic (ProhibitPrivateSubroutineUse)

    # get list apps
    my @aList = $Client->listApps();
    is( $aList[0][0]->{'name'}, 'KittenApp', "$SubTestName - App list first name is : $aList[0][0]->{'name'}" );
}

sub testMyApps {
    my $self   = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetMyAppsContent, } );

    like( $Client->_myAppsUrl(), qr#appstore/shopGUID/installations#, "$SubTestName - my apps url OK" ); ## no critic (ProhibitPrivateSubroutineUse)

    # get my apps
    my @aList = $Client->myApps();
    is( $aList[0][0]->{'name'}, 'KittenApp', "$SubTestName - My First App is : $aList[0][0]->{'name'}" );
    is( $aList[0][1]->{'name'}, 'KittenApp2', "$SubTestName - My Second App is : $aList[0][1]->{'name'}" );
    is( $aList[0][0]->{'logoUrl'}, 'http://somehost/someLogo.png', "$SubTestName - My First App Logo is : $aList[0][0]->{'logoUrl'}" );
    is( $aList[0][1]->{'logoUrl'}, 'http://somehost/someLogo2.png', "$SubTestName - App Second App Logo is : $aList[0][1]->{'logoUrl'}" );
}

sub testGetAppActionsUninstallApp() {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetAppActions, } );

    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_appLinksUrl(APP_ID), qr#appstore/shopGUID/installations/333/actions#, "$SubTestName - get app action url ok" );

    my $response = $Client->getAppLinks(APP_ID);

    my  @aLinksActions = $response->{'links'};
    is( $aLinksActions[0][0]->{'rel'}, 'installApp', "$SubTestName - correct rel of installApp link" );
    is( $aLinksActions[0][1]->{'rel'}, 'uninstallApp', "$SubTestName - correct rel of uninstallApp link" );
    is( $aLinksActions[0][2]->{'rel'}, 'openApp', "$SubTestName - correct rel of openApp link" );

    is( $aLinksActions[0][0]->{'href'},
            'http://localhost:8088/rs/appstore/shopGUID/installations/' . APP_ID,
            "$SubTestName - correct installApp link location"
    );
    is( $aLinksActions[0][1]->{'href'},
            'http://localhost:8088/rs/appstore/shopGUID/installations/' . APP_ID,
            "$SubTestName - correct uninstallApp link location"
    );
    is( $aLinksActions[0][2]->{'href'},
            'http://localhost:8088/rs/appstore/shopGUID/installations/' . APP_ID,
            "$SubTestName - correct openApp link location"
    );
}

#=============================================================================================
# Mocks
#=============================================================================================
sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _assertGetAppActions {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    my $SubTestName = (caller(0))[3];

    ## no critic (ProhibitComplexRegexes)
    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/installations/333/actions|, "$SubTestName - URL" );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
    {
        "links": [
        {
            "rel": "installApp",
            "href": "http://localhost:8088/rs/appstore/shopGUID/installations/333",
            "title": null
        },{
            "rel": "uninstallApp",
            "href": "http://localhost:8088/rs/appstore/shopGUID/installations/333",
            "title": null
        },{
            "rel": "openApp",
            "href": "http://localhost:8088/rs/appstore/shopGUID/installations/333",
            "title": null
        }
  ]
}
EOL
    );
}

sub _assertPostAppConsentContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    my $SubTestName = (caller(0))[3];

    ## no critic (ProhibitComplexRegexes)
    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/installations/333|, "$SubTestName - URL" );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
       {
                "shopId":"sID",
                "appId":"aID",
                "callbackUrl":"http://callback",
                "state":"PENDING"
       }
EOL
    );

}

sub _assertGetAppContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    my $SubTestName = (caller(0))[3];

    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/apps/333|, "$SubTestName - URL" );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
       {
            "id": "333",
            "scopes": [ "products_read" ],
            "iconUrl": "http://i.imgur.com/dKXXmmKs.png",
            "logoUrl": "http://hostaname/logo.jpg",
            "name": "bourneIdentityGET",
            "shortDescription": "short description",
            "video": "http://video.url",
            "images": [
                {
                    "image": [
                        {
                            "url": "http://i.imgur.com/dKXXmmKt.png",
                            "classifier": "small"
                        },
                        {
                            "url": "http://i.imgur.com/dKXXmmK.png",
                            "classifier": "large"
                        }
                    ]
                },
                {
                    "image": [
                        {
                            "url": "http://i.imgur.com/jkvFOEet.png",
                            "classifier": "small"
                        },
                        {
                            "url": "http://i.imgur.com/jkvFOEe.png",
                            "classifier": "large"
                        }
                    ]
                },
                {
                    "image": [
                        {
                            "url": "http://i.imgur.com/CCRbV9Ft.png",
                            "classifier": "small"
                        },
                        {
                            "url": "http://i.imgur.com/CCRbV9F.png",
                            "classifier": "large"
                        }
                    ]
                }
            ],
            "features": [
                "feature1",
                "feature2"
            ],
            "description": "description",
            "price": "free",
            "developer": "mister developer",
            "requirements": "requirement 1, requirement 2",
            "support": {
                "email": "support@comp.any",
                "website": "http://comp.any"
            },
            "links": []
        }
EOL
    );
}

sub _assertGetAppListContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;
    my $SubTestName = (caller(0))[3];

    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/apps|, "$SubTestName - URL" );
    like( $URL, qr|shopType=eCom100|, "$SubTestName - shopType" );
    like( $URL, qr|locale=en_GB|, "$SubTestName - locale" );
    like( $URL, qr|page=0|, "$SubTestName - page" );
    like( $URL, qr|businessUnit=blabla|, "$SubTestName - businessUnit" );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
[
    {
           "id": "54e207786a1a63f3019bb67b",
           "name": "KittenApp",
           "shopGUID": "developerguid",
           "callbackUrl": "callback",
           "notificationUrl": "notification",
           "logoUrl": "http://somehost/someLogo.png",
           "price": "free",
           "shopType": "eCom100"
   }
]
EOL
    );
}

sub _assertGetMyAppsContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
[
    {
           "id": "54e207786a1a63f3019bb67b",
           "name": "KittenApp",
           "logoUrl": "http://somehost/someLogo.png"
    },
    {
           "id": "54e207786a1a63f3019bb67c",
           "name": "KittenApp2",
           "logoUrl": "http://somehost/someLogo2.png"
    },
    {
           "id": "54e207786a1a63f3019bb67d",
           "name": "KittenApp3",
           "logoUrl": "http://somehost/someLogo3.png"
    }
]
EOL
    );
}

sub _assertFindAppIdByUrlContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
        {
           "id": "54e207786a1a63f3019bb67b",
           "name": "KittenApp",
           "logoUrl": "http://somehost/someLogo.png"
        }
EOL
    );
}

__PACKAGE__->runScript;
done_testing();
