#========================================================================================
# package      DE_EPAGES::Catalog::API::Catalog
# state        public
#----------------------------------------------------------------------------------------
# description  Functions to assign or remove products to categories.
#========================================================================================
package DE_EPAGES::Catalog::API::Catalog;
use base qw(Exporter);

use strict;
our @EXPORT_OK = qw (
    AssignProductToCategory
    IsProductAssignedToCategory
    IsProductAssignedToCategoryWithParent
    RemoveProductFromCategory
    RenumberCategoryProducts
    RenumberProductCategories
    SortProductList
    SearchTerms
);

use DE_EPAGES::DataCache::API::Cache qw (UpdateCache);
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Catalog::DAL::Category qw ( DAL_GetInfoSearchTermsBySiteID );
use DE_EPAGES::Catalog::DAL::CategoryProductMap qw (
    DAL_GetProductIDsByCategoryID
    DAL_RenumberCategoryProductsByCategoryID
);
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::TestParameter qw (TestObject TestString TestInteger);
use DE_EPAGES::Object::API::Constants qw (POSITION_START POSITION_STEP);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias Transaction);
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::Catalog::API::Constants;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NOT_A_CATEGORY'  => 'Object with path #Path is not a category.',
        'SHOP_MISMATCH'   => 'Product #ProductPath and Category #CategoryPath do not belong to the same shop.'
    };
}
#ERRORDEFINITION END

use constant BIG_POSITION => 9999;

#========================================================================================
# function     AssignProductToCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ret = AssignProductToCategory($Product, $Category);
#               $ret = AssignProductToCategory($Product, $Category, 25);
#----------------------------------------------------------------------------------------
# description  Assign a product to a catalog. The products will not be renumbered.
#               If the product is alreday assigned to the category, nothing happens.
#               If the super product of a subproduct is already assigned, nothing happens.
#----------------------------------------------------------------------------------------
# input        $Product   | the product  | product object
# input        $Category  | the category | category object
# input        $Position  | the position for sorting the product list | integer (optional)
# input        $PositionCategory | the position for sorting the category list | integer (optional)
# return       $ret       | 1 - product has been assigned
#                            2 - product already assigned before; position has been updated if passed
#                            3 - super product already assigned before; no changes apply | integer
#========================================================================================
sub AssignProductToCategory {
    my ($Product, $Category, $Position, $PositionCategory) = @_;
    TestObject('Product',  $Product,  'DE_EPAGES::Product::API::Object::Product');
    TestObject('Category', $Category);
    Error('NOT_A_CATEGORY', {'Path' => $Category->get('Path')}) unless $Category->instanceOf('Category');

    #--- product and category must belong to the same shop
    Error('SHOP_MISMATCH', {'ProductPath' => $Product->get('Path'), 'CategoryPath' => $Category->get('Path')})
        if ($Category->getSite()->id != $Product->getSite()->id);

    my $categoryID = $Category->id;
    my $productID  = $Product->id;
    my $MapClass = LoadClassByAlias('CategoryProductMap');

    #--- check if product already assigned
    foreach my $Map (@{ $MapClass->getObjects('ByCategoryIDProductID', [$categoryID, $productID]) }) {

        #--- product assigned already, update position if specified
        if (defined($Position) && $Position =~ /^\d+$/) {
            $Map->set({'Position' => $Position});
        }
        #--- product assigned already, update position for category if specified
        if (defined($PositionCategory) && $PositionCategory =~ /^\d+$/) {
            $Map->set({'PositionCategory' => $PositionCategory});
        }
        return 2;
    }

    #--- handle variations; check if super product already assigned
    my $SuperProduct = $Product->get('SuperProduct');
    if ($SuperProduct) {
        #--- check if super product already assigned
        return 3 if 0 < scalar @{ $MapClass->findObjects('ByCategoryIDProductID', [$categoryID, $SuperProduct->id]) };
    }

    my $hParams = {
        'Object'  => $Category,
        'Product' => $Product,
        'AllowedToAddProducts' => 1,
    };
    TriggerHook('API_BeforeAssignProductToCategory', $hParams);

    if ($hParams->{'AllowedToAddProducts'}) {
        #--- assign product
        $Position = BIG_POSITION unless (defined($Position) && $Position =~ /^\d+$/);
        $PositionCategory = BIG_POSITION unless (defined($PositionCategory) && $PositionCategory =~ /^\d+$/);
        $MapClass->insertObject({
            'Product'  => $Product,
            'Category' => $Category,
            'Position' => $Position,
            'PositionCategory' => $PositionCategory,
        });
    }

    return 1;
}

#========================================================================================
# function     RemoveProductFromCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ret = RemoveProductFromCategory($Product, $Category);
#----------------------------------------------------------------------------------------
# description  Remove a product from a category. The products will not be renumbered.
#               If the product is not yet assigned no changes apply, even if its
#               superproduct is assigned to the category.
#----------------------------------------------------------------------------------------
# input        $Product   | the product  | product object
# input        $Category  | the category | category object
# return       $ret       | 1 - product has been removed
#                            2 - product had not been already assigned before | integer
#========================================================================================
sub RemoveProductFromCategory {
    my ($Product, $Category) = @_;
    TestObject('Product',  $Product, 'DE_EPAGES::Product::API::Object::Product');
    TestObject('Category', $Category);
    Error('NOT_A_CATEGORY', {'Path' => $Category->get('Path')}) unless $Category->instanceOf('Category');

    #--- product and category must belong to the same shop
    Error('SHOP_MISMATCH', {'ProductPath' => $Product->get('Path'), 'CategoryPath' => $Category->get('Path')})
        if ($Category->getSite()->id != $Product->getSite()->id);

    my $categoryID = $Category->id;
    my $productID  = $Product->id;

    #--- remove map object if product is assigned
    foreach my $Map (@{ LoadClassByAlias('CategoryProductMap')->getObjects('ByCategoryIDProductID', [$categoryID, $productID]) }) {
        $Map->delete;
        return 1;
    }

    #--- product had not been assigned before
    return 2;
}

#========================================================================================
# function     RenumberCategoryProducts
# state        public
#----------------------------------------------------------------------------------------
# syntax       RenumberCategoryProducts($Category);
#----------------------------------------------------------------------------------------
# description  Renumber the products assigned to a category.
#----------------------------------------------------------------------------------------
# input        $Category  | the category | category object
#========================================================================================
sub RenumberCategoryProducts {
    my ($Category) = @_;
    TestObject('Category', $Category);
    Error('NOT_A_CATEGORY', {'Path' => $Category->get('Path')}) unless $Category->instanceOf('Category');

    my $dbi = GetCurrentDBHandle();
    my $aAffectedProductMapIDs = DAL_GetProductIDsByCategoryID($Category->id);

    # renumber on db level
    DAL_RenumberCategoryProductsByCategoryID($Category->id, $RENUMBER_STEP_SIZE);

    # remove cache for category
    UpdateCache($dbi->cacheKey, 'Object', $Category->id);
    UpdateCache($dbi->cacheKey, 'InfoObject', $Category->id);
    UpdateCache($dbi->cacheKey, 'InfoContentObject', $Category->id);

    # remove DAL Cache for category
    my $DAL_PACKAGE = DE_EPAGES::Database::API::DAL->load('DE_EPAGES::Object::API::Object::Object');
    $DAL_PACKAGE->cacheUpdate($Category->id, undef, 0);

    # remove caches for categoryproductmaps
    foreach my $aDBResultRow (@$aAffectedProductMapIDs){
        UpdateCache($dbi->cacheKey, 'Object', $aDBResultRow->[0]);
        UpdateCache($dbi->cacheKey, 'InfoObject', $aDBResultRow->[0]);
        UpdateCache($dbi->cacheKey, 'InfoContentObject', $aDBResultRow->[0]);
        UpdateCache($dbi->cacheKey, 'InfoCategoryProductMap', $aDBResultRow->[0]);

        # remove DAL cache for categoryproductmap
        $DAL_PACKAGE = DE_EPAGES::Database::API::DAL->load('DE_EPAGES::Object::API::Object::Object');
        $DAL_PACKAGE->cacheUpdate($aDBResultRow->[0], undef, 0);
    }

    # synchronize cache

    return;
}

#========================================================================================
# function     RenumberProductCategories
# state        public
#----------------------------------------------------------------------------------------
# syntax       RenumberProductCategories($Product);
#----------------------------------------------------------------------------------------
# description  Renumber the categories assigned to a product.
#----------------------------------------------------------------------------------------
# input        $Product  | the product | product object
#========================================================================================
sub RenumberProductCategories {
    my ($Product) = @_;
    TestObject('Product', $Product);

    my $aMaps = $Product->get('CategoryProductMaps'); # sorted already

    my $position = POSITION_START;
    my $SuperProduct = $Product->get('SuperProduct');
    if ($SuperProduct) {
        $position = POSITION_START * (scalar @{ $SuperProduct->get('CategoryProductMaps') }) + POSITION_START;
    }
    foreach my $Map(@$aMaps) {
        $Map->set({'PositionCategory', $position});
        $position += POSITION_STEP;
    }

    return;
}

#========================================================================================
# function     IsProductAssignedToCategoryWithParent
# state        public
#----------------------------------------------------------------------------------------
# syntax       if (IsProductAssignedToCategoryWithParent($Product, $Category)) {...
#----------------------------------------------------------------------------------------
# description  Indicates whether or not a product is assigned to a category.
#               For subproducts true will be returned if the superproduct or the
#               subproduct (or both) are assigned to the category.
#----------------------------------------------------------------------------------------
# input        $Product   | the product  | product object
# input        $Category  | the category | category object
# return       $ret       | true - product is assigned to category
#                            false - product is not assigned to category | boolean
#========================================================================================
sub IsProductAssignedToCategoryWithParent {
    my ($Product, $Category) = @_;
    TestObject('Product',  $Product, 'DE_EPAGES::Product::API::Object::Product');
    TestObject('Category', $Category);
    Error('NOT_A_CATEGORY', {'Path' => $Category->get('Path')}) unless $Category->instanceOf('Category');

    #--- product and category must belong to the same shop
    Error('SHOP_MISMATCH', {'ProductPath' => $Product->get('Path'), 'CategoryPath' => $Category->get('Path')})
        if ($Category->getSite()->id != $Product->getSite()->id);

    my $categoryID = $Category->id;
    my $MapClass = LoadClassByAlias('CategoryProductMap');

    #--- check if product assigned to category
    return 1 if scalar @{$MapClass->findObjects('ByCategoryIDProductID', [$categoryID, $Product->id])};

    #--- for sub products, check if super product assigned to category
    my $SuperProduct = $Product->get('SuperProduct');
    if ($SuperProduct) {
        return 1 if scalar @{$MapClass->findObjects('ByCategoryIDProductID', [$categoryID, $SuperProduct->id])};
    }

    #--- neither product nor super product assigned to category
    return 0;
}

#========================================================================================
# function     IsProductAssignedToCategory
# state        public
#----------------------------------------------------------------------------------------
# syntax       if (IsProductAssignedToCategory($Product, $Category)) {...
#----------------------------------------------------------------------------------------
# description  Indicates whether or not a product is assigned to a category.
#               For subproducts true will be returned only if the subproduct itself is
#               assigned to the category.
#----------------------------------------------------------------------------------------
# input        $Product   | the product  | product object
# input        $Category  | the category | category object
# return       $ret       | true - product is assigned to category
#                            false - product is not assigned to category | boolean
#========================================================================================
sub IsProductAssignedToCategory {
    my ($Product, $Category) = @_;
    TestObject('Product',  $Product, 'DE_EPAGES::Product::API::Object::Product');
    TestObject('Category', $Category);
    Error('NOT_A_CATEGORY', {'Path' => $Category->get('Path')}) unless $Category->instanceOf('Category');

    #--- product and category must belong to the same shop
    Error('SHOP_MISMATCH', {'ProductPath' => $Product->get('Path'), 'CategoryPath' => $Category->get('Path')})
        if ($Category->getSite()->id != $Product->getSite()->id);

    #--- check if product assigned to category
    my $aMaps = LoadClassByAlias('CategoryProductMap')->findObjects('ByCategoryIDProductID', [$Category->id, $Product->id]);
    return scalar @$aMaps;
}

#========================================================================================
# function     SortProductList
# state        public
#----------------------------------------------------------------------------------------
# syntax       SortProductList($Category, 'byStockLevel');
#----------------------------------------------------------------------------------------
# description  Changes the sort order of products in category list accordning to the
#               given sort criteria.
#----------------------------------------------------------------------------------------
# input        $Category  | the category | category object
# input        $sortName  | sort criteria, i.e. alphabetize, byPriceASC,
#               byPriceDESC, byNew, byStockLevel, byProductAlias | string
#========================================================================================
sub SortProductList {
    my ($Category, $sortName) = @_;

    my $aMaps = $Category->get('CategoryProductMaps');
    my $sortSub = _sortSub($sortName, $Category->getSite()->get('CurrencyID') );
    my @Maps = sort $sortSub @$aMaps;
    my $Position = POSITION_START;
    RunBatchProcess('UpdateProducts', sub{
        Transaction( sub {
            foreach my $Map (@Maps) {
                $Map->set({'Position' => $Position});
                $Position += POSITION_STEP;
            }
            RenumberCategoryProducts($Category);
        });
    });

    return;
}

#========================================================================================
# function     _sortSub
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $sub = _sortSub('byStockLevel', 'EUR');
#               my @SortedMap = sort $sub @UnsortedMap
#----------------------------------------------------------------------------------------
# description  returns a anonymous sub to sort product map arrays
#
#----------------------------------------------------------------------------------------
# input        $sortName  | sort criteria, i.e. alphabetize, byPriceASC,
#               byPriceDESC, byNew, byStockLevel, byProductAlias | string
# input        $Currency  | the currency id, i.e. 'EUR' | string
#----------------------------------------------------------------------------------------
# return       sub  | sort function | sub
#========================================================================================
sub _sortSub {
    my ( $sortName, $Currency ) = @_;

    if ( $sortName eq 'alphabetize' || !$sortName ) {
        return sub {
            return lc($a->get('Product')->get('NameOrAlias')) cmp lc($b->get('Product')->get('NameOrAlias'));
        }
    }

    elsif ( $sortName eq 'byProductAlias' ) {
        return sub {
            return lc($a->get('Product')->alias) cmp lc($b->get('Product')->alias);
        }
    }

    elsif ( $sortName eq 'byNew' ) {
        return sub {
            my $IsNewA = $a->get('Product')->get('DefaultSubProduct') ? $a->get('Product')->get('DefaultSubProduct')->get('IsNew') : $a->get('Product')->get('IsNew');
            my $IsNewB = $b->get('Product')->get('DefaultSubProduct') ? $b->get('Product')->get('DefaultSubProduct')->get('IsNew') : $b->get('Product')->get('IsNew');
            if    ( $IsNewA && !$IsNewB) { return -1 } #a newer than b
            elsif (!$IsNewA &&  $IsNewB) { return +1 } #a older than b
            else                         { return  0 } #else equal
        }
    }

    elsif ( $sortName eq 'byStockLevel' ) {
        return sub {
            my $StockA = $a->get('Product')->get('StockLevel');
            my $StockB = $b->get('Product')->get('StockLevel');
            $StockA = 0 unless defined $StockA;
            $StockB = 0 unless defined $StockB;
            return $StockB <=> $StockA;
        }
    }

    elsif ( $sortName eq 'byPriceASC' ) {
        return sub {
            my $PriceA = $a->get('Product')->getListPrice($Currency);
            my $PriceB = $b->get('Product')->getListPrice($Currency);
            $PriceA = 0 unless defined $PriceA;
            $PriceB = 0 unless defined $PriceB;
            return $PriceA <=> $PriceB;
        }
    }

    elsif ( $sortName eq 'byPriceDESC' ) {
        return sub {
            my $PriceA = $a->get('Product')->getListPrice($Currency);
            my $PriceB = $b->get('Product')->getListPrice($Currency);
            $PriceA = 0 unless defined $PriceA;
            $PriceB = 0 unless defined $PriceB;
            return $PriceB <=> $PriceA;
        }
    }

    else {
        Error('SortCriteriaNotFound', { sortName => $sortName });
    }

}

#========================================================================================
# function     SearchTerms
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $ahSearchTerms = SearchTerms($SiteID, $Number);
#               my $ahSearchTerms = SearchTerms($SiteID, $Number, $OrderBy, $OrderDirection);
#----------------------------------------------------------------------------------------
# description  Looks for a limited number of search terms of a certain site and returns them in sorted order.
#----------------------------------------------------------------------------------------
# input        $SiteID | SiteID| integer
# input        $Number | max. number of resulting elements | integer
# input        $OrderBy | order by column name (optional, default="CountRequests") | string
# input        $OrderDirection | sort result ascending or descending ["ASC"|"DESC"] (optional, default="DESC") | string
# return       $ahSearchTerms | sorted list of search terms | ref.array.hash
#========================================================================================
sub SearchTerms($$;$$) {
    my ( $SiteID, $Number, $OrderBy, $OrderDirection ) = @_;

    TestInteger('SiteID', $SiteID);
    TestInteger('Number', $Number, 0);

    if($OrderBy){
        TestString('OrderBy', $OrderBy);
    } else {
        $OrderBy = 'CountRequests';
    }

    if($OrderDirection){
        TestString('OrderDirection', $OrderDirection, 3, 4);
    } else {
       $OrderDirection = 'DESC';
    }

    my @hSearchTerms;
    foreach my $aItem (@{ DAL_GetInfoSearchTermsBySiteID($SiteID, $Number, $OrderBy, $OrderDirection) }){
        push @hSearchTerms, {
                'SearchTermID'  => $aItem->[0],
                'Term'          => $aItem->[1],
                'CountResults'  => $aItem->[2],
                'CountRequests' => $aItem->[3],
            };
    }
    return \@hSearchTerms;

}

1;
