use strict;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main() {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    my $FakeRequester = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', 'Shop loaded' );
    $Shop->set({
        'DomainName'        => 'example.com',
        'HasSSLCertificate' => 1
    });

    my $RootCategory = $Shop->get('Categories');

    my $Cat = $RootCategory->child('MinimalCategory');

    my $Products = $Shop->child('Products');
    my $Product = $Products->child('MinimalProduct1');

    my $URLsToTest = {
        $Shop->get('WebUrlSSL') => 'https://example.com/',
        $Cat->get('WebUrlSSL') => 'https://example.com/MINC',
        $Product->get('WebUrlSSL') => 'https://example.com/MPN1',
        $Product->get('WebUrlSSL') . '&ViewAction=ViewProductRating' => 'https://example.com/epages/Store.sf/en_GB/?ObjectPath=/Test/TestShop/Products/MinimalProduct1&ViewAction=ViewProductRating'
    };

    foreach my $SiteURL (keys %{$URLsToTest}) {
        ok( languageLink($FakeRequester, $SiteURL) == 1, "$SiteURL: check language link" );
        my $Can = canonicalTag($FakeRequester, $SiteURL);
        # html encode
        my $URLToCompare = $URLsToTest->{$SiteURL};
        $URLToCompare =~ s/&/&amp;/;
        ok(  $Can eq $URLToCompare, "$SiteURL: check canonical tag." );
    }

    my $RegistrationPage = $Shop->get('WebUrlSSL') . '?ViewAction=ViewRegistration';
    ok( languageLink($FakeRequester, $RegistrationPage) == 0, "$RegistrationPage: check language link" );
    ok( canonicalTag($FakeRequester, $RegistrationPage) eq '', "$RegistrationPage: check canonical tag." );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
    done_testing();
}

sub languageLink() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my $ContainsLink = ($SiteContent =~ /<link rel="alternate" hreflang=".*?" href=".*?" >/);
    return $ContainsLink;
}

sub canonicalTag() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my ($Href) = ($SiteContent =~ /<link rel="canonical" href="(.*?)" \/>/);
    return $Href;
}

sub Init() {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub Cleanup() {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);