#========================================================================================
# package      DE_EPAGES::Customer::API::CSV::CustomerExportImport;
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements the high level interface for import and export of
#               customer data to CVS files.
#========================================================================================
package DE_EPAGES::Customer::API::CSV::CustomerExportImport;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectExportImport;

use strict;
use DE_EPAGES::Customer::API::Constants;
use DE_EPAGES::Customer::API::CSV::CustomerExportDriver;
use DE_EPAGES::Customer::API::CSV::CustomerImportHandler;
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Object::API::Table::Locale qw(GetInfoLocale);
use DE_EPAGES::Core::API::Warning qw(Warning);

#========================================================================================
# function     warningCodes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aWarningCodes = $Driver->warningCodes();
#----------------------------------------------------------------------------------------
# description  Returns the warning codes for csv export.
#----------------------------------------------------------------------------------------
# return       $aWarningCodes | warning codes | array.string
#========================================================================================
sub warningCodes {
    my $self = shift;
    return [ @{$self->SUPER::warningCodes}, @CSV_WARNING_CODES_CUSTOMER ];
}

#========================================================================================
# function     exportDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->exportDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub exportDriver {
    my $self = shift;
    return 'DE_EPAGES::Customer::API::CSV::CustomerExportDriver';
}

#========================================================================================
# function     importDriver
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->importDriver();
#----------------------------------------------------------------------------------------
# description  Return driver package.
#----------------------------------------------------------------------------------------
# return       $DriverPackage  | package to driver | string
#========================================================================================
sub importDriver {
    my $self = shift;
    return 'DE_EPAGES::Customer::API::CSV::CustomerImportHandler';
}

#========================================================================================
# function     getExportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Driver->getExportObjects($Shop, $aObjects);
#----------------------------------------------------------------------------------------
# description  Returns list of objects to export.
#----------------------------------------------------------------------------------------
# input        $Shop           | alias of shop          | object
# input        $aObjects       | list of exportable objects | ref.array.integer
# return       $aExportObjects | edited list of exportable objects | ref.array
#========================================================================================
sub getExportObjects {
    my $self = shift;
    my ($Shop, $aObjects) = @_;

    if (not defined $aObjects){
        my @Objects;
        push @Objects, @{$Shop->folder('Customers')->findChildren} if defined $Shop->child('Customers');
        my $ItemCount = scalar(@Objects);
        Warning('NO_CUSTOMERITEMS') unless $ItemCount;
        return \@Objects;
    } else{
        my $ItemCount = scalar(@$aObjects);
        Warning('NO_CUSTOMERITEMS') unless $ItemCount;
    }
    return $aObjects;
}

#========================================================================================
# function     csvAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $aCSVAttributes = $Driver->csvAttributes($Shop, $LocaleID, $aLanguageCode2s);
#----------------------------------------------------------------------------------------
# description  Return list of exportable csv attributes.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $LocaleID | locale identifier (e.g. en_GB) | string
# input        $aLanguageCode2s | array of languages identified by code2 | ref.array.string
# return       $aAttributes | list of attributes | ref.array.hash
#========================================================================================
sub csvAttributes {
    my $self = shift;
    my ($Shop, $LocaleID, $aLanguageCode2s) = @_;

    my $hLocale = GetInfoLocale($LocaleID);
    my $ahLanguages = $self->getLanguages($Shop, $aLanguageCode2s);

    # default Customer attributes
    my $CustomerClass = LoadClassByAlias('Customer');
    my @hCustomerAttributes = map {
        {
            'AttributeClass' => $CustomerClass,
            'Attribute' => $_
        }
    }
    sort {$a->get('Position') <=> $b->get('Position')}
    @{$CustomerClass ->attributes};

    # user defined Customer attributes
    my $ShopCustomerClass = $Shop->get('CustomerClass');
    push @hCustomerAttributes, map {
        {
            'AttributeClass' => $ShopCustomerClass,
            'Attribute' => $_
        }
    }
    sort {$a->get('Position') <=> $b->get('Position')}
    @{$ShopCustomerClass->directAttributes};

    # Address and User attributes
    my %SpecialClasses = map { $_=> LoadClassByAlias($_) } qw( Address User );
    foreach my $Class ( values %SpecialClasses ) {
        push @hCustomerAttributes, map {
            {
                'AttributeClass' => $Class,
                'Attribute' => $_
            }
        }
        sort {$a->get('Position') <=> $b->get('Position')}
        @{$Class->attributes};
    }

    my @CSVAttributes;
    my @ShippingAttributes;
    my $hAddressSetVisibleFields = {};
    $hAddressSetVisibleFields = $Shop->get([qw(BillingAddressSetVisibleFields ShippingAddressSetVisibleFields)]) 
                                if $Shop->get('IsUnityRegistered');

    foreach my $hAttribute ( @hCustomerAttributes ) {
        next unless IsAttributeForCSV($hAttribute, $Shop, $hAddressSetVisibleFields);
        my $Attribute = $hAttribute->{'Attribute'};
        my %Info;
        my $Alias = $Attribute->alias;
        my $AliasShipping = undef;

        my $AttributeClass = $hAttribute->{'AttributeClass'};
        my $ClassAlias = $AttributeClass->alias;
        if( exists $SpecialClasses{$ClassAlias} ) {
            $Info{'ClassAlias'} = $ClassAlias;
            if ($ClassAlias eq 'Address') {
                $AliasShipping = 'ShippingAddress.' . $Alias;
                $Alias = 'BillingAddress.' . $Alias;
            } elsif ($ClassAlias eq 'User') {
                $Alias = 'User.' . $Alias;
            }
        } else {
            $Info{'ClassAlias'} = 'CustomerClass';
        }
        $Info{'Alias'} = $Alias;

        push @CSVAttributes, @{ $self->getAttributeInfo($Attribute, $hLocale->{'LanguageID'}, $ahLanguages, \%Info)};

        if ($AliasShipping){
            $Info{'Alias'} = $AliasShipping;
            push @ShippingAttributes, @{ $self->getAttributeInfo($Attribute, $hLocale->{'LanguageID'}, $ahLanguages, \%Info)};
        }

    }

    if (@ShippingAttributes){
        push @CSVAttributes, @ShippingAttributes;
    }
    return \@CSVAttributes;
}

sub IsAttributeForCSV {
    my ($hAttribute, $Shop, $hAddressSetVisibleFields) = @_;

    my $Attribute = $hAttribute->{'Attribute'};
    my $Alias = $Attribute->alias;
    my $AttributeAlias = $Alias;
    my $AttributeClassAlias = $hAttribute->{'AttributeClass'}->alias;
    if ($AttributeClassAlias eq 'Address') {
        $Alias = 'BillingAddress.' . $Alias;
    } elsif ($AttributeClassAlias eq 'User') {
        $Alias = 'User.' . $Alias;
    }

    # if we have unity we only export fields from the ShippingAddressSetVisibleFields and BillingAddressSetVisibleFields
    # and we ignore attributes we can not hanlde in unity e.g. some user attributes (see Constants.pm)
    if ($Shop->get('IsUnityRegistered')){

        # we have no information if the address is a shipping or a billing address, 
        # so we add both informations from the AddressSetVisibleFields
        my  @AddressFields = split(/,/,$hAddressSetVisibleFields->{'BillingAddressSetVisibleFields'}.','.
                                       $hAddressSetVisibleFields->{'ShippingAddressSetVisibleFields'});

        # we export address fields from this list
        if($AttributeClassAlias eq 'Address'){
            return 0 if (not ElementOf($AttributeAlias, @AddressFields) and not ElementOf($Alias, @{$CSV_SPECIAL_CUSTOMERITEMS_UNITY->{1}}));
        }
               
        # exclude some fields for unity
        return 0 if ElementOf($Alias, @{$CSV_SPECIAL_CUSTOMERITEMS_UNITY->{0}});
   
    }

    return 0 if ElementOf($Alias, @{$CSV_SPECIAL_CUSTOMERITEMS->{0}});
    return 0 if $Attribute->get('IsArray') and not ElementOf($Alias, @{$CSV_SPECIAL_CUSTOMERITEMS->{1}}) ;
    # not this attribute if it is not exportable and not needed
    return 0 unless (  $Attribute->get('IsExportable') or ElementOf($Alias, @{$CSV_SPECIAL_CUSTOMERITEMS->{1}}) );
    return 1;
}

1;
