#========================================================================================
# package      DE_EPAGES::Customer::API::CSV::CustomerImportHandler;
# base         DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler
# state        public
#----------------------------------------------------------------------------------------
# description  This module implements a handler to export customer data to a CVS file.
#               See <package DE_EPAGES::Customer::API::CSV::CustomerExportImport> for
#               usage examples.
#========================================================================================
package DE_EPAGES::Customer::API::CSV::CustomerImportHandler;
use base DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler;

use strict;
use DateTime;
use DE_EPAGES::Address::API::Address qw( CheckZipcode );
use DE_EPAGES::CSVExportImport::API::Tools qw ( ParseDateTime ParseValue );
use DE_EPAGES::Core::API::Error qw ( HandleError GetError ExistsError Error);
use DE_EPAGES::Core::API::TestParameter qw ( TestEMailAddress );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Customer::API::Shop qw ( GetCustomerForUser );
use DE_EPAGES::Customer::API::Table::CustomerUser qw ( InsertCustomerUser );
use DE_EPAGES::MailType::API::NewPassword qw( SendNewPasswordToken );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias Transaction );
use DE_EPAGES::Object::API::Table::Country qw( ExistsCountryByCode2 GetPKeyCountriesByCode2 ExistsCountryByCode3 GetPKeyCountriesByCode3 );
use DE_EPAGES::Presentation::API::User qw ( CreateUser GetUserFolder GetUsersByEMail);
use DE_EPAGES::Shop::API::Address qw ( CheckStringVATID CheckVATID UpdateVATID );
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler = DE_EPAGES::Customer::API::CSV::CustomerImportHandler->new(%options);
#----------------------------------------------------------------------------------------
# description  create a new csv import handler
#----------------------------------------------------------------------------------------
# input        %options |  importer options | hash
# return       $Handler |  importer | object
#========================================================================================
sub new {
    my $class = shift;
    my %params = @_;

    my $self = $class->SUPER::new(%params);

    $self->deactivateEmails($params{'deactivateEmails'});

    return $self;
}

#========================================================================================
# function     deactivateEmails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->deactivateEmails();
#----------------------------------------------------------------------------------------
# description  Setter and getter for the option to deativate emails when creating a customer
#----------------------------------------------------------------------------------------
# input        $deactivateEmails | deactivate the password email or not | boolean
#----------------------------------------------------------------------------------------
# return       $hAttributeValues | 0 - send an email, 1 - send no email | boolean
#========================================================================================
sub deactivateEmails {
    my $self = shift;
    my ($deactivateEmails) = @_;

    $self->{'deactivateEmails'} = $deactivateEmails if defined $deactivateEmails;

    return $self->{'deactivateEmails'};
}

#ERRORDEFINITION
sub ErrorDefinition {
    my $hSuperErrorDefinition = __PACKAGE__->SUPER::ErrorDefinition();
    return {
        %$hSuperErrorDefinition,
        'VOID_FIELD' => 'line #Line, column #ID: #Error',
    }
}
#ERRORDEFINITION

#========================================================================================
# function     analyzeAttributeLine
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributeValues = $self->analyzeAttributeLine;
#----------------------------------------------------------------------------------------
# description  Analyzes the current line in the CSV file and returns the attribute
#               values that should be imported.
#----------------------------------------------------------------------------------------
# return       $hAttributeValues | attribute name => value hash | ref.hash
#========================================================================================
sub analyzeAttributeLine {
    my $self = shift;

    my $DoNotImport      = 0;
    my $aItems            = $self->topID('Item');
    my $ImportAttributes  = $self->topID('ImportAttributes');
    my $Shop = $self->object;
    my $Folder = $Shop->folder('Customers');

    # alias is mandatory
    my $CustomerAlias = $aItems->[$ImportAttributes->{'AliasColumn'}];
    $CustomerAlias =~ s/\s+$//; $CustomerAlias =~ s/^\s+//;
    if ($CustomerAlias eq '') {
        Warning( 'CUSTOMER_EMPTY_ALIAS', { Line => $self->lineNumber, ID => $ImportAttributes->{'AliasColumn'}+1} );
        $DoNotImport = 1;
        return;
    }

    my $hFormat = $self->locale->get( [qw(LocaleID DecimalSep ThousandSep)] );

    my %CustomerAttributes = (
        Class => $Shop->get('CustomerClass'),
        Parent => $Folder,
    );
    my %UserAttributes;
    my %AddressAttributes;
    my %LocalizedAttributes;
    for (my $i=0; $i<scalar(@$aItems) ;$i++) {
        my $hAttribute = $ImportAttributes->{$i};

        # ignore unknown attribute columns (see also DE_EPAGES::ShopCSVExportImport::API::CSV::ShopObjectImportHandler::getUsedAttribute)
        next if exists $hAttribute->{'IGNORE_COLUMN'};

        my $Value = $aItems->[$i];
        my $Attribute = $hAttribute->{'Attribute'};
        my $Type = $hAttribute->{'Type'};
        my $AttributeAlias = $hAttribute->{'Alias'};
        my $ClassAlias = $hAttribute->{'ClassAlias'};

        # strip leading and trailing spaces
        if( defined $Value ) {
            $Value =~ s/\s+$//; $Value =~ s/^\s+//;
            $Value = undef if $Value eq '';
        }

        if ($ClassAlias eq 'User') {
            $UserAttributes{$AttributeAlias} = $Value;
        }
        elsif ($ClassAlias eq 'Address') {
            $AddressAttributes{$AttributeAlias} = $Value;
        }
        else {
            if ($hAttribute->{'IsObject'}) {
                if ($Type eq 'CustomerGroup') {
                    $CustomerAttributes{'CustomerGroup'} = $self->_getCustomerGroup( $Value, $CustomerAlias )
                        if defined $Value;
                }
                elsif ($Type eq 'TaxArea') {
                    $CustomerAttributes{'TaxArea'} = $self->_getTaxArea( $Value, $CustomerAlias )
                        if defined $Value;
                }
                elsif ($Type =~ /^PreDef(Localized)?String$/) {
                    $self->_setPreDefValue( $Attribute, $Value, $CustomerAlias, \%CustomerAttributes );
                }
                else {
                    $CustomerAttributes{$AttributeAlias} = undef;
                }
            }
            else { # no Object
                # only parse the value if this is no date/time type
                # date/time types are parsed in _getDate or _getTime function (EPG-36966)
                unless( $Type =~ /Date$/ or $Type =~ /DateTime$/ or $Type =~ /^Time$/) {
                    if( defined $Value) {
                        # note: ParseValue sets $Value to undef if it was ''
                        eval { ParseValue( $AttributeAlias, $Type, \$Value, $hFormat ) };
                        if ( ExistsError() ) {
                            Warning( 'VOID_FIELD', { Attribute => $AttributeAlias, Line => $self->lineNumber, ID => $i+1, Error => GetError()->shortMessage } );
                            $DoNotImport = 1;
                            next;
                        }
                    }
                }

                if ($Type =~ /^Localized/) {
                    $LocalizedAttributes{$hAttribute->{'LanguageID'}}->{$AttributeAlias} = $Value;
                }
                elsif ($Type =~ /Date$/ or $Type =~ /DateTime$/) {
                    $CustomerAttributes{$AttributeAlias} = $self->_getDate($AttributeAlias, $Type, $Value, $CustomerAlias);
                }
                elsif ($Type =~ /^Time$/) {
                    $CustomerAttributes{$AttributeAlias} = $self->_getTime( $AttributeAlias, $Value, $CustomerAlias );
                }
                else {
                    $CustomerAttributes{$AttributeAlias} = $Value;
                }
            }
        }
    }

    #error if existing user login given BEFORE insert any data
    if ( !$self->_checkUserAlias($UserAttributes{'User.Alias'}, $CustomerAlias ) ) {
        return undef;
    }

    #error if existing user email given BEFORE insert any data
    if ( !$self->_checkUserEMail($UserAttributes{'User.Alias'},$CustomerAlias, $UserAttributes{'User.EMail'}) ) {
        return undef;
    }

    if ( !$self->_checkVATID($AddressAttributes{'BillingAddress.VATID'}, $AddressAttributes{'BillingAddress.Country'}) ) {
        $DoNotImport = 1;
    }
    if ( !$self->_checkVATID($AddressAttributes{'ShippingAddress.VATID'}, $AddressAttributes{'ShippingAddress.Country'}) ) {
        $DoNotImport = 1;
    }

    if ( !$self->_checkZipcode($AddressAttributes{'BillingAddress.Zipcode'}, $AddressAttributes{'BillingAddress.Country'}) ) {
        $DoNotImport = 1;
    }

    if ( !$self->_checkGender($AddressAttributes{'BillingAddress.Gender'}, $CustomerAlias)
      || !$self->_checkGender($AddressAttributes{'ShippingAddress.Gender'}, $CustomerAlias) ) {
        $DoNotImport = 1;
    }

    if ( !$self->_checkAddressAttributes(\%AddressAttributes, $CustomerAlias) ) {
        $DoNotImport = 1;
    }

    return {
        DoNotImport => $DoNotImport,
        Object => $Folder->existsChild($CustomerAlias) ? $Folder->child($CustomerAlias) : undef,
        DirectAttributes => \%CustomerAttributes,
        UserAttributes => \%UserAttributes,
        AddressAttributes => \%AddressAttributes,
        LocalizedAttributes => \%LocalizedAttributes,
    };
}

#========================================================================================
# function     _getCustomerGroup
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $TaxArea = $self->_getCustomerGroup( $GroupAlias, $CustomerAlias );
#----------------------------------------------------------------------------------------
# description  return customer group object by given Alias,
#               warn if customer group with this Alias not exists
#               return default customer group if no Alias given or group not exists
#----------------------------------------------------------------------------------------
# input        $GroupAlias | alias of customer group | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $CustomerGroup | customer group object | object
#========================================================================================
sub _getCustomerGroup {
    my $self = shift;
    my ( $GroupAlias, $CustomerAlias ) = @_;
    return undef unless defined $GroupAlias;

    my $Shop = $self->object;
    my @Groups = @{$Shop->get('CustomerGroups')};
    my ($Group, $DefaultGroup);
    foreach my $ShopGroup (@Groups) {
        $Group = $ShopGroup if $GroupAlias eq $ShopGroup->alias;
        $DefaultGroup = $ShopGroup if $ShopGroup->get('IsNewMemberDefault');
    }
    return $Group if $Group;

    #log warnin if no CustomerGroup by given CustomerAlias
    Warning( 'CUSTOMERGOUP_DOESNT_EXIST', {
        'Line' => $self->lineNumber,
        'CustomerGroup' => $GroupAlias,
        'CustomerNo' => $CustomerAlias,
    });

    return $DefaultGroup if defined $DefaultGroup;
    return shift @Groups;
}

#========================================================================================
# function     _getTaxArea
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $TaxArea = $self->_getTaxArea( $TaxAreaAlias, $CustomerAlias );
#----------------------------------------------------------------------------------------
# description  return tax area object by given Alias,
#               warn if tax area with this Alias not exists
#               return undef if no Alias given or area not exists
#----------------------------------------------------------------------------------------
# input        $TaxAreaAlias | alias of tax area | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $TaxArea | tax area object | object
#========================================================================================
sub _getTaxArea {
    my $self = shift;
    my ( $TaxAreaAlias, $CustomerAlias ) = @_;
    my $Shop = $self->object;

    my ( $TaxArea ) = grep{ $TaxAreaAlias eq $_->alias } @{$Shop->get('TaxMatrix')->get('TaxAreas')};
    return $TaxArea if $TaxArea;

    #log warnin if no TaxArea by given TaxAreaAlias
    Warning( 'TAXAREA_DOESNT_EXIST', {
        'Line' => $self->lineNumber,
        'TaxArea' => $TaxAreaAlias,
        'CustomerNo' => $CustomerAlias,
    }) if defined $TaxAreaAlias;

    return undef;
}

#========================================================================================
# function     _getDate
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Date = $self->_getDate( $Attribute, $Type, $Value, $CustomerAlias );
#----------------------------------------------------------------------------------------
# description  parse date or datetime value
#               returns value if format ok
#               returns undef if FORMAT_NOT_DATETIME with a warning
#               pass any other errors
#----------------------------------------------------------------------------------------
# input        $Attribute | date attribute | string
# input        $Type | date or datetime | string
# input        $Value | attribute datetime value | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $Value | attribute datetime value | DateTime
#========================================================================================
sub _getDate {
    my $self = shift;
    my ($Attribute, $Type, $Value, $CustomerAlias) = @_;
    if (defined $Value) {
        eval { ParseDateTime($Attribute, $Type, \$Value, {'LocaleID' => $self->locale->get('LocaleID')}); };

        HandleError(
            OnError => sub {
                my ($Error) = @_;
                if (   $Error->code eq 'FORMAT_NOT_DATETIME'
                    or $Error->code eq 'DATETIME_TOO_LARGE'
                    or $Error->code eq 'DATETIME_TOO_SMALL') {

                    Warning(
                        'FORMAT_NOT_DATETIME', {
                            'Line'       => $self->lineNumber,
                            'CustomerNo' => $CustomerAlias,
                            'Attribute'  => $Attribute,
                            'Value'      => $Value
                        });
                    $Value = undef;
                    return 'FIXED';
                }
            });
    }
    return $Value;
}

#========================================================================================
# function     _getTime
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $Time = $self->_getTime( $Attribute, $Value, $CustomerAlias );
#----------------------------------------------------------------------------------------
# description  parse time value
#               returns value is format ok
#               returns undef is FORMAT_NOT_TIME with a warning
#----------------------------------------------------------------------------------------
# input        $Attribute | time attribute | string
# input        $Value | attribute time value | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $Value | attribute datettime value | DateTime
#========================================================================================
sub _getTime {
    my $self = shift;
    my ($Attribute, $Value, $CustomerAlias) = @_;
    if (defined $Value) {
        my ($hour, $minute) = split /:/, $Value;
        if (defined $hour and defined $minute) {
            return DateTime->new(year => 2000, month => 1, day => 1, hour => $hour, minute => $minute);
        }
        else {
            Warning(
                'FORMAT_NOT_TIME', {
                    'Line'       => $self->lineNumber,
                    'CustomerNo' => $CustomerAlias,
                    'Attribute'  => $Attribute,
                    'Value'      => $Value
                });
            return undef;
        }
    }
    else {
        return undef;
    }
}

#========================================================================================
# function     _setPreDefValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setPreDefValue( $Attribute, $Value, $CustomerAlias, $hAttributeValues );
#----------------------------------------------------------------------------------------
# description  check predefined value and
#               delete value and warn if variation value does not  exist
#               set direct attribute to value if it OK
#----------------------------------------------------------------------------------------
# input        $Attribute | pre defined attribute | DE_EPAGES::Object::API::Object::Attribute
# input        $Value | attribute value | string
# input        $CustomerAlias | alias of customer | string
# input        $hAttributeValues | attribute values | ref.hash
#========================================================================================
sub _setPreDefValue {
    my $self = shift;
    my ( $Attribute, $Value, $CustomerAlias, $hAttributeValues ) = @_;

    my $AttributeAlias = $Attribute->alias;
    if( defined $Value ) {
        my ($PreDefinedValue) = grep { $_->alias eq $Value } @{$Attribute->get('PreDefAttributes')};
        if ( !defined $PreDefinedValue ){
             Warning( 'WRONG_PREDEF', {
                'Line' => $self->lineNumber,
                'Value' => $Value,
                'Attribute' => $AttributeAlias,
                'CustomerNo' => $CustomerAlias,
             });
        }
        else {
             $hAttributeValues->{$AttributeAlias} = $PreDefinedValue;
        }
    }
    else {
        $hAttributeValues->{$AttributeAlias} = undef;
    }
}

#========================================================================================
# function     _checkUserAlias
# state        private
#----------------------------------------------------------------------------------------
# syntax       if ( $self->_checkUserAlias( $UserAlias, $CustomerAlias ) )
#----------------------------------------------------------------------------------------
# description  return false (0) if user exist and is not related to customer
#               return true (1) if no alias given or
#               user still not exists or is related to customer
#----------------------------------------------------------------------------------------
# input        $UserAlias | alias of tax area | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $isUserAliasOK | false if user alias wrong | boolean
#========================================================================================
sub _checkUserAlias {
    my $self = shift;
    my ( $UserAlias, $CustomerAlias ) = @_;
    return 1 unless $UserAlias;
    my $Shop = $self->object;

    my $UserFolder = GetUserFolder($Shop);
    if ( $UserFolder->existsChild($UserAlias) ) {
        my $CSV_User = $UserFolder->child($UserAlias);
        if ( $CSV_User ) {
            my $UserCustomer = GetCustomerForUser($CSV_User,$Shop);
            if ( !defined $UserCustomer || $CustomerAlias ne $UserCustomer->alias ) {
                Warning( 'LOGIN_EXISTS', {
                    'CustomerNo' => $CustomerAlias,
                    'Login' => $UserAlias,
                    'Line' => $self->lineNumber,
                });
                return 0;
            }
        }
    }
    return 1;
}

sub _checkUserEMail {
    my $self = shift;
    my ($UserAlias, $CustomerAlias, $NewEmail) = @_;

    return 1 unless $NewEmail;
    if ( !defined($UserAlias) ) {
        Warning( 'VOID_FIELD', { Attribute => 'User.Alias', Line => $self->lineNumber } );
        return 0;
    }

    my $Shop = $self->object;
    my $UserFolder = GetUserFolder($Shop);
    if ( $UserFolder->existsChild($UserAlias) ) {
        my $User = $UserFolder->child($UserAlias);
        # test if users tries to change the email address and wants to reset to the user alias
        # or simply chnage to an other email address
        my $aUsers = GetUsersByEMail($NewEmail,$Shop);
        if (scalar (@$aUsers) and ($User->get('EMail') ne $NewEmail) ) {
            # test if user tries to set the email address to the alias
            if (not $User->get('Alias') eq $NewEmail ) {
                Warning( 'LOGIN_EXISTS', {
                    'CustomerNo' => $CustomerAlias,
                    'UserAlias' => $UserAlias,
                    'Login' => $NewEmail,
                    'Line' => $self->lineNumber,
                });
                return 0;
            }
        }

    }# end if existsChild

    return 1;

}

#========================================================================================
# function     _checkVATID
# state        private
#----------------------------------------------------------------------------------------
# syntax       if ( $self->_checkVATID( $VATID, $CountryCode2 ) )
#----------------------------------------------------------------------------------------
# description  return false (0) if vatid is invalid
#               return true (1) if vatid is valid
#----------------------------------------------------------------------------------------
# input        $VATID | vat id | string
# input        $CountryCode2 | country code2 | string
#----------------------------------------------------------------------------------------
# return       $isVATIDOK | false if vatid in invalid | boolean
#========================================================================================
sub _checkVATID {
    my $self = shift;
    my ( $VATID, $CountryCode2 ) = @_;
    return 1 unless $VATID;
    return 1 unless $CountryCode2;
    if (!CheckStringVATID($VATID)) {
        Warning( 'VATIDIsNotValid', {
            'VATID' => $VATID,
            'Line' => $self->lineNumber,
        });
        return 0;
    }
    else {
        my $Shop = $self->object;
        my $CountryID = GetPKeyCountriesByCode2($CountryCode2)->[0];
        $VATID = UpdateVATID($VATID, $CountryID, $Shop);
        if (CheckVATID($VATID, $CountryID, $Shop) != 1) {
            Warning( 'VATIDIsNotFromSameCountry', {
                'VATID' => $VATID,
                'CountryID' => $CountryID,
                'Line' => $self->lineNumber,
            });
            return 0;
        }
    }
    return 1;
}

#========================================================================================
# function     _checkZipcode
# state        private
#----------------------------------------------------------------------------------------
# syntax       if ( $self->_checkZipcode( $VATID, $CountryCode2 ) )
#----------------------------------------------------------------------------------------
# description  return false (0) if Zipcode is invalid
#               return true (1) if Zipcode is valid
#----------------------------------------------------------------------------------------
# input        $Zipcode | vat id | string
# input        $CountryCode2 | country code2 | string
#----------------------------------------------------------------------------------------
# return       $isZipcodeOK | false if Zipcode in invalid | boolean
#========================================================================================
sub _checkZipcode {
    my $self = shift;
    my ( $Zipcode, $CountryCode2 ) = @_;
    return 1 unless $Zipcode;
    return 1 unless $CountryCode2;

    my $CountryID = GetPKeyCountriesByCode2($CountryCode2)->[0];

    return 1 if CheckZipcode($Zipcode, $CountryID);

    Warning( 'ZIPCODE_NOT_VALID', {
        'Zipcode' => $Zipcode,
        'Line' => $self->lineNumber,
    });
    return 0;
}

#========================================================================================
# function     _checkGender
# state        private
#----------------------------------------------------------------------------------------
# syntax       $isGenderOK = $self->_checkGender( $GenderID, $CustomerAlias )
#----------------------------------------------------------------------------------------
# description  return false if Gender is invalid
#               return true if Gender is valid
#----------------------------------------------------------------------------------------
# input        $GenderID | gender id | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $isGenderOK | false if $GenderID is invalid | boolean
#========================================================================================
sub _checkGender {
    my $self = shift;
    my ( $GenderID, $CustomerAlias ) = @_;

    return 1 if !defined $GenderID;
    return 1 if $GenderID eq '1' || $GenderID eq '2';

    Warning( 'GENDER_NOT_VALID', {
        'Gender' => $GenderID,
        'Line' => $self->lineNumber,
        'CustomerNo' => $CustomerAlias,
    });
    return 0;
}

#========================================================================================
# function     _checkAddressAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $OK = $self->_checkAddressAttributes( $hAttributeValues, $CustomerAlias )
#----------------------------------------------------------------------------------------
# description  Tests the length of the address attributes.
#----------------------------------------------------------------------------------------
# input        $hAttributeValues | all address attribute values | ref.string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $OK | false if any address attribute has an invalid value | boolean
#========================================================================================
sub _checkAddressAttributes {
    my $self = shift;
    my ( $hAttributeValues, $CustomerAlias ) = @_;

    my $OK = 1;
    my $AddressClass = LoadClassByAlias('Address');
    while( my ($Key, $Value) = each %$hAttributeValues) {
        next unless defined $Value;
        if ($Key =~ /^(?:Billing|Shipping)Address\.(.+)/ ) {
            my $AttributeAlias = $1;
            my $Attribute = $AddressClass->attribute($AttributeAlias);
            next unless $Attribute->type eq 'String';

            my $MaxLength = $Attribute->get('Length');
            next unless defined $MaxLength;

            if (length $Value > $MaxLength) {
                Warning( 'STRING_TOO_LONG', {
                    'Attribute' => $Key,
                    'Line' => $self->lineNumber,
                    'Value' => $Value,
                    'MaxLength' => $MaxLength,
                    'CustomerNo' => $CustomerAlias,
                });
                $OK = 0;
            }
        }
    }

    return $OK;
}

#========================================================================================
# function     _getLocale
# state        private
#----------------------------------------------------------------------------------------
# syntax       $UsedLocaleID = $self->_getLocale( $LocaleID, $CustomerAlias ) )
#----------------------------------------------------------------------------------------
# description  returns the locale id for the user.
#               returns the default shop locale if $LocaleID is not valid for the shop.
#----------------------------------------------------------------------------------------
# input        $LocaleID | locale id | string
# input        $CustomerAlias | alias of customer | string
#----------------------------------------------------------------------------------------
# return       $UsedLocaleID | corrected locale id | string
#========================================================================================
sub _getLocale {
    my $self = shift;
    my ( $LocaleID, $CustomerAlias ) = @_;

    return $LocaleID if $self->object->existsSiteLocale($LocaleID);

    Warning( 'INVALID_USER_LOCALE', {
        'LocaleID' => $LocaleID,
        'Line' => $self->lineNumber,
        'CustomerNo' => $CustomerAlias,
    });
    return $self->object->get('LocaleID');
}

#========================================================================================
# function     baseAttributeClassAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->baseAttributeClassAlias();
#----------------------------------------------------------------------------------------
# description  return Base AttributeClass Alias, here : 'CustomerClass'
#----------------------------------------------------------------------------------------
# return        $BaseAttributeClassAlias | attribute class alias of normal attributes | string
#========================================================================================
sub baseAttributeClassAlias {
    my $self = shift;
    return 'CustomerClass';
}

#========================================================================================
# function     importSpecialData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importSpecialData($Object, $hAttributeValues);
#----------------------------------------------------------------------------------------
# description  possibility to import special object data, called by importObject
#----------------------------------------------------------------------------------------
# input        $Object | object which is imported | object
# input        $hAttributeValues    | attribute values | ref.hash
#========================================================================================
sub importSpecialData {
    my $self = shift;
    my ( $Customer, $hAttributeValues ) = @_;

    # import AddressAttributes
    my %BillingAddressData;
    my %ShippingAddressData;
    my $Shop = $Customer->getSite();

    while( my ($Key, $Value) = each %{$hAttributeValues->{'AddressAttributes'}}) {
        if ($Key =~ /^BillingAddress\.(.+)/ ) {
            $BillingAddressData{$1} = $Value;
        }
        elsif ($Key =~ /^ShippingAddress\.(.+)/ ) {
            $ShippingAddressData{$1} = $Value;
        }
    }
    foreach my $DateTimeAttribute (qw(Birthday VerifiedOn)) {
        if ( exists $BillingAddressData{$DateTimeAttribute} ) {
            if ($BillingAddressData{$DateTimeAttribute}) {
                my $Value = $BillingAddressData{$DateTimeAttribute};
                eval {
                    ParseDateTime($DateTimeAttribute, 'Date', \$Value, {'LocaleID' => $self->locale->get('LocaleID')});
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    if ($Error->code eq 'FORMAT_NOT_DATETIME' or
                        $Error->code eq 'DATETIME_TOO_LARGE'  or
                        $Error->code eq 'DATETIME_TOO_SMALL' ) {
                        Warning('FORMAT_NOT_DATETIME', {
                            'Line' => $self->lineNumber,
                            'CustomerNo' => $Customer->alias,
                            'Attribute' => $DateTimeAttribute,
                            'Value' => $Value
                        });
                        $BillingAddressData{$DateTimeAttribute} = undef;
                    } else {
                        Error($Error);
                    }
                } else {
                    $BillingAddressData{$DateTimeAttribute} = $Value;
                }
            }
            else {
                delete $BillingAddressData{$DateTimeAttribute}
            }
        }
        if ( exists $ShippingAddressData{$DateTimeAttribute} ) {
            if ($ShippingAddressData{$DateTimeAttribute}) {
                my $Value = $ShippingAddressData{$DateTimeAttribute};
                eval {
                    ParseDateTime($DateTimeAttribute, 'Date', \$Value, {'LocaleID' => $self->locale->get('LocaleID')});
                };
                if (ExistsError()) {
                    my $Error = GetError();
                    if ($Error->code eq 'FORMAT_NOT_DATETIME' or
                        $Error->code eq 'DATETIME_TOO_LARGE'  or
                        $Error->code eq 'DATETIME_TOO_SMALL' ) {
                        Warning('FORMAT_NOT_DATETIME', {
                            'Line' => $self->lineNumber,
                            'CustomerNo' => $Customer->alias,
                            'Attribute' => $DateTimeAttribute,
                            'Value' => $Value
                        });
                        $ShippingAddressData{$DateTimeAttribute} = undef;
                    } else {
                        Error($Error);
                    }
                } else {
                    $ShippingAddressData{$DateTimeAttribute} = $Value;
                }
            } else {
                delete $ShippingAddressData{$DateTimeAttribute}
            }
        }
    }

    $self->addCountryIDToBillingAddress($Customer, \%BillingAddressData);
    $self->saveBillingAddress($Customer, \%BillingAddressData);

    # import UserAttributes
    my %UserData;
    foreach my $EntryKey (keys %{$hAttributeValues->{'UserAttributes'}}) {
        my $Entry = $hAttributeValues->{'UserAttributes'}{$EntryKey};
        $EntryKey =~ s/^User\.//;
        $UserData{$EntryKey} = $Entry if defined $Entry;
    }

    # import user only if Alias is given
    if( defined $UserData{'Alias'} ) {
        my $UserFolder = GetUserFolder($Shop);
        if ( !$UserFolder->existsChild( $UserData{'Alias'} ) ) {
            # create new user for customer
            $UserData{'Password'} = 'epages';
            $self->_insertUser( $Customer, \%UserData );
        }
        else {
            # update user
            my $User = $UserFolder->child( $UserData{'Alias'} );
            $self->_updateUser( $Customer, $User, \%UserData, $Customer->getSite()->get('IsUnityRegistered') );
        }
    }

    # for unity we set the  customers billing address data also to the users billing address
    # beacuse we have no extra user handling and can not sync data
    if ($Customer->getSite()->get('IsUnityRegistered') and scalar(@{$Customer->get('Users')})) {

        my $User          = $Customer->get('Users')->[0];
        my $AddressFolder = $User->folder('Addresses');
        my $NewAlias      = $AddressFolder->nextAlias('BillingAddress %s');
        my $CustomerBillingAddress = $Customer->get('BillingAddress');
        my $UserBillingAddress     = $User->get('BillingAddress');

        # clone the billing address and use this for the users BillingAddress
        my $NewBillingAddress = $CustomerBillingAddress->clone({ 'Alias' => $NewAlias, 'Parent' => $AddressFolder });
        $User->set({'BillingAddress' => $NewBillingAddress});
        # delete old address
        $UserBillingAddress->delete;
    }

    $self->_setShipping( $Customer, \%ShippingAddressData );
    return;
}

#========================================================================================
# function     saveBillingAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->saveBillingAddress($Customer, \%BillingAddressData);
#----------------------------------------------------------------------------------------
# description  saves the billing address data on the customer
#----------------------------------------------------------------------------------------
# input        $Customer               | customer which should be imported | object
# input        $hBillingAddressData    | attribute values | ref.hash
#----------------------------------------------------------------------------------------
# return       $Customer | altered customer | object
#========================================================================================
sub saveBillingAddress {
    my $self = shift;
    my ($Customer, $hBillingAddressData) = @_;

    Transaction( sub {
        my $BillingAddress = $Customer->get('BillingAddress');
        if (defined $BillingAddress) {
            $BillingAddress->set($hBillingAddressData);
        } else {
            my $AddressFolder = $Customer->folder('Addresses');
            my $CustomerBillingAddress = $AddressFolder->insertChild('Address', {
                    %{$hBillingAddressData},
                    'Alias' => $AddressFolder->nextAlias('BillingAddress %s')
                });
            $Customer->set({'BillingAddress' => $CustomerBillingAddress});
        }
    });

    return $Customer;
}

#========================================================================================
# function     addCountryIDToBillingAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->addCountryIDToBillingAddress($Customer, \%BillingAddressData);
#----------------------------------------------------------------------------------------
# description  adds the coutry id to the billing address data hash
#----------------------------------------------------------------------------------------
# input        $Customer               | customer which should be imported | object
# input        $hBillingAddressData    | attribute values | ref.hash
#----------------------------------------------------------------------------------------
# return       $hBillingAddressData    | attribute values | ref.hash
#========================================================================================
sub addCountryIDToBillingAddress {
    my $self = shift;
    my ($Customer, $hBillingAddressData) = @_;

    my $CountryCodeFromCSV = delete $hBillingAddressData->{'Country'};

    if (defined $CountryCodeFromCSV) {
        $hBillingAddressData->{'CountryID'} = $self->getCountryIDFromCSV($Customer,$CountryCodeFromCSV);
    }
    else {
        $hBillingAddressData->{'CountryID'} = $self->getCountryIDFromDatabase($Customer);
    }

    return $hBillingAddressData;
}

#========================================================================================
# function     getCountryIDFromDatabase
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getCountryIDFromDatabase($Customer);
#----------------------------------------------------------------------------------------
# description  gets the country id from the database
#----------------------------------------------------------------------------------------
# input        $Customer              | customer which should be imported | object
#----------------------------------------------------------------------------------------
# return       $CoutryID    | country id | scalar
#========================================================================================
sub getCountryIDFromDatabase {
    my $self = shift;
    my ($Customer) = @_;

    my $OriginalBillingAddress = $Customer->get('BillingAddress');

    if (defined $OriginalBillingAddress && defined $OriginalBillingAddress->get('CountryID')) {
        return $OriginalBillingAddress->get('CountryID');
    }
    else {
        return $self->getDefaultShopCountryID($Customer->getSite()->get('ShopCountries'));
    }
}

#========================================================================================
# function     getCountryIDFromCSV
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getCountryIDFromCSV($Customer, $CountryCodeFromCSV);
#----------------------------------------------------------------------------------------
# description  gets the country id from the csv value
#----------------------------------------------------------------------------------------
# input        $Customer              | customer which should be imported | object
# input        $CountryCodeFromCSV    | csv vlaue of the field country | scalar
#----------------------------------------------------------------------------------------
# return       $CoutryID    | country id | scalar
#========================================================================================
sub getCountryIDFromCSV {
    my $self = shift;
    my ($Customer,$CountryCodeFromCSV) = @_;

    if (ExistsCountryByCode2($CountryCodeFromCSV)) {
        return GetPKeyCountriesByCode2($CountryCodeFromCSV)->[0];
    } elsif (ExistsCountryByCode3($CountryCodeFromCSV)) {
        return GetPKeyCountriesByCode3($CountryCodeFromCSV)->[0];
    } else {
        Warning('UNKNOWN_COUNTRYCODE', {
                'Line' => $self->lineNumber,
                'CustomerNo' => $Customer->alias,
                'Attribute' => 'BillingAddress.Country',
            });
    }

    return;
}

#========================================================================================
# function     getDefaultShopCountryID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getDefaultShopCountryID(ahShopCountries);
#----------------------------------------------------------------------------------------
# description  gets the default shop country id
#----------------------------------------------------------------------------------------
# input        $ahShopCountries | all shop countries | ref.array
#----------------------------------------------------------------------------------------
# return       $CoutryID    | country id | scalar
#========================================================================================
sub getDefaultShopCountryID {
    my $self = shift;
    my ($ahShopCountries) = @_;

    my ($hCountry) = grep {$_->{'IsDefault'}} @$ahShopCountries;

    return $hCountry->{'CountryID'};
}

#========================================================================================
# function     _setShipping
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setShipping( $hShippingAddressData, $Customer );
#----------------------------------------------------------------------------------------
# description  set shipping address to user of customer
#----------------------------------------------------------------------------------------
# input        $Customer | customer object | object
# input        $hShippingAddressData | shipping address data | ref.hash
#========================================================================================
sub _setShipping {
    my $self = shift;
    my ( $Customer, $hShippingAddressData ) = @_;
    my $Users = $Customer->get('Users');
    my $User = $Users->[0];
    my $ShippingCountryCode = delete $hShippingAddressData->{'Country'};
    if (defined $ShippingCountryCode) {
        if (defined $ShippingCountryCode) {
            if (ExistsCountryByCode2($ShippingCountryCode)) {
                $hShippingAddressData->{'CountryID'} = GetPKeyCountriesByCode2($ShippingCountryCode)->[0];
            } elsif (ExistsCountryByCode3($ShippingCountryCode)) {
                $hShippingAddressData->{'CountryID'} = GetPKeyCountriesByCode3($ShippingCountryCode)->[0];
            } else {
                Warning('UNKNOWN_COUNTRYCODE', {
                    'Line' => $self->lineNumber,
                    'CustomerNo' => $Customer->alias,
                    'Attribute' => 'ShippingAddress.Country',
                });
            }
        }
    }
    else {
        $hShippingAddressData->{'CountryID'} = undef;
    }

    if (defined $User) {
        my $UserShippingAddress = $User->get('ShippingAddress');
        if (grep { defined $_ } values %$hShippingAddressData) {
            if ($UserShippingAddress){
                #update shipping address
                Transaction( sub {
                    $UserShippingAddress->set($hShippingAddressData);
                });
            } else {
                unless (defined $ShippingCountryCode) {
                    my $Shop = $Customer->getSite();
                    my ($hCountry) = grep {$_->{'IsDefault'}} @{$Shop->get('ShopCountries')};
                    $hShippingAddressData->{'CountryID'} = $hCountry->{'CountryID'};
                }
                Transaction( sub {
                    my $AddressFolder = $User->folder('Addresses');
                    $UserShippingAddress = $AddressFolder->insertChild('Address', {
                        %{$hShippingAddressData},
                        'Alias' => $AddressFolder->nextAlias('ShippingAddress %s')
                    });
                    $User->set({'ShippingAddress' => $UserShippingAddress});
                });
            }
        } else {
            #if ShippingAddress is empty it is deleted so that epages conncects it to the BillingAddress
            $UserShippingAddress->delete if $UserShippingAddress;
        }
    } else {
        # You can only import the delivery address for customers that have a user account.
        Warning('NO_SHIPPINGADDRESS_FOR_CUSTOMERS', {
            'Line' => $self->lineNumber,
            'CustomerNo' => $Customer->alias,
            'Attribute' => 'ShippingAddress',
        }) if grep { defined $_ } values %$hShippingAddressData;
    }
}

sub _insertUser {
    my $self = shift;
    my( $Customer, $hUserData ) = @_;

    my $Shop = $Customer->getSite();

    # some defaults for Unity
    if ($Shop->get('IsUnityRegistered') and not $hUserData->{'LocaleID'}){
        $hUserData->{'LocaleID'} = $Shop->get('LocaleID');
    }

    $hUserData->{'LocaleID'} = $self->_getLocale($hUserData->{'LocaleID'}, $Customer->alias);
    $hUserData->{'CurrencyID'} = $Shop->get('CurrencyID') unless defined $hUserData->{'CurrencyID'};

    my $CustomerBillingAddress = $Customer->get('BillingAddress');
    $hUserData->{'EMail'} = $CustomerBillingAddress->get('EMail') unless $hUserData->{'EMail'};
    $hUserData->{'Name'} = $CustomerBillingAddress->get('FullName') unless $hUserData->{'Name'};
    $hUserData->{'Name'} = $Customer->get('Name') unless $hUserData->{'Name'};

    my $User = CreateUser( $Shop, $hUserData );
    my $UserBillingAddress = $CustomerBillingAddress->clone({ 'Parent' => $User->folder('Addresses') });
    $User->set({'BillingAddress' => $UserBillingAddress});
    InsertCustomerUser({
        'UserID' => $User->id,
        'CustomerID' => $Customer->id
    });

    return if $self->deactivateEmails();

    # send password email
    my $MailType = GetShopMailType( $Shop, 'UserByMerchantWelcome', 1 );
    return unless defined $MailType;

    my $Email = $User->get('EMail');
    if (defined $Email) {
        my $Valid = 1;
        eval {
            TestEMailAddress('UserEmail', $Email);
        };
        if (ExistsError()) {
            my $Error = GetError();
            if( $Error->code eq 'FORMAT_NOT_EMAILADDRESS' ) {
                $Valid = 0;
                Warning('FORMAT_NOT_EMAILADDRESS', {
                    'Line' => $self->lineNumber,
                    'Message' => $Customer->alias,
                    'EMail' => $Email
                });
            } else {
                Error($Error);
            }
        }
        if ($Valid) {
            # return if we not have a Shop-From address (no message, because merchant know this)
            my $From = $MailType->from;
            return unless defined $From;
            SendNewPasswordToken($Shop, $User, $MailType) if $MailType;
        } else {
            Warning('NO_RECIPIENTS', {
                'Line' => $self->lineNumber,
                'Message' => $Customer->alias
            });
        }
    }

    return;
}

sub _updateUser {
    my $self = shift;
    my( $Customer, $User, $hUserData, $IgnoreDifferenceForUnity ) = @_;

    if ($hUserData->{'LocaleID'}) {
        $hUserData->{'LocaleID'} = $self->_getLocale($hUserData->{'LocaleID'}, $Customer->alias);
    }

    $User->set( $hUserData );

    my $aRegistrationSet = $Customer->getSite()->get('RegistrationSet');
    my $Difference = 0;
    foreach my $hItem (@$aRegistrationSet) {
        if ($hItem->{'IsVisible'} && $hItem->{'Attribute'}->get('AttributeClass')->alias eq 'Address') {
            my $AttributeAlias = $hItem->{'Attribute'}->alias;
            my $CustomerAttribute = $Customer->get('BillingAddress')->get($AttributeAlias);
            my $UserAttribute = $User->get('BillingAddress')->get($AttributeAlias);
            if (defined $CustomerAttribute and defined $UserAttribute and ($CustomerAttribute ne $UserAttribute)) {
                $Difference = 1 unless $IgnoreDifferenceForUnity;
            }
        }
    }

    my $BillingAddress = $Customer->get('BillingAddress');
    Warning('SHOW_COMPARE_ADDRESSES', {
        'Line' => $self->lineNumber,
        'Customer' => $BillingAddress->get('FullName'),
        'ID' => $Customer->alias,
        'CustomerID' => $Customer->id,
        'UserID' => $User->id
    }) if $Difference;

    return;
}

#========================================================================================
# function     getUsedAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getUsedAttribute($HeaderString, $ColumnIndex);
#----------------------------------------------------------------------------------------
# description  return attribute info, called by analyzeHeaderLine
#               for Unity we ignore UNKNOWN_ATTR error and use the $self->SUPER::getUsedAttribute function
#               for handling UNKNOWN_ATTR errors (we simply ignore unknown attributes)
#               otherwise throws Error 'UNKNOWN_ATTR' if $hHeader->{'AliasString'} isn't a key of $self->attributes
#               call function testHeaderColumn before return $hAttribute
#----------------------------------------------------------------------------------------
# input        $HeaderString | header string | string
# input        $ColumnIndex |  index of current column | integer
# return       $hAttribute | properties of column of CSV file with following keys
#                       <ul>
#                           <li>Attribute  | attribute object | object
#                           <li>Alias  | alias of the attribute | string
#                           <li>AliasString  | alias string of the column e.g Shoe/ShoeWeight/en | string
#                           <li>ClassAlias | alias of the attribute class e.g. Shoe | string
#                           <li>HeaderString  | header string of the column e.g 'Schuh/Gewicht/English [Shoe/ShoeWeight/en]' | string
#                           <li>IsBaseClassAttribute | 0 or 1, set to 1 if attribute is from base class | boolean
#                           <li>IsObject   | 0 or 1, set to 1 if Attribute IsObject | boolean
#                           <li>Type       | Attribute type, e.g. LocalizedString | String
#                           <li>LanguageID | only set if Type =~ /Localized/ | Integer
#                           <li>CurrencyID | only set if Type eq 'Price' | String
#                           <li>TaxModel   | 0 (==net) or 1, only set if Type eq 'Price' | Integer
#                       </ul> | ref.hash
#========================================================================================
sub getUsedAttribute {
    my $self = shift;
    my ( $HeaderString, $ColumnIndex) = @_;

    my $hHeader = $self->headerColumn($HeaderString, $ColumnIndex+1);
    my $hAttributes = $self->attributes;
    my $hAttribute = $hAttributes->{$hHeader->{'AliasString'}};

    # for unity we ignore some columns and use the super function wich fakes this columns
    return $self->SUPER::getUsedAttribute($HeaderString, $ColumnIndex) 
           if $self->object->get('IsUnityRegistered') and not defined $hAttribute;

    Error('UNKNOWN_ATTR',{'ID' => $ColumnIndex+1, 'Column' => $HeaderString, 'FileName' => $self->{'FileName'} }) unless defined $hAttribute;

    my $aHeaderParts = $hHeader->{'HeaderParts'};
    if ($hAttribute->{'IsBaseClassAttribute'} == 1) {
        unshift @$aHeaderParts, $self->baseAttributeClassAlias;
    } else {
        unshift @$aHeaderParts, $hAttribute->{'ClassAlias'};
    }
    $self->testHeaderColumn($hAttribute, $hHeader, $ColumnIndex );
    return $hAttribute;
}

1;
