#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::PackstationShippingAddress
# state        public
# description  Class package to handle shipping label address.
#========================================================================================
package DE_EPAGES::DHLIntraship::API::PackstationShippingAddress;

use strict;
use DateTime;
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::Constants qw ( $GERMAN_LOCALE %PACKSTATION_ADDRESS_LENGTHS );
use DE_EPAGES::DHLIntraship::API::Tools;
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Table::Country qw ( GetPKeyCountryByCode2 );

use constant OTHER_PACKSTATION_VALUE => 'other';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new();
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self   | self   | DE_EPAGES::DHLIntraship::API::PackstationShippingAddress
#========================================================================================
sub new {
    my $class = shift;

    my $self = bless {}, $class;

    return $self;
}

#==================================================================================================================
# function     createPackstationShippingAddress
# state        public
#-----------------------------------------------------------------------------------------------------------------
# syntax       PackstationShippingAddress = PackstationShippingAddress->createPackstationShippingAddress($Servlet);
#-----------------------------------------------------------------------------------------------------------------
# description  It sets the Packstation shipping address
#-----------------------------------------------------------------------------------------------------------------
# input        $Servlet | Servlet | object
#==================================================================================================================
sub createPackstationShippingAddress {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form();
    my $Basket   = $Servlet->object();
    my $User     = $Basket->get('User');

    my $ShippingAddress = $Basket->get('ShippingAddress') // $User->get('BillingAddress');
    
    if (not isValid($ShippingAddress)){
        $Form->executeFormError({
            'Name'       => 'InvalidShippingAddress',
            'Reason'     => 'InvalidShippingAddress',
            'ViewAction' => 'ViewMultiCheckoutShipping'
        });
        return;
    }

    my $MethodName = (caller(0))[3];

    DE_EPAGES::DHLIntraship::API::Tools->LogShopContext(
        $Basket->getSite(),
        $MethodName,
        sub {
            my $PickUpAddress = $ShippingAddress->get('SelectedShippingPickupStation');

            if (not $PickUpAddress) {
                my $hSelectedAddress = $self->_getSelectedAddress($Servlet, $Form);
                my $PackstationAddress  = $self->_createShippingAddress($ShippingAddress, $hSelectedAddress, $User);
                $Basket->set({ 'ShippingAddress' => $PackstationAddress });
            }
        }
    );
    return;
}

#----------------------------------------------------------------------------------------
sub _getSelectedAddress {
    my $self = shift;
    my ($Servlet, $Form) = @_;

    my $ServletParams = $Servlet->params();
    my $PackstationAddress = $ServletParams->{'location'};

    if ($PackstationAddress eq OTHER_PACKSTATION_VALUE){
        return $self->_getOtherPackstation($ServletParams, $Form);
    }

    return $self->_getPackstationAddress($ServletParams, $Form, $PackstationAddress);
}

#----------------------------------------------------------------------------------------
sub _getOtherPackstation {
    my $self = shift;
    my ($ServletParams, $Form) = @_;

    my $hSelectedAddress;
    my $PackstationId = $ServletParams->{'packstationId'};
    my $Zipcode = $ServletParams->{'zipcode'};
    my $City = $ServletParams->{'city'};

    if (isValid($PackstationId) && isValid($Zipcode) && isValid($City)) {
        my $PostNumber = $ServletParams->{'PostNumber'};

        $hSelectedAddress->{'packstationId'} = $PackstationId;
        $hSelectedAddress->{'zipcode'} = $Zipcode;
        $hSelectedAddress->{'city'} = $City;
        $hSelectedAddress->{'postnumber'} = $PostNumber;
    } else {
        $Form->executeFormError({
            'Name'       => 'MissingPackstationData',
            'Reason'     => 'Packstation Daten',
            'ViewAction' => 'ViewMultiCheckoutShipping'
        });
    }

    return $hSelectedAddress;
}

#----------------------------------------------------------------------------------------
sub _getPackstationAddress {
    my $self = shift;
    my ($ServletParams, $Form, $PackstationAddress) = @_;

    $self->_LogShippingFulfillment('DHLIntraship', 'User defined Packstation Address fields', $PackstationAddress);

    my $hSelectedAddress;
    if ( $PackstationAddress =~ /^(\d{3})[|](.*?)[|](.*?)[|](\d{5})[|](.*?)$/ix ){
        $hSelectedAddress->{'packstationId'}    = $1,
        $hSelectedAddress->{'name'}             = $2,
        $hSelectedAddress->{'address'}          = $3,
        $hSelectedAddress->{'zipcode'}          = $4,
        $hSelectedAddress->{'city'}             = $5,
        $hSelectedAddress->{'postnumber'}       = $ServletParams->{'PostNumber'}
    } else {
        $Form->executeFormError({
            'Name'       => 'PackStationAddressInValid',
            'Reason'     => 'Versandadresse',
            'ViewAction' => 'ViewMultiCheckoutShipping'
        });
    }

    return $hSelectedAddress;
}

#----------------------------------------------------------------------------------------
sub _createShippingAddress {
    my $self = shift;
    my ($ShippingAddress, $hSelectedAddress, $User) = @_;

    $self->_LogShippingFulfillment('DHLIntraship', 'Packstation Address fields', $hSelectedAddress);
    my $PickupStation = $self->_LoadObjectByPath('/ShippingPickupStations/DHLIntashipPackstation');
    my $Folder = $User->folder('Addresses');

    my $hNewShippingAddressData = {
        'FirstName'                     => $ShippingAddress->get('FirstName'),
        'LastName'                      => $ShippingAddress->get('LastName'),
        'DisplayName'                   => substr($ShippingAddress->get('FullName'), 0, $PACKSTATION_ADDRESS_LENGTHS{'name'}),
        'AddressExtension'              => $hSelectedAddress->{'postnumber'},
        'Street'                        => 'Packstation ' . $hSelectedAddress->{'packstationId'},
        'Zipcode'                       => substr($hSelectedAddress->{'zipcode'}, 0, $PACKSTATION_ADDRESS_LENGTHS{'zipcode'}),
        'City'                          => substr($hSelectedAddress->{'city'}, 0 , $PACKSTATION_ADDRESS_LENGTHS{'city'}),
        'CountryID'                     => $self->_GetKeyCountryByCode($GERMAN_LOCALE),
        'IsPickupAddress'               => 1,
        'SelectedShippingPickupStation' => $PickupStation,
        'EMail'                         => $ShippingAddress->get('EMail'),
        'DHLPackstationId'              => $hSelectedAddress->{'packstationId'},
        'DHLPostNumber'                 => $hSelectedAddress->{'postnumber'}
    };

    $hNewShippingAddressData->{'Alias'} = $Folder->nextAlias('PickUpAddress%d');

    my $PackstationAddress = $Folder->insertChild(
        'Address',
        $hNewShippingAddressData
    );

    return $PackstationAddress;
}

#----------------------------------------------------------------------------------------
sub _LogShippingFulfillment {
    my $self = shift;
    my ($ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section, $aFilter) = @_;

    LogShippingFulfillment($ShippingFulfillmentTypeAlias, $Name, $DebugStuff, $Section, $aFilter);
    return;
}

#----------------------------------------------------------------------------------------
sub _GetKeyCountryByCode {
    my $self = shift;
    my ($Locale) = @_;
    return GetPKeyCountryByCode2($Locale);
}

#----------------------------------------------------------------------------------------
sub _LoadObjectByPath {
    my $self = shift;
    my ($ClassPath) = @_;
    return LoadObjectByPath($ClassPath);
}

1;
