#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  Useful functions for this cartridge.
#========================================================================================
package DE_EPAGES::DHLIntraship::API::Tools;
use DE_EPAGES::Core::API::Log qw ( GetLog LogContext );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::DHLIntraship::API::Constants qw (
    $GERMANY_COUNTRY_ID
    $MAX_ATTRIBUTE_LENGTH
    $EKP_LENGHT
    $PRODUCTCODE_LENGHT
    %MAPPING_PRODUCTCODE_FULLPRODUCTNAME
    %ADDRESS_LENGTH_INTERVALS
);

use strict;


use constant DHL_NATIONAL_PACKET_ID => '01';
use constant DHL_WORLD_PACKET_ID => '53';

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Class constructor.
#----------------------------------------------------------------------------------------
# return       $self | self | object
#========================================================================================
sub new {
    my $class = shift;
    my $self = bless {}, $class;
    return $self;
}

#========================================================================================
# function     GetCustomerNumber
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetCustomerNumber( $ShippingFulfillmentMethod, $Address );
#----------------------------------------------------------------------------------------
# description  Gets Customer Number.
#----------------------------------------------------------------------------------------
# input        $ShippingFulfillmentMethod | ShippingFulfillmentMethodDHLIntraship | object
# input        $Address | Address | object
# return       $CustomerNumber | CustomerNumber | string
#========================================================================================
sub GetCustomerNumber {
    my $self = shift;
    my ($ShippingFulfillmentMethod, $Address) = @_;

    my $CustomerNumber = '';
    $CustomerNumber .= $ShippingFulfillmentMethod->get('EKP');

    if ($Address->get('CountryID') eq $GERMANY_COUNTRY_ID){
        $CustomerNumber .= DHL_NATIONAL_PACKET_ID;
        $CustomerNumber .= $ShippingFulfillmentMethod->get('TeilnehmerNational');
    } else {
        $CustomerNumber .= DHL_WORLD_PACKET_ID;
        $CustomerNumber .= $ShippingFulfillmentMethod->get('TeilnehmerWeltpaket');
    }

    return $CustomerNumber;
}

#========================================================================================
# function     GetPackageType
# state        public
#----------------------------------------------------------------------------------------
# syntax       GetPackageType( $CustomerNumber );
#----------------------------------------------------------------------------------------
# description  Obtains package type.
#----------------------------------------------------------------------------------------
# input        $CustomerNumber | CustomerNumber | string
# return       $PackageType | Package Type | string
#========================================================================================
sub GetPackageType {
    my $self = shift;
    my ($CustomerNumber) = @_;

    unless (isValid($CustomerNumber)) {
        GetLog->error('Cannot obtain Package Type: Customer Number is either null or empty');
        return;
    }

    my $ProductCodeID = substr($CustomerNumber,$EKP_LENGHT,$PRODUCTCODE_LENGHT);

    return $MAPPING_PRODUCTCODE_FULLPRODUCTNAME{$ProductCodeID};;
}

#========================================================================================
# function     IsDateInThePast
# state        public
#----------------------------------------------------------------------------------------
# syntax       IsDateInThePast( $Date );
#----------------------------------------------------------------------------------------
# description  Finds out whether a date is in the past or not.
#----------------------------------------------------------------------------------------
# input        $Date | Date | DateTime
# return       $IsDateInThePast | Whether a date is in the past or not. | boolean
#========================================================================================
sub IsDateInThePast {
    my $self = shift;
    my ($Date) = @_;

    return 1 if !isValid($Date);

    my $CurrentDate = $self->_GetCurrentDateTime();
    GetLog->info(sprintf('Current date: %s', $CurrentDate));
    return 0 if (DateTime->compare($CurrentDate, $Date) <= 0);

    my $DeltaDays = ($CurrentDate->delta_days($Date))->in_units('days');
    GetLog->debug(sprintf('Delta days between current and given dates: %u', $DeltaDays));
    return ($DeltaDays > 0) ? 1 : 0;
}

#========================================================================================
# function     truncateAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       truncateAttribute( $Attribute, $FieldName );
#----------------------------------------------------------------------------------------
# description  Truncates an attribute to its maximum allowed length.
#----------------------------------------------------------------------------------------
# input        $Attribute          | Attribute to truncate.         | String
# input        $FieldName          | Name of the field to truncate. | String
# return       $TruncatedAttribute | Truncated attribute.           | object
#========================================================================================
sub truncateAttribute {
    my $self = shift;
    my ( $Attribute, $FieldName ) = @_;

    my $TruncatedAttribute = $Attribute;
    my $MaxLength          = $MAX_ATTRIBUTE_LENGTH->{$FieldName};
    my $OriginalLength     = length($Attribute);
    if ( defined($MaxLength) && $MaxLength < $OriginalLength ) {
        $TruncatedAttribute = substr( $Attribute, 0, $MaxLength );
    }

    return $TruncatedAttribute;
}

#========================================================================================
# function     LogShopContext
# state        public
#----------------------------------------------------------------------------------------
# syntax       LogShopContext( $ShopObject, $MethodName, $FunctionPointer );
#----------------------------------------------------------------------------------------
# description  it allows to include shop context into logs for the specified function pointer
#----------------------------------------------------------------------------------------
# input        $ShopObject | an epages object that belongs to a shop | object
# input        $MethodName | name of the method | string
# input        $FunctionPointer | reference pointer to a code | ref.code
#========================================================================================
sub LogShopContext {
    my $self = shift;
    my ($ShopObject, $MethodName, $FunctionPointer) = @_;

    TestObject('ShopObject', $ShopObject, 'DE_EPAGES::Object::API::Object::Object');
    my $Shop = $ShopObject->getSite();

    LogContext(
            {
                'Shop ID' => $Shop->id(),
                'Shop Alias' => $Shop->alias(),
                'EntryPoint' => $MethodName
            },
            $FunctionPointer
    );

    return;
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _GetCurrentDateTime {
    my $self = shift;

    return GetCurrentDateTime();
}

1;
