#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::RemoteCreateShippingOrder
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for createShippingOrder SOAP call (integration tests)
#========================================================================================
package DE_EPAGES::DHLIntraship::t::RemoteCreateShippingOrder;

use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;

use Test::More;

use strict;

use constant ROOTOBJECT => 'Test';
use constant ROOTPATH => '/' . ROOTOBJECT;
use constant SHOPPATH => ROOTPATH . '/TestShopDHLVersenden';
use constant RESPONSE_SHIPMENT_NUMBER_MAX_LENGTH => 39;

sub Init {

    return if ExistsObjectByPath(ROOTPATH) && $ENV{'SKIP_CLEANUP'};

    Cleanup();

    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => ROOTOBJECT } );
    note 'importing shop ' . SHOPPATH;
    ImportObjects('import.xml', ROOTPATH);

    return;
}

sub Cleanup {

    return if $ENV{'SKIP_CLEANUP'};

    my $System = LoadRootObject();
    if( $System->existsChild(ROOTOBJECT) ) {
        note 'deleting shop ' . SHOPPATH;
        $System->child(ROOTOBJECT)->delete();
    }

    return;
}

sub test {
    my $self = shift;

    $self->shipping_label_request_positive_test();
    $self->packstation_shipping_label_request_positive_test();
    $self->shipping_label_request_negative_test();
}

#========================================================================================
# Test methods
#========================================================================================
sub test_xml_import {
    my $subTestName = (caller(0))[3];

    my $testImportOK = 0;

    my $Shop = LoadObjectByPath(SHOPPATH);
    $testImportOK = isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop', "$subTestName - test shop found");

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/DHLIntraship');
    $testImportOK = isa_ok($ShippingFulfillmentMethod,
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship',
            "$subTestName - test ShippingFulfillmentMethodDHLIntraship found")
                        && $testImportOK;

    my $Customer = $Shop->path('Customers/TestCustomer');
    $testImportOK = isa_ok($Customer,
            'DE_EPAGES::Customer::API::Object::Customer',
             "$subTestName - test customer found")
                        && $testImportOK;

    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    $testImportOK = isa_ok($Order,
            'DE_EPAGES::Order::API::Object::CustomerOrder',
             "$subTestName - test order found")
                        && $testImportOK;

    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    $testImportOK = isa_ok($ShippingLabel,
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
             "$subTestName - test ShippingLabelDHLIntraship found")
                        && $testImportOK;

    return $testImportOK;
}

sub shipping_label_request_positive_test {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $Shop = LoadObjectByPath(SHOPPATH);
    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    $ShippingLabel->set( { 'EstimatedDeliveryDate' => GetCurrentDateTime() } );

    my $DHLWSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder',
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser',
        $ShippingLabel->get('IsTestLabel'));
    my $aResponse = $DHLWSClient->createShipmentOrder($ShippingLabel);

    is(@{$aResponse}[0]->{'StatusCode'}, 0, "$subTestName - Shipping label response status code is ok (zero)");

    cmp_ok( length( @{$aResponse}[0]->{'ShipmentNumber'} ), '>', 0,
            "$subTestName - Shipping label response shipment number is > 0");

    cmp_ok( length( @{$aResponse}[0]->{'ShipmentNumber'} ), '<=', RESPONSE_SHIPMENT_NUMBER_MAX_LENGTH,
            "$subTestName - Shipping label response shipment number is <= " . RESPONSE_SHIPMENT_NUMBER_MAX_LENGTH);

}

sub packstation_shipping_label_request_positive_test {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $Shop = LoadObjectByPath(SHOPPATH);
    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5679/ShippingLabels/1234-5679-1');
    $ShippingLabel->set( { 'EstimatedDeliveryDate' => GetCurrentDateTime() } );

    my $DHLWSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder',
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser',
        $ShippingLabel->get('IsTestLabel'));
    my $aResponse = $DHLWSClient->createShipmentOrder($ShippingLabel);

    is(@{$aResponse}[0]->{'StatusCode'}, 0, "$subTestName - Packstation shipping label response status code is ok (zero)");

    cmp_ok( length( @{$aResponse}[0]->{'ShipmentNumber'} ), '>', 0,
            "$subTestName - Packstation shipping label response shipment number is > 0");

    cmp_ok( length( @{$aResponse}[0]->{'ShipmentNumber'} ), '<=', RESPONSE_SHIPMENT_NUMBER_MAX_LENGTH,
            "$subTestName - Packstation shipping label response shipment number is <= " . RESPONSE_SHIPMENT_NUMBER_MAX_LENGTH);

}

sub shipping_label_request_negative_test {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $Shop = LoadObjectByPath(SHOPPATH);
    my $ShippingLabel = $Shop->path('Customers/TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');

    my $Address = $ShippingLabel->get('Address');
    $Address->set( { 'EMail' => '' } );

    my $DHLWSClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Builder',
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelResponse::Parser',
        $ShippingLabel->get('IsTestLabel'));
    my $aResponse = $DHLWSClient->createShipmentOrder($ShippingLabel);
    my $Response = @$aResponse[0];

    cmp_ok( $Response->{'StatusCode'}, '!=', 0,
            "$subTestName - Response is a validation error (no receiver email is available)"
    );
}


RunScript(
    Sub => sub {
        RunOnStore( 'Store' => GetTestSite(), Sub => sub {
            Init();
            unless ( test_xml_import() ) {
                done_testing();
                exit;
            }

            SKIP: {
                skip 'external services tests' if $ENV{'SKIP_EXTERNAL_TESTS'};
                my $self = __PACKAGE__;
                $self->test();
            }
            done_testing();
            Cleanup();
        });
    }
);

1;