#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::PackstationShippingAddress
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for PackstationShippingAddress.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::PackstationShippingAddress;

use base DE_EPAGES::Core::API::Test;
use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::DHLIntraship::API::PackstationShippingAddress;

use Test::More;
use Test::Exception;;
use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef String );
use Test::Mockify::Verify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

sub test {
    my $self = shift;
    $self->_test_create_shipping_address_with_defined_pickup_station();
    $self->_test_create_shipping_address_with_pickup_station();
    $self->_test_create_shipping_address_with_user_pickup_station();
    $self->_test_invalid_shipping_address();
    $self->_test_invalid_user_defined_pickup_station();
    $self->_test_invalid_city_pickup_station();
    done_testing();
}

sub _test_create_shipping_address_with_defined_pickup_station {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $hAddress = {
        'SelectedShippingPickupStation' => 'Defined Dummy PickupStation Object',
        'FirstName'                     => 'FirstName',
        'LastName'                      => 'LastName',
        'FullName'                      => 'FirstName LastName',
        'Street'                        => 'Street1',
        'Zipcode'                       => 'Zipcode',
        'City'                          => 'City1',
        'AddressExtension'              => 'AddressExtension',
        'CountryID'                     => 'CountryID',
        'IsPickupAddress'               => 'IsPickupAddress',
        'EMail'                         => 'EMail',
        'DHLPackstationId'              => 'DHLPackstationId'
    };

    # cloning $hAddress into $hExpectedAddress
    my $hExpectedAddress = {%$hAddress};

    my $UserAddress;
    my $User = $self->_createUser($UserAddress);

    my $BasketAddress = $self->_createShippingAddress($hAddress);
    my $Basket = $self->_createBasket($User, $BasketAddress);

    my $hPackstationAddress = {};
    my $Servlet = $self->_createServlet($Basket, $hPackstationAddress);

    my $SUT = $self->_createSUT($Basket);
    $SUT->createPackstationShippingAddress($Servlet);

    is_deeply($Basket->get('ShippingAddress')->{'Attributes'}, $hExpectedAddress, "$SubTestName - same Shipping Address when pickup station is already defined");

    return;
}

sub _test_create_shipping_address_with_pickup_station {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $hAddress = {
        'SelectedShippingPickupStation' => undef,
        'FirstName'                     => 'FirstName',
        'LastName'                      => 'LastName',
        'FullName'                      => 'FirstName LastName',
        'Street'                        => 'Street1',
        'Zipcode'                       => 'Zipcode',
        'City'                          => 'City1',
        'AddressExtension'              => 'AddressExtension',
        'CountryID'                     => 'CountryID',
        'IsPickupAddress'               => 'IsPickupAddress',
        'EMail'                         => 'EMail',
        'DHLPackstationId'              => 'DHLPackstationId'
    };

    my $UserAddress;
    my $User = $self->_createUser($UserAddress);

    my $BasketAddress = $self->_createShippingAddress($hAddress);
    my $Basket = $self->_createBasket($User, $BasketAddress);

    my $hPackstationAddress = {
        'location' => 'other',
        'packstationId' => '123',
        'name' => 'packstation_Name',
        'address' => 'packstation_Address',
        'zipcode' => '20355',
        'city' => 'packstation_City',
        'PostNumber' => '50000000'
    };

    my $Servlet = $self->_createServlet($Basket, $hPackstationAddress);

    my $SUT = $self->_createSUT($Basket);
    $SUT->createPackstationShippingAddress($Servlet);

    my $hExpectedAddress = {
          'DHLPackstationId' => '123',
          'FirstName' => 'FirstName',
          'Street' => 'Packstation 123',
          'City' => 'packstation_City',
          'EMail' => 'EMail',
          'Alias' => 'next Alias',
          'Zipcode' => '20355',
          'DHLPostNumber' => '50000000',
          'LastName' => 'LastName',
          'CountryID' => 1,
          'SelectedShippingPickupStation' => 'Loaded Dummy PickupStation Object',
          'DisplayName' => 'FirstName LastName',
          'AddressExtension' => '50000000',
          'IsPickupAddress' => 1
    };

    is_deeply($Basket->get('ShippingAddress')->{'Attributes'}, $hExpectedAddress, "$SubTestName - correct Shipping Address when pickup station selected");
    return;
}

sub _test_create_shipping_address_with_user_pickup_station {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $hAddress = {
        'SelectedShippingPickupStation' => undef,
        'FirstName'                     => 'FirstName',
        'LastName'                      => 'LastName',
        'FullName'                      => 'FirstName LastName',
        'Street'                        => 'Street1',
        'Zipcode'                       => 'Zipcode',
        'City'                          => 'City1',
        'AddressExtension'              => 'AddressExtension',
        'CountryID'                     => 'CountryID',
        'IsPickupAddress'               => 'IsPickupAddress',
        'EMail'                         => 'EMail',
        'DHLPackstationId'              => 'DHLPackstationId'
    };

    my $UserAddress;
    my $User = $self->_createUser($UserAddress);

    my $BasketAddress = $self->_createShippingAddress($hAddress);
    my $Basket = $self->_createBasket($User, $BasketAddress);

    my $hPackstationAddress = {
        'location' => '123|packstation_Name|packstation_Address. 25|20355|packstation_City',
        'PostNumber' => '50000000'
    };

    my $Servlet = $self->_createServlet($Basket, $hPackstationAddress);

    my $SUT = $self->_createSUT($Basket);
    $SUT->createPackstationShippingAddress($Servlet);

    my $hExpectedAddress = {
          'DHLPackstationId' => '123',
          'FirstName' => 'FirstName',
          'Street' => 'Packstation 123',
          'City' => 'packstation_City',
          'EMail' => 'EMail',
          'Alias' => 'next Alias',
          'Zipcode' => '20355',
          'DHLPostNumber' => '50000000',
          'LastName' => 'LastName',
          'CountryID' => 1,
          'SelectedShippingPickupStation' => 'Loaded Dummy PickupStation Object',
          'DisplayName' => 'FirstName LastName',
          'AddressExtension' => '50000000',
          'IsPickupAddress' => 1
    };

    is_deeply($Basket->get('ShippingAddress')->{'Attributes'}, $hExpectedAddress, "$SubTestName - correct Shipping Address when using user defined pickup station");
    return;
}

sub _test_invalid_shipping_address {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $UserAddress;
    my $User = $self->_createUser($UserAddress);

    my $BasketAddress;
    my $Basket = $self->_createBasket($User, $BasketAddress);

    my $hPackstationAddress = {};
    my $Servlet = $self->_createServlet($Basket, $hPackstationAddress);

    my $SUT = $self->_createSUT($Basket);

    throws_ok(sub {
            $SUT->createPackstationShippingAddress($Servlet);
        }, qr/InvalidShippingAddress/, "$SubTestName - throws error when Shipping Address is not Valid");
    return;
}

sub _test_invalid_user_defined_pickup_station {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $hAddress = {
        'SelectedShippingPickupStation' => undef,
        'FirstName'                     => 'FirstName',
        'LastName'                      => 'LastName',
        'FullName'                      => 'FirstName LastName',
        'Street'                        => 'Street1',
        'Zipcode'                       => 'Zipcode',
        'City'                          => 'City1',
        'AddressExtension'              => 'AddressExtension',
        'CountryID'                     => 'CountryID',
        'IsPickupAddress'               => 'IsPickupAddress',
        'EMail'                         => 'EMail',
        'DHLPackstationId'              => 'DHLPackstationId'
    };

    my $UserAddress;
    my $User = $self->_createUser($UserAddress);

    my $BasketAddress = $self->_createShippingAddress($hAddress);
    my $Basket = $self->_createBasket($User, $BasketAddress);

    my $hPackstationAddress = {
        'location' => '123|',
        'PostNumber' => '50000000'
    };

    my $Servlet = $self->_createServlet($Basket, $hPackstationAddress);

    my $SUT = $self->_createSUT($Basket);

    throws_ok(sub {
            $SUT->createPackstationShippingAddress($Servlet);
        }, qr/Versandadresse/, "$SubTestName - throws error when user defined Packstation is not Valid");
    return;
}

sub _test_invalid_city_pickup_station {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];
    
    my $hAddress = {
        'SelectedShippingPickupStation' => undef,
        'FirstName'                     => 'FirstName',
        'LastName'                      => 'LastName',
        'FullName'                      => 'FirstName LastName',
        'Street'                        => 'Street1',
        'Zipcode'                       => 'Zipcode',
        'City'                          => '',
        'AddressExtension'              => 'AddressExtension',
        'CountryID'                     => 'CountryID',
        'IsPickupAddress'               => 'IsPickupAddress',
        'EMail'                         => 'EMail',
        'DHLPackstationId'              => 'DHLPackstationId'
    };

    my $UserAddress;
    my $User = $self->_createUser($UserAddress);

    my $BasketAddress = $self->_createShippingAddress($hAddress);
    my $Basket = $self->_createBasket($User, $BasketAddress);

    my $hPackstationAddress = {
        'location' => 'other',
        'packstationId' => '123',
        'name' => 'packstation_Name',
        'address' => 'packstation_Address',
        'zipcode' => '20355',
        'city' => '',
        'PostNumber' => '50000000'
    };

    my $Servlet = $self->_createServlet($Basket, $hPackstationAddress);
    my $SUT = $self->_createSUT($Basket);

    throws_ok(sub {
            $SUT->createPackstationShippingAddress($Servlet);
        }, qr/Packstation Daten/, "$SubTestName - throws error when Packstation city is not Valid");
    return;
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;
    my ($Basket, $hPackstationAddress) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Basket);

    $MockifyServlet->addMockWithReturnValue('form', $self->_createForm());
    $MockifyServlet->addMockWithReturnValue('params', $hPackstationAddress);

    return $MockifyServlet->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($Basket) = @_;

    my $aParameterList = [];
    my $MockedSUT = Test::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::PackstationShippingAddress',
        $aParameterList );

    $MockedSUT->mock('_LogShippingFulfillment')
        ->when(String('DHLIntraship'), String('Packstation Address fields'), HashRef())
        ->thenReturnUndef();

    $MockedSUT->mock('_LogShippingFulfillment')
        ->when(String('DHLIntraship'), String('User defined Packstation Address fields'), String('123|packstation_Name|packstation_Address. 25|20355|packstation_City'))
        ->thenReturnUndef();

    $MockedSUT->mock('_LogShippingFulfillment')
        ->when(String('DHLIntraship'), String('User defined Packstation Address fields'), String('123|'))
        ->thenReturnUndef();

    $MockedSUT->mock('_LoadObjectByPath')
        ->when(String('/ShippingPickupStations/DHLIntashipPackstation'))
        ->thenReturn('Loaded Dummy PickupStation Object');
    
    $MockedSUT->mock('_GetKeyCountryByCode')
        ->when(String('DE'))
        ->thenReturn(1);

    return $MockedSUT->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShippingAddress {
    my $self = shift;
    my ($hAttributes) = @_;

    my $MockedAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hAttributes
    );

    return $MockedAddress->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createUser {
    my $self = shift;
    my ($hAddress) = @_;

    my $hParameters = {'BillingAddress' => $hAddress};

    my $MockifyUser = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Permission::API::Object::User',
        $hParameters
    );

    $MockifyUser->addMockWithReturnValueAndParameterCheck('folder', $self->_createFolder(), [{'string' => 'Addresses'}]);

    return $MockifyUser->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createFolder {
    my $self = shift;

    my $MockedFolder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object', {});

    $MockedFolder->mock('nextAlias', 'next Alias', [{'string'=>'PickUpAddress%d'}]);
    
    $MockedFolder->mock('insertChild', sub {
        shift;
        my ($FolderName, $hNewShippingAddressData) = @_;
        return $self->_createShippingAddress($hNewShippingAddressData); 
    });

    return $MockedFolder->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my ($User, $ShippingAddress) = @_;

    my $hParameters = {
            'ShippingAddress' => $ShippingAddress,
            'User' => $User,
            'Site' => $self->_createShop()
    };

    my $MockedBasket = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Order::API::Object::Basket',
        $hParameters
    );

    return $MockedBasket->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'ID' => 12345,
        'Alias' =>'DemoShop',
    };

    my $MockedShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    my $Shop  = $MockedShop->getMockObject();
    $Shop->set({'Site' => $Shop});

    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createForm {
    my $self = shift;

    my $MockedForm = Test::Mockify->new('DE_EPAGES::Presentation::API::Form');

    my $ExecuteFormErrorSub = sub {
        my ($hMessages) = @_;
        Error($hMessages->{'Reason'});
    };

    $MockedForm->mock('executeFormError')->when(HashRef())->thenCall($ExecuteFormErrorSub);
    $MockedForm->mock('executeFormError')->when(String('123'))->thenCall($ExecuteFormErrorSub);

    return $MockedForm->getMockObject();
}

__PACKAGE__->runScript();
