﻿#========================================================================================
# §package      DE_EPAGES::Design::API::Object::Style
# §state        public
# §base         DE_EPAGES::Shop::API::Object::ShopObject
# §description  object interface for Style.
#========================================================================================
package DE_EPAGES::Design::API::Object::Style;
use base DE_EPAGES::Shop::API::Object::ShopObject;

use strict;
use utf8;
use DE_EPAGES::Core::API::File qw (CreateDirectory GetFileContent DeleteFile CopyDirectory WriteFile ConvertPathChars ExistsDirectory RenameDirectory DeleteDirectory GetFileNames);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadClassByAlias );
use DE_EPAGES::Presentation::API::Object::ObjectPageType;
use DE_EPAGES::Presentation::API::PageType qw( LoadPageTypeByAlias );
use DE_EPAGES::Presentation::API::Template qw ( ProcessPage GetTLEProcessor );
use DE_EPAGES::TLE::API::TLE qw( GetByteCodeByText );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# §function     className
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $ClassName = $Style->className;
#----------------------------------------------------------------------------------------
# §description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# §return       $ClassName | class name | String
#========================================================================================
sub className  { 'Style' }

#========================================================================================
# §function     dbPackage
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $DbPackage = $Style->dbPackage;
#----------------------------------------------------------------------------------------
# §description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# §return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# §function     insert
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Style = DE_EPAGES::Design::API::Object::Style->insert($hInfo);
# §example      my $Class = LoadClassByAlias('Style');
#               my $Style = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# §description  Insert this info as object, afterwards the hook 'OBJ_InsertStyle' is triggered.
#----------------------------------------------------------------------------------------
# §input        $hInfo | attributes of new object | ref.hash
# §return       $Style | new object | object
# §hook         OBJ_InsertStyle | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>StyleID       - style id             - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    CreateDirectory($self->get('PublicPath'));

    TriggerHook('OBJ_InsertStyle', {'Object' => $self, 'StyleID' => $self->id });

    delete $hInfo->{$_} foreach qw ( Shop LayoutTemplate RoofIsVisible HeaderIsVisible TopIsVisible LeftIsVisible RightIsVisible BottomIsVisible FooterIsVisible ChangeDate LayoutImageSet LayoutIconSet LayoutWidth ContentFontFamily StyleTemplate Counter IsInvalid ); ## no critic qw(ProhibitLongLines)

    return $self;
}

#========================================================================================
# §function     deleteFiles
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Object->deleteFiles;
#----------------------------------------------------------------------------------------
# §description  Deletes the directories PublicPath and calls super.
#========================================================================================
sub deleteFiles {
    my $self = shift;

    DeleteDirectory( $self->get('PublicPath'),  {'recursive' => 1} );
    $self->getSite()->clearSavedPages;
    $self->SUPER::deleteFiles;
    return;
}

#========================================================================================
# §function     delete
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Style->delete;
#----------------------------------------------------------------------------------------
# §description  Deletes this object, therefore the hook 'OBJ_DeleteStyle' is triggered.
#----------------------------------------------------------------------------------------
# §hook         OBJ_DeleteStyle | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>StyleID       - style id             - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteStyle', {'Object' => $self, 'StyleID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# §function     set
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Style->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# §description  This function triggers hooks 'OBJ_BeforeUpdateStyle' and 'OBJ_AfterUpdateStyle'.
#----------------------------------------------------------------------------------------
# §input        $hValues | attribute name => value hash | ref.hash
# §input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# §hook         OBJ_BeforeUpdateStyle | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>StyleID       - style id             - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# §hook         OBJ_AfterUpdateStyle | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>StyleID       - style id             - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    # remember if the Alias or Parent has changed
    my $ChangedAlias = 0;
    my $ChangedParent = 0;

    # update children cache if parent or alias was changed
    if (exists $hValues->{'Parent'}) {
        my $Parent = $self->parent or die 'Can\'t change the parent of the root object';
        die 'Can\'t change the parent to me ' .  $Parent->id if $Parent->id == $self->id;
        $ChangedParent = $Parent->id != $hValues->{'Parent'}->id;
        delete $hValues->{'Parent'} unless $ChangedParent;
    } # else no parent change
    if (exists $hValues->{'Alias'}) {
        my $Alias = $self->alias;
        $ChangedAlias = $Alias ne $hValues->{'Alias'};
        delete $hValues->{'Alias'} unless $ChangedAlias;
    } # save default button style values
    if (exists $hValues->{'CustomizableButtons'} and not defined $self->get('DefaultButtonStyle')) {
        $hValues->{'DefaultButtonStyle'} = $hValues->{'CustomizableButtons'};
    }
    return unless scalar keys %$hValues;

    TriggerHook('OBJ_BeforeUpdateStyle', {'Object'=>$self, 'Values'=> $hValues, 'StyleID' => $self->id, 'LanguageID' => $LanguageID});

    # remember the old directory names, because they will be renamed
    my ($OldPublicPath, $OldPrivatePath, $OldStaticPath);
    if( $ChangedAlias || $ChangedParent ) {
        $OldPublicPath = $self->get('PublicPath');
        $OldPrivatePath = $self->get('PrivatePath');
        $OldStaticPath = $self->get('StaticPath');
    }

    my $ChangeStyle = 0;
    my @Keys = keys %$hValues;
    if (grep { not ElementOf($_, qw(IsTemporary ChangeDate ChangedByWizard StyleTemplate IsInvalid Counter)) } @Keys) {
        $hValues->{'IsInvalid'} = 1;
        $ChangeStyle = 1;
    }

    $self->SUPER::set($hValues, $LanguageID);

    # rename the object directories if the path has changed
    if( $ChangedParent || $ChangedAlias ) {
        $self->_deleteAttribute('FileAlias') if $ChangedAlias; ##no critic (ProhibitPrivateSubroutineUse)
        RenameDirectory( $OldPublicPath,  $self->get('PublicPath'),  { create_directory => 1 } ) if ExistsDirectory( $OldPublicPath );
        RenameDirectory( $OldPrivatePath, $self->get('PrivatePath'), { create_directory => 1 } ) if ExistsDirectory( $OldPrivatePath );
        RenameDirectory( $OldStaticPath,  $self->get('StaticPath'),  { create_directory => 1 } ) if ExistsDirectory( $OldStaticPath );
    }

    if ($ChangeStyle) {
        $self->updatePageCache;
    }

    TriggerHook('OBJ_AfterUpdateStyle',  {'Object'=>$self, 'Values'=> $hValues, 'StyleID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# §function     importXML
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Object = $Package->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# §description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# §input        $Handler | xml import handler with support function addObject | object
# §input        $hElement | XML element structure, see XML::SAX | ref.hash
# §input        $Class | class object; the new object will be an instance of this class | object
# §input        $hAttributes | attributes hash | ref.hash
# §return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    # fake for reset
    my $XMLParentObject = $Handler->topID('Object');
    if ($XMLParentObject->instanceOf('Style')) {
        $hAttributes->{'Alias'} = $XMLParentObject->alias;
        $hAttributes->{'Parent'} = $XMLParentObject->parent;
    }

    return $class->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);
}

#========================================================================================
# §function     navbarElements
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $aNavBarElements = $Style->navbarElements( $NavBarName, {'Alignment' => 'Left'} );
#----------------------------------------------------------------------------------------
# §description  Returns the navigation bar elements that are assigned to the navigation
#               bar with the given name in this style. The elements are ordered by
#               Position.
#----------------------------------------------------------------------------------------
# §input        $NavBarName | navigation bar alias | string
# §input        $hParams | params hash | ref.hash
# §return       $aNavBarElements | navigation bar elements | ref.array.object
#========================================================================================
sub navbarElements {
    my $self = shift;
    my ($NavBarName, $hParams) = @_;

    my $Alignment = exists $hParams->{'Alignment'} ? $hParams->{'Alignment'} : undef;
    my $Class = LoadClassByAlias('NavBarElement');
    my $NavBar = LoadRootObject->child('NavBars')->child($NavBarName);
    my $NavBarId = $NavBar->id;

    my $aElements = [sort { int $a->get('Position') <=> int $b->get('Position') }
                   grep { $_->get('NavBar')->id == $NavBarId }
                   @{ $Class->getObjects('ByStyleID', [ $self->id ]) }];

    return defined $Alignment ? [grep { $_->get('Alignment') eq $Alignment } @$aElements ] : $aElements;

}

#========================================================================================
# §function     writeCSSNoInc
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Style->writeCSSNoInc;
#----------------------------------------------------------------------------------------
# §description  Write Cascading Style Sheet for this style to
#               $PublicPath/StorefrontStyle_<Counter>.css and
#               $PublicPath/EditorStyle_<Counter>.css, but doesnt increment the counter.
#               Other cartridges can add their own css files by the implementing the
#               hook "API_WriteCSS".
#========================================================================================
sub writeCSSNoInc {
    my $self = shift;

    my $hVars = $self->tleHash;

    $self->_renderStyleExtensionStyles($hVars);

    $self->writeCSSFile('StorefrontStyle', 'SF-Style', $hVars);
    $self->writeCSSFile('EditorStyle', 'SF-EditorStyle', $hVars);

    TriggerHook('API_WriteCSS', { 'Object' => $self, 'Vars' => $hVars });

    $self->removeObsoleteStyleSheets unless $self->getSite()->get('IsPageCacheUsed');
    $self->set({ 'IsInvalid' => 0 });
}

#========================================================================================
sub _renderStyleExtensionStyles {
    my $self = shift;
    my ($hVars) = @_;

   # flush cached attribute
    $self->_deleteAttribute('StyleExtensionCSS');   ##no critic (ProhibitPrivateSubroutineUse)

    # Check if style has StyleExtension.css
    my $StyleExtensionCSSPath = $self->get('StyleExtensionCSS');

    if (defined $StyleExtensionCSSPath) {
        my $rContent = undef;
        if (defined LoadRootObject()->get('StaticResourceURL')) {
            $rContent = GetFileContent("$StyleExtensionCSSPath");
        } else {
            $rContent = GetFileContent("$ENV{'EPAGES_SHARED'}/$StyleExtensionCSSPath");
        }
        my $cByteCode = GetByteCodeByText($rContent);

        my $TLEProcessor = GetTLEProcessor('writeCSSNoIncPageProcessor');
        $TLEProcessor->reset if $TLEProcessor->can('reset');

        my $Content = $TLEProcessor->replaceTLEs($cByteCode, $hVars);
        $hVars->{'StyleExtensionStyles'} = "\n $Content \n";
    }
}

#========================================================================================
# §function     writeCSSFile
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Style->writeCSSFile($Alias, $PageTypeAlias, $hVars);
# §example      $Style->writeCSSFile('StorefrontStyle', 'SF-Style', $hVars);
#----------------------------------------------------------------------------------------
# §description  Writes the actual css file for the given pagetype with the given vars at
#               $PublicPath/<$Alias>_<Counter>.css
#========================================================================================
# §input        $Alias | File name prefix alias | string
# §input        $PageTypeAlias | alias of the pagetype | string
# §input        $hVars | TLE vars hash | ref.hash
#========================================================================================
sub writeCSSFile {
    my $self = shift;
    my ($Alias, $PageTypeAlias, $hVars) = @_;

    my $TLEProcessor = GetTLEProcessor('writeCSSNoIncPageProcessor');
    $TLEProcessor->reset if $TLEProcessor->can('reset');

    my $ObjectPageType = DE_EPAGES::Presentation::API::Object::ObjectPageType->load(LoadPageTypeByAlias($PageTypeAlias), $self);
    my $Content = ProcessPage($self, $ObjectPageType, 'en', $hVars, $TLEProcessor, undef, undef, 'css');

    WriteFile($self->get('PublicPath') . '/' . $Alias . '_' . $self->get('Counter') . '.css', \$Content, { 'create_directory' => 1 });
}

#========================================================================================
# §function     writeCSS
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Style->writeCSS;
#----------------------------------------------------------------------------------------
# §description  Write Cascading Style Sheet for this style to
#               $PublicPath/StorefrontStyle_<Counter>.css and
#               $PublicPath/EditorStyle_<Counter>.css, increments the counter before.
#========================================================================================
sub writeCSS {
    my $self = shift;

    my $Counter = $self->get('Counter') // 0;
    $self->set({'Counter' => $Counter + 1});

    $self->writeCSSNoInc;
}

#========================================================================================
# §function     buildCSSWebPath
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Style->buildCSSWebPath($StyleSheetAlias);
# §example      $Style->buildCSSWebPath('StorefrontStyle');
#----------------------------------------------------------------------------------------
# §description  Builds the URL where the given style sheet alias is located.
#               The style will be generated if the IsInvalid flag is set.
#----------------------------------------------------------------------------------------
# §input        $Alias | style sheet alias | string
# §return       $CSSWebPath | style sheet web path | string
#========================================================================================
sub buildCSSWebPath {
    my $self = shift;
    my ($Alias) = @_;

    $self->writeCSS if $self->get('IsInvalid');
    return $self->get('WebPath') . '/' . $Alias . '_' . $self->get('Counter') . '.css';
}

#========================================================================================
# §function     removeObsoleteStyleSheets
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Style->removeObsoleteStyleSheets;
#----------------------------------------------------------------------------------------
# §description  Removes obsolete .css files, generated by <function writeCSSNoInc>.
#               Other cartridges can add their own obsolete files by the implementing the
#               hook "API_RemoveObsoleteCSS".
#========================================================================================
sub removeObsoleteStyleSheets {
    my $self = shift;

    $self->removeCSSFiles(['StorefrontStyle', 'EditorStyle']);

    TriggerHook('API_RemoveObsoleteCSS', { 'Object' => $self });
}

#========================================================================================
# §function     removeCSSFiles
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Style->removeCSSFiles(\@FileNamePrefixes);
# §example      $Style->removeCSSFiles(['StorefrontStyle']);
# §example      $Style->removeCSSFiles(['StorefrontStyle', 'EditorStyle']);
#----------------------------------------------------------------------------------------
# §description  Actually deletes all obsoletes CSS files in the style's public path whose
#               file name begins with the given prefixes.
#----------------------------------------------------------------------------------------
# §input        $aFileNamePrefixes | list of file name prefixes  | ref.array
#========================================================================================
sub removeCSSFiles {
    my $self = shift;
    my ($aFileNamePrefixes) = @_;

    my $PublicPath = $self->get('PublicPath');
    return unless ExistsDirectory($PublicPath);

    my $Counter = $self->get('Counter') // 0;
    my $RegEx = join('|', map { quotemeta($_ . '_') } grep { /\S/ } @$aFileNamePrefixes);

    # delete files with a smaller counter value
    DeleteFile($_) foreach
        grep { /(?:$RegEx)(\d+)\.css$/ and $1 < $Counter }
        @{GetFileNames($PublicPath)};
}

#========================================================================================
# §function     clone
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Clone = $Style->clone($hInfo, $Recursive);
#----------------------------------------------------------------------------------------
# §description  Clones style and Navbar elements.
#----------------------------------------------------------------------------------------
# §input        $hInfo | hash with attribute values that are different in the clone object.
#                       Note that the clone cannot have the same Parent and the same Alias.
#                    | hash ref
# §input        $Recursive | clone child object too (optional, default is false) | boolean
#----------------------------------------------------------------------------------------
# §return       $Clone | style object clone | object
#========================================================================================
sub clone {
    my $self = shift;
    my ($hInfo, $Recursive) = @_;

    my $Clone = $self->SUPER::clone($hInfo, $Recursive);
    my $aNavBarElements = $Clone->folder('NavBarElements')->children;
    foreach my $NavBarElement ( @$aNavBarElements) {
        $NavBarElement->set({ 'Style'=> $Clone });
    }

    return $Clone;
}

#========================================================================================
# §function     duplicate
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Dublicate = $Style->duplicate;
#----------------------------------------------------------------------------------------
# §description  duplicates style
#----------------------------------------------------------------------------------------
# §return       $Clone | duplicated style object | object
#========================================================================================
sub duplicate {
    my $self = shift;

    my $OldAlias = $self->alias;
    my $cnt = 1;
    my $NewAlias = $OldAlias;
    # test if Alias has already an ext like [1] and replace ext if exists
    while($self->parent->existsChild($NewAlias)) {
        $NewAlias =~ s/(.*?)(\[\d+\]|)$/$1\[$cnt\]/;
        $cnt++;
    }

    return $self->clone( { 'Alias' => $NewAlias }, 1 );
}

#========================================================================================
# §function     filePath
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Path = $Style->filePath;
#----------------------------------------------------------------------------------------
# §description  Call <function DE_EPAGES::Core::API::File::ConvertPathChars> with site 'Alias' and parent of site.
#----------------------------------------------------------------------------------------
# §return       $Path | relative file path | string
#========================================================================================
sub filePath {
    my $self = shift;

    return join '/', $self->getSite()->get('FilePath'), ConvertPathChars($self->parent->alias), ConvertPathChars($self->alias);
}

#========================================================================================
# §function     canInvalidatePageCache
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Object->canInvalidatePageCache;
#----------------------------------------------------------------------------------------
# §description  Overwrites method DE_EPAGES::Object::API::Object::Object->canInvalidatePageCache
#               to invalidate page cache.
#----------------------------------------------------------------------------------------
# §return       $Result | if $Object can invalidate pagecache: 1 else: 0 | boolean
#========================================================================================
sub canInvalidatePageCache {
    my $self = shift;
    return $self->isDefault ? 1 : 0;
}

sub reset {
    my $self = shift;
    my ($StyleTemplate) = @_;
    $StyleTemplate = $self->get('StyleTemplate') unless defined $StyleTemplate;

    # prepare import
    my $allAttributes = $self->exportableAttributes;
    foreach my $Attribute (keys %$allAttributes) {
      if ($Attribute =~ /^(Roof|Bottom|Content|Footer|Header|Top|Left|Right|Layout|Wizard|UserDefined|LockAdvancedSettings)/x) { ## no critic qw(ProhibitComplexRegexes)
          $allAttributes->{$Attribute} = '';
      } else {
          delete $allAttributes->{$Attribute};
      }
    }

    # flush cached attribute
    $self->_deleteAttribute('StyleExtensionCSS');   ##no critic (ProhibitPrivateSubroutineUse)

    $self->set({
        %$allAttributes,
        RoofIsVisible    => 0,
        HeaderIsVisible  => 1,
        TopIsVisible     => 1,
        LeftIsVisible    => 1,
        RightIsVisible   => 1,
        BottomIsVisible  => 1,
        FooterIsVisible  => 1,
        ChangeDate       => undef,
    });

    #delete old NavBarElements
    $_->delete foreach @{$self->folder('NavBarElements')->children};

    my $System = LoadRootObject();
    my $Directory = join ('/', $System->get('PublicStoreTypePath'), $StyleTemplate->get('StyleDir'));

    $self->set({'StyleTemplate' => $StyleTemplate, 'IsImportedWithoutTemplate' => 0});

    my $TargetDir = $self->get('PublicPath');
    CopyDirectory($Directory, $TargetDir);
    CopyDirectory($Directory . '/MediaGallery', $self->getSite()->get('MediaGallery')->get('PublicPath')) if ExistsDirectory($Directory . '/MediaGallery');

    # delete the SubStyleDirectory because we do not need this subdir
    my $SubStyleDir = $TargetDir.'/SubStyles';
    DeleteDirectory($SubStyleDir,{'recursive' => 1}) if ExistsDirectory($SubStyleDir);
    DeleteFile($TargetDir.'/'.$_) foreach (
        'export.xml',
        'img_colorpreview.gif',
        'img_medium_stylepreview.gif',
        'img_small_stylepreview.gif',
        'StorefrontStyle.css',
        'StyleExtension.css',
        'StyleExtension.js'
    );

    $self->set({'IsInvalid' => 1});
    return;
}

#========================================================================================
# §function     isDefault
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $IsDefault = $Style->isDefault;
# §example      if( $Style->isDefault ) { ... }
#----------------------------------------------------------------------------------------
# §description  Returns true if this style is the default style of the shop.
#----------------------------------------------------------------------------------------
# §return       $IsDefault | true if this style is the default style | boolean
#========================================================================================
sub isDefault {
    my $self = shift;

    my $DefaultStyle = $self->getSite()->get('DefaultStyle');
    return 0 unless defined $DefaultStyle;
    return $self->id() eq $DefaultStyle->id ? 1 : 0;
}

#========================================================================================
# §function     default
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hValues = DE_EPAGES::Design::API::Object::Style->default( $Parent, $hInfo );
# §example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# §description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# §input        $Parent | parent object of the new object | object
# §input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# §return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'LayoutTemplate' => 'Layout1',
        'RoofIsVisible' => 0,
        'HeaderIsVisible' => 1,
        'TopIsVisible' => 1,
        'LeftIsVisible' => 1,
        'RightIsVisible' => 1,
        'BottomIsVisible' => 1,
        'FooterIsVisible' => 1,
        'Shop'=> $Parent->getSite(),
        'Counter' => 0,
        'IsInvalid' => 1,
    };
}

1;
