#========================================================================================
# package      package DE_EPAGES::Design::API::Style;
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides functions to work with Styles.
#========================================================================================
package DE_EPAGES::Design::API::Style;
use base Exporter;
use strict;

our @EXPORT_OK = qw( AddStyle ExistsStyle Style AddStyleLogo DeleteStyleLogo InvalidateAllShopStyles ValidateAndCompareToDefaultButtonValues
 DeleteTemporaryStyles ValidateButtonValues ProcessStyleTransformation );

use DE_EPAGES::Object::API::Factory           qw( LoadRootObject );
use DE_EPAGES::Core::API::Image               qw( ResizeImage );
use DE_EPAGES::Core::API::Error               qw( ExistsError GetError );
use DE_EPAGES::Core::API::File                qw( ExistsFile DeleteFile );
use DE_EPAGES::Core::API::PerlTools           qw( ElementOf GetCurrentDateTime );
use DE_EPAGES::Core::API::Transformation      qw( Transform );
use DE_EPAGES::Core::API::Transformation::ImportHandler qw( ReadTransformation );
use DE_EPAGES::Core::API::Warning             qw( Warning );
use DE_EPAGES::Trigger::API::Trigger          qw( TriggerHook );

#========================================================================================
# function     IsSubStyle
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $IsSubStyle = IsSubStyle($StyleName);
#----------------------------------------------------------------------------------------
# description  If StyleName contains the string "SubStyles/" return is 1
#----------------------------------------------------------------------------------------
# input        $StyleName | name of the style | string
# return       $IsSubStyle |  Style is Sub Style, 0 or 1 | boolean
#========================================================================================
sub IsSubStyle {
    my ($StyleName) = @_;

    # filter the string "SubStyles/" from the Alias
    return $StyleName =~ /SubStyles/ ? 1 : 0;
}

#========================================================================================
# function     AddStyle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $WantedStyle = AddStyle($Shop, $StyleTemplate);
#----------------------------------------------------------------------------------------
# description  Adds a Style to the shops style list.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $StyleTemplate | style template | object
# return       $WantedStyle | created style object | object
#========================================================================================
sub AddStyle {
    my ($Shop, $StyleTemplate ) = @_;

    my %Info = (
        'Object' => $Shop,
        'StyleTemplate' => $StyleTemplate,
        'Style' => undef,
    );
    TriggerHook('API_AddStyle', \%Info);
    return $Info{'Style'} if defined $Info{'Style'};
}

#========================================================================================
# function     ExistsStyle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = ExistsStyle($Shop, $StyleTemplate);
#----------------------------------------------------------------------------------------
# description  Returns true if the style with the given style template exists in this shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $StyleTemplate | style template | object
# return       $Exists | true if style exists | boolean
#========================================================================================
sub ExistsStyle {
    my ($Shop, $StyleTemplate) = @_;

    my $Parent = $StyleTemplate->get('ParentStyleTemplate')->alias;
    my $ShortStyleName = $Parent ne 'System'
        ?   join ('/', $Parent, $StyleTemplate->alias)
        :   $StyleTemplate->alias;

    return 0 unless $Shop->existsChild('Styles');
    return $Shop->child('Styles')->existsChild($ShortStyleName);
}

#========================================================================================
# function     Style
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Style = Style($Shop, $StyleName);
#----------------------------------------------------------------------------------------
# description  Returns style of this shop with the StyleName.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
# input        $StyleTemplate | style template | object
# return       $Style | style | object
#========================================================================================
sub Style {
    my ($Shop, $StyleTemplate) = @_;

    my $Parent = $StyleTemplate->get('ParentStyleTemplate')->alias;
    my $ShortStyleName = $Parent ne 'System'
        ?   join ('/', $Parent, $StyleTemplate->alias)
        :   $StyleTemplate->alias;

    return $Shop->child('Styles')->child($ShortStyleName);
}

#========================================================================================
# function     AddStyleLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddStyleLogo($Style, $ShopLogo, $Extension);
#----------------------------------------------------------------------------------------
# description  Sets the logo for a style.
#----------------------------------------------------------------------------------------
# input        $Style | style object | object
# input        $ShopLogo | public path for shoplogo | string
# input        $Extension | image extension | string
#========================================================================================
sub AddStyleLogo {
    my ($Style, $ShopLogo, $Extension) = @_;

    my $StyleLogo = $Style->get('PublicPath') . '/logo' . $Extension;
    my $ScaleImageSize = $Style->get('ScaleLogoSize');

    eval {
        if (defined $Style->get('Logo')){
            my $OldStyleLogo = $Style->get('PublicPath') . '/' . $Style->get('Logo');
            DeleteFile($OldStyleLogo) if ExistsFile($OldStyleLogo) and not $OldStyleLogo eq $StyleLogo;
        }
        if ($ScaleImageSize > 0) {
            ResizeImage($ShopLogo, $StyleLogo, undef, $ScaleImageSize);
        } else {
            ResizeImage($ShopLogo, $StyleLogo, -$ScaleImageSize);
        }
        $Style->set({'Logo' => 'logo' . $Extension});
    };
    if ( ExistsError() ){
        my $Error = GetError();
        if (ElementOf ($Error->code, 'FILE_NOT_FOUND', 'IMAGE_MAGICK', 'UNKNOWN_IMAGE_SIZE', 'IMAGE_MAGICK')){
            Warning('ResizeImageFailed', {
                'Code'      => $Error->code,
                'FileName'  => $ShopLogo,
                'StyleID'   => $Style->id,
            });
        }
    }
}

#========================================================================================
# function     DeleteStyleLogo
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteStyleLogo($Style);
#----------------------------------------------------------------------------------------
# description  Sets the logo for a style.
#----------------------------------------------------------------------------------------
# input        $Style | style object | object
#========================================================================================
sub DeleteStyleLogo {
    my ($Style) = @_;
    my $StyleLogo = $Style->get('Logo');
    eval {
        if (defined $StyleLogo){
            $Style->set({'Logo' => undef});
            my $DeleteLogo = $Style->get('PublicPath').'/'.$StyleLogo;
            DeleteFile($DeleteLogo) if ExistsFile($DeleteLogo);
        }
    };
    if ( ExistsError() ){
        my $Error = GetError();
        if (ElementOf ($Error->code, 'FILE_NOT_FOUND')){
            Warning('DeleteImageFailed', {
                'Code'      => $Error->code,
                'FileName'  => $StyleLogo,
                'StyleID'   => $Style->id,
            });
        }
    }
}

#========================================================================================
# function     InvalidateAllShopStyles
# state        public
#----------------------------------------------------------------------------------------
# syntax       InvalidateAllShopStyles($Shop);
#----------------------------------------------------------------------------------------
# description  Invalidates all styles of a given shop.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub InvalidateAllShopStyles {
    my ($Shop) = @_;
    my $aStyles = $Shop->child('Styles')->children;
    foreach my $Style (@$aStyles) {
        $Style->set({'IsInvalid' => 1});
    }
}

#========================================================================================
# function     DeleteTemporaryStyles
# state        public
#----------------------------------------------------------------------------------------
# syntax       DeleteTemporaryStyles($Shop);
#----------------------------------------------------------------------------------------
# description  Deletes all style from shop with attribute IsTemporary=1 except the
#               default one. The flag for the default one will be corrected. Cause for
#               programming is that the shopwizard could be chanceled (e.g. close
#               browser) and all the temporary style resist.
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | object
#========================================================================================
sub DeleteTemporaryStyles {
    my ($Shop) = @_;

    my $DefaultStyle = $Shop->get('DefaultStyle');
    $DefaultStyle->set( { 'IsTemporary' => 0 } );

    my $aStyles = $Shop->get('Styles');
    foreach my $Style (@$aStyles) {
        $Style->delete if $Style->get('IsTemporary');
    }
}

#========================================================================================
# function     ValidateButtonValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       ValidateButtonValues($hCustomizableBasketButton, $hVALIDATION);
#----------------------------------------------------------------------------------------
# description  Validates attributes of customizable buttons (basket button, other buttons)
#----------------------------------------------------------------------------------------
# input        $Form | Servlet form objevt | object
# input        $hButtonValues | hash with values to validate e.g.
#                                <ul>
#                                    <li>Gradient colors</li>
#                                    <li>Font (color, size)</li>
#                                    <li>Padding (width, height)</li>
#                                    <li>Border (color radius)</li>
#                                 </ul> allowed hashes:
#                                 Design/API/Constants.pm CUSTOMIZABLE_BUTTONS,
#                               | hash.ref
# input        $hVALIDATION | hash with validation information
#                                (type, max, min, attribute and key names) e.g.
#                                 Design/API/Constants.pm CUSTOMIZABLE_BUTTONS,
#                            | hash.ref
#----------------------------------------------------------------------------------------
# return       $hNewButtonValues | hash with the new validated button values | hash.ref
#========================================================================================
sub ValidateButtonValues {
    my ($Form, $hButtonValues, $hVALIDATION) = @_;

    my $hNewButtonValues;
    if(defined $hButtonValues){
        foreach my $Attribute (keys %$hVALIDATION) {
            foreach my $hKey (@{$hVALIDATION->{$Attribute}}) {
                my $KeyName = $hKey->{'Name'};
                my $NewValue = $hButtonValues->{$Attribute}->{$KeyName};
                if(defined $NewValue) {
                    $hNewButtonValues->{$Attribute}->{$KeyName} = $Form->formField($NewValue, $hKey);
                } else {
                    $hNewButtonValues->{$Attribute}->{$KeyName} = undef;
                }
            }
        }
    }

    return $hNewButtonValues;
}

#========================================================================================
# function     ValidateAndCompareToDefaultButtonValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       ValidateAndCompareToDefaultButtonValues($hCustomizableBasketButton, $hVALIDATION);
#----------------------------------------------------------------------------------------
# description  Validates attributes of customizable buttons (basket button, other buttons)
#               and checks if the values are different to the default button style settings.
#----------------------------------------------------------------------------------------
# input        $Form | Servlet form objevt | object
# input        $hButtonValues | hash with values to validate e.g.
#                                <ul>
#                                    <li>Gradient colors</li>
#                                    <li>Font (color, size)</li>
#                                    <li>Padding (width, height)</li>
#                                    <li>Border (color radius)</li>
#                                 </ul> allowed hashes:
#                                 Design/API/Constants.pm CUSTOMIZABLE_BUTTONS,
#                               | hash.ref
# input        $hDefaultValues (optional) | hash with default values (Style DefaultButtonStyle) | hash.ref
# input        $hVALIDATION | hash with validation information
#                                (type, max, min, attribute and key names) e.g.
#                                 Design/API/Constants.pm CUSTOMIZABLE_BUTTONS,
#                            | hash.ref
#----------------------------------------------------------------------------------------
# return       $hNewButtonValues | hash with the new validated button values | hash.ref
#========================================================================================
sub ValidateAndCompareToDefaultButtonValues {
    my ($Form, $hButtonValues, $hVALIDATION, $hDefaultValues) = @_;

    my $hNewButtonValues;
    my $IsCustomized = 0;
    if(defined $hButtonValues){
        foreach my $Attribute (keys %$hVALIDATION) {
            foreach my $hKey (@{$hVALIDATION->{$Attribute}}) {
                my $KeyName = $hKey->{'Name'};
                my $NewValue = $hButtonValues->{$Attribute}->{$KeyName};
                $hNewButtonValues->{$Attribute}->{$KeyName} = $Form->formField($NewValue, $hKey);
                if(defined $hDefaultValues) {
                    my $DefaultValue = $hDefaultValues->{$Attribute}->{$KeyName};
                    if($hNewButtonValues->{$Attribute}->{$KeyName} ne $DefaultValue) {
                        $IsCustomized = 1;
                    }
                }
            }
        }
    }
    if(defined $hDefaultValues) {
        $hNewButtonValues->{'IsCustomized'} = $IsCustomized;
    }

    return $hNewButtonValues;
}

#========================================================================================
# function     ProcessStyleTransformation
# state        public
#----------------------------------------------------------------------------------------
# syntax       ProcessStyleTransformation($Form, $Style);
#----------------------------------------------------------------------------------------
# description  Processes a new style transformation.
#----------------------------------------------------------------------------------------
# input        $Form | Servlet form object | object
# input        $Style | style object | object
#----------------------------------------------------------------------------------------
# return       $hResult | hash with transformation results | hash.ref
#========================================================================================
sub ProcessStyleTransformation {
    my ($Form, $Style) = @_;

    my $System = LoadRootObject();

    my $ImageSetDir = $System->get('PublicStoreTypePath').'/'.$Style->get('LayoutImageSet');
    my $ImportTransformXMLFile = $Style->get('PublicPath').'/transform.xml';

    my $XmlFile = "$ImageSetDir/transform.xml";

    if(ExistsFile($ImportTransformXMLFile)) {
        $XmlFile = $ImportTransformXMLFile;
    }

    my $hTransformation = ReadTransformation($XmlFile);
    my $hObjects = $hTransformation->{'Objects'};

    # mixin attributes from form or (when not submitted) from object
    my @MixinAttributes = qw(WizardColor1 WizardColor2 WizardColor3 WizardImage1 WizardImage2 WizardImage3);
    my @MixinImages = qw(WizardImage1 WizardImage2 WizardImage3);
    my @ChangedAttributes = ();
    foreach my $Attribute (grep { exists($hObjects->{$_}) } @MixinAttributes) {
        my $Value = undef;
        if ($Form->existsValue($Attribute)) {
            $Value = $Form->value($Attribute);
            push @ChangedAttributes, $Attribute;
        }
        elsif ($Style->class->existsAttribute($Attribute)) {
            $Value = $Style->get($Attribute);
        }
        $Value = undef unless defined $Value and $Value =~ /\S/;

        # add correct path to images
        if (defined $Value && ElementOf($Attribute, @MixinImages)) {
            my $SrcVal;
            if($Value =~ /^(MediaGallery|\/MediaGallery)/) {
                $SrcVal = $Style->getSite()->get('PublicPath') . "/$Value";
            } elsif($Value =~ /^EP_Images\//) {
                $SrcVal = $System->get('ImageDatabasePublicPath')  . "/original/$Value";
            } else {
                $SrcVal = $Style->get('PublicPath') . "/$Value";
            }

            $hObjects->{$Attribute}->{'TargetValue'} = $SrcVal;
            $hObjects->{$Attribute}->{'SourceValue'} = $SrcVal;
        }
        $hObjects->{$Attribute}->{'Value'} = $Value if defined $Value;
    }
    my $hVars = {'SourceDirectory' => "$ImageSetDir/Source",
                 'TargetDirectory' => $Style->get('PublicPath'),
                 'BaseDirectory' => $Style->get('PublicPath')};
    my $hResult = Transform($hTransformation, $hVars, \@ChangedAttributes);

    # set all known object attributes from hash
    my $FilePath = $System->get('PublicPath').'/'.$Style->get('FilePath');
    my $hSetAttributes = {map{$_ => $hResult->{$_}} grep{$Style->class->existsAttribute($_)} keys %$hResult};
    foreach my $attr (keys %$hSetAttributes) {
        $hSetAttributes->{$attr} =~ s/^\Q $FilePath \E[\\\/]*//x if ($attr =~ /Image$/);
    }
    my $date = GetCurrentDateTime();
    $hSetAttributes->{'ChangeDate'} = $date;
    $hSetAttributes->{'ChangedByWizard'} = $date;

    # if PreserveOldBackgroundImage parameter supplied, restore the former layerbackgroundimage if user-set background image is in place
    if($Form->value('PreserveOldBackgroundImage')) {
        my $CurrentLayoutBackgroundImage = $Style->get('LayoutBackgroundImage');
        if($CurrentLayoutBackgroundImage && (
            $CurrentLayoutBackgroundImage =~ /MediaGallery\// ||
            $CurrentLayoutBackgroundImage =~ /\/EP_Images\// )
          ){
            delete $hSetAttributes->{'LayoutBackgroundImage'}
        }
    }

    # EPG-29527 prevent image replacement and EPG-30136 only for shopwizard
    if ($Form->value('ComingFromShopWizard') && !$Style->get('UseHeaderBackgroundImage')
     && !exists($hSetAttributes->{'WizardColor1'}) && !exists($hSetAttributes->{'WizardColor2'})) {
        delete $hSetAttributes->{'HeaderBackgroundImage'};
    }

    $Style->set($hSetAttributes);

    return $hResult;
}

1;