#========================================================================================
# package      DE_EPAGES::MultiStoreCommon::API::Object::ObjectUpdate
# state        public
# base         DE_EPAGES::Object::API::Object::DALObject
# description  object interface for ObjectUpdate.
#========================================================================================
package DE_EPAGES::MultiStoreCommon::API::Object::ObjectUpdate;
use base DE_EPAGES::Object::API::Object::DALObject;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::MultiStoreCommon::API::Table::ObjectUpdate qw (
    GetObjectUpdateByObjectPathShopID
    DeleteObjectUpdateByShopIDObjectUpdateID
);

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ObjectUpdate->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ObjectUpdate' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ObjectUpdate->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ObjectUpdate = DE_EPAGES::MultiStoreCommon::API::Object::ObjectUpdate->insert($hInfo);
# example      my $Class = LoadClassByAlias('ObjectUpdate');
#               my $ObjectUpdate = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertObjectUpdate' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ObjectUpdate | new object | object
# hook         OBJ_InsertObjectUpdate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectUpdateID - object update identifier - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    # preparation
    my $aExistingEntries = GetObjectUpdateByObjectPathShopID($hInfo->{'ObjectPath'},$hInfo->{'Shop'}->id());
    my $Count = scalar @$aExistingEntries;

    # check if specific redundant entries already exists (EPC-350)
    if ($Count > 0){
        # avoid multiple redundant entries in objectupdate table (EPC-350)
        my @ObjectUpdateIDs;
        foreach my $hEntry (@$aExistingEntries){
            push(@ObjectUpdateIDs, $hEntry->{'ObjectUpdateID'});
        }

        # delete all redundant entries
        DeleteObjectUpdateByShopIDObjectUpdateID($hInfo->{'Shop'}->id(), \@ObjectUpdateIDs )
    }

    # insert object
    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertObjectUpdate', {'Object' => $self, 'ObjectUpdateID' => $self->id });
    delete $hInfo->{$_} foreach qw ( Shop ObjectPath Remove UpdateTime Value ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ObjectUpdate->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteObjectUpdate' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteObjectUpdate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectUpdateID - object update identifier - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteObjectUpdate', {'Object' => $self, 'ObjectUpdateID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ObjectUpdate->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateObjectUpdate' and 'OBJ_AfterUpdateObjectUpdate'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateObjectUpdate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectUpdateID - object update identifier - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateObjectUpdate | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ObjectUpdateID - object update identifier - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateObjectUpdate', {'Object'=>$self, 'Values'=> $hValues, 'ObjectUpdateID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateObjectUpdate',  {'Object'=>$self, 'Values'=> $hValues, 'ObjectUpdateID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::MultiStoreCommon::API::Object::ObjectUpdate->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Remove' => 0,
        'UpdateTime' => GetCurrentDateTime(),
    };
}

1;
