#========================================================================================
# package      DE_EPAGES::MultiStoreCommon::API::Table::ObjectUpdate
# state        public
# description  interface for table ObjectUpdate
#========================================================================================
package DE_EPAGES::MultiStoreCommon::API::Table::ObjectUpdate;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw (
    GetInfoObjectUpdate
    GetObjectUpdateByObjectPathShopID
    DeleteObjectUpdateByShopIDObjectUpdateID
    DeleteFromObjectUpdateByShopIDUpdateTime
    DeleteFromObjectUpdateByShopIDObjectUpdateID
);

use DE_EPAGES::Core::API::PerlTools qw (
    GetLoopFromDoubleArray
);
use DE_EPAGES::Core::API::TestParameter qw (
    TestArray
    TestInteger
    TestDateTime
);
use DE_EPAGES::Database::API::Connection qw (
    GetCurrentDBHandle
);

use DE_EPAGES::MultiStoreCommon::API::MultiStoreLog qw (
    MultiStoreLogDebug
);

#========================================================================================
# function     GetInfoObjectUpdate
#----------------------------------------------------------------------------------------
# syntax       my $ahInfo = GetInfoObjectUpdate($ShopID,$LB,$UB);
#----------------------------------------------------------------------------------------
# description  Returns attributes and its values of each record of table ObjectUpdate as
#               array of hashes within certains bounds. Bounds are the primary keys.
#----------------------------------------------------------------------------------------
# input        $ShopID | shop id | Integer
# input        $LB | lower bound in range of data records | Integer
# input        $UB | upper bound in range of data records | Integer
# return       $ahInfo | list of object infos | ref.array.hash
#========================================================================================
sub GetInfoObjectUpdate {
    my ( $ShopID, $LB, $UB ) = @_;

    MultiStoreLogDebug("GetInfoObjectUpdate($ShopID, [$LB], [$UB])");

    TestInteger('ShopID', $ShopID);
    TestInteger('LowerBound', $LB);

    my $dbi = GetCurrentDBHandle();
    if( $UB ){
        TestInteger('UpperBound', $UB);
    } else {
        $UB = $dbi->execute('SELECT MAX(objectupdateid) FROM objectupdate')->[0][0];
    }

    my $dbResult = $dbi->execute(<<'SQL', [ $ShopID, $LB, $UB ]);
SELECT objectupdateid, shopid, objectpath, remove, updatetime, value FROM objectupdate WHERE shopid = ? AND objectupdateid > ? AND objectupdateid <= ?
SQL
    my $ItemsList = GetLoopFromDoubleArray($dbResult, [qw ( ObjectUpdateID ShopID ObjectPath Remove UpdateTime Value )]);
    my $ItemsCount = scalar @$ItemsList;

    MultiStoreLogDebug("GetInfoObjectUpdate() items to update = $ItemsCount");

    return $ItemsList;
}


#========================================================================================
# function     DeleteFromObjectUpdateByShopIDUpdateTime
#----------------------------------------------------------------------------------------
# syntax       DeleteFromObjectUpdateByShopIDUpdateTime($ShopID,$UpdateTime);
#----------------------------------------------------------------------------------------
# description  Deletes each record from table ObjectUpdate which is older than <$UpdateTime> days.
#----------------------------------------------------------------------------------------
# input        $ShopID | shop id | Integer
# input        $UpdateTime | update time | DateTime-String
#========================================================================================
sub DeleteFromObjectUpdateByShopIDUpdateTime {
    my ( $ShopID, $UpdateTime ) = @_;

    MultiStoreLogDebug("DeleteFromObjectUpdateByShopIDUpdateTime($ShopID, [$UpdateTime])");

    TestInteger('ShopID', $ShopID);
    TestDateTime('UpdateTime', $UpdateTime);

    GetCurrentDBHandle()->do('DELETE FROM objectupdate WHERE shopid=? AND updatetime < ?', [ $ShopID, $UpdateTime ]);
}

#========================================================================================
# function     DeleteFromObjectUpdateByShopIDObjectUpdateID
#----------------------------------------------------------------------------------------
# syntax       DeleteFromObjectUpdateByShopIDObjectUpdateID($ShopID,$aObjectUpdateIDs);
#----------------------------------------------------------------------------------------
# description  Deletes each record from table ObjectUpdate whose key appears in the 2nd param.
#----------------------------------------------------------------------------------------
# input        $ShopID | shop id | Integer
# input        $aObjectUpdateIDs | List of ObjectUpdateIDs | ref.array.int
#========================================================================================
sub DeleteFromObjectUpdateByShopIDObjectUpdateID {
    my ( $ShopID, $aObjectUpdateIDs ) = @_;

    MultiStoreLogDebug("DeleteFromObjectUpdateByShopIDObjectUpdateID($ShopID, [?])");

    TestInteger('ShopID', $ShopID);
    TestArray('ObjectUpdateIDs', $aObjectUpdateIDs);

    my $ItemsCount = scalar @$aObjectUpdateIDs;
    MultiStoreLogDebug("DeleteFromObjectUpdateByShopIDObjectUpdateID() items to delete = $ItemsCount");
    return unless $ItemsCount;

    my ($ObjectUpdateID) = sort { $b <=> $a } @$aObjectUpdateIDs;
    MultiStoreLogDebug("DeleteFromObjectUpdateByShopIDObjectUpdateID() ObjectUpdateID = $ObjectUpdateID");

    GetCurrentDBHandle()->do('DELETE FROM objectupdate WHERE shopid=? AND objectupdateid <= (?)', [ $ShopID, $ObjectUpdateID ]);

    return;
}

#========================================================================================
# function     GetObjectUpdateByObjectPathShopID
#----------------------------------------------------------------------------------------
# syntax       GetObjectUpdateByObjectPathShopID($ObjectPath,$ShopID);
#----------------------------------------------------------------------------------------
# description  Get object updtae entries by path and shopID.
#----------------------------------------------------------------------------------------
# input        $ObjectPath | path of object | String
# input        $ShopID | shop id | Integer
# return       $ahObjectUpdates | List of ObjectUpdates | ref.array.hash
#========================================================================================
sub GetObjectUpdateByObjectPathShopID{
    my ($ObjectPath,$ShopID) = @_;

    # preparation
    my $dbi = GetCurrentDBHandle();

    # get result from table
    my $dbResult = $dbi->execute(<<'SQL', [ $ObjectPath, $ShopID ]);
SELECT objectupdateid,shopid,value,updatetime,remove FROM objectupdate WHERE objectpath = ? AND shopid = ?
SQL

   my $ItemsList = GetLoopFromDoubleArray($dbResult, [qw ( ObjectUpdateID ShopID Value UpdateTime Remove)]);

   return $ItemsList;
}

#========================================================================================
# function     DeleteObjectUpdateByShopIDObjectUpdateID
#----------------------------------------------------------------------------------------
# syntax       DeleteObjectUpdateByShopIDObjectUpdateID($ShopID,$aObjectUpdateIDs);
#----------------------------------------------------------------------------------------
# description  Deletes each record from table ObjectUpdate with specific shopid and object.
#----------------------------------------------------------------------------------------
# input        $ShopID | shop id | Integer
# input        $aObjectUpdateIDs | List of ObjectUpdateIDs | ref.array.int
#========================================================================================
sub DeleteObjectUpdateByShopIDObjectUpdateID {
    my ( $ShopID, $aObjectUpdateIDs ) = @_;

    MultiStoreLogDebug("DeleteFromObjectUpdateByShopIDObjectUpdateID($ShopID, [?])");

    TestInteger('ShopID', $ShopID);
    TestArray('ObjectUpdateIDs', $aObjectUpdateIDs);

    GetCurrentDBHandle()->do('DELETE FROM objectupdate WHERE shopid=? AND objectupdateid IN (?)', [ $ShopID, @$aObjectUpdateIDs ]);

    return;
}

1;