#========================================================================================
# package      DE_EPAGES::Order::Hooks::Shop
# state        public
# description  React on changing shop relevant data.
#========================================================================================
package DE_EPAGES::Order::Hooks::Shop;

use strict;
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime AddDuration );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias Transaction );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Order::API::Cartridge;
use DE_EPAGES::Order::API::Constants qw ( MAX_DAYS_OF_LATEST_ORDER_FOR_UNITY_MIGRATION );
use DE_EPAGES::Order::API::Shop qw ( ProcessOrderDocumentTextAttributesByLanguageID GetLatestOrderCreationDate );
use DE_EPAGES::Presentation::API::Template qw ( Translate );
use DE_EPAGES::Shop::API::Constants qw ( FEATURES_TO_EXCLUDE_FOR_UNITY_MIGRATION );
use DE_EPAGES::ShopMailType::API::ShopMailType qw( GetShopMailType );

#========================================================================================
# function     OnBeforeUpdateShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  OnBeforeUpdateShop is triggered, change tax matrix at baskets
#               (tax area at lineitemcontainer and tax class at lineitems).
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object
#                   <li>Values - without TaxMatrix no action
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateShop {
    my ($hParams) = @_;

    _ChangeTaxAreaAndClassOnOrdersBaskets($hParams)   if exists $hParams->{'Values'}{'TaxMatrix'};
    _ChangeTaxAreaAndClassOnOrdersBaskets($hParams,1) if exists $hParams->{'Values'}{'TaxMatrixDigital'};

    return;

}


sub _ChangeTaxAreaAndClassOnOrdersBaskets {
    my ($hParams,$IsDigital) = @_;

    my $Postfix = $IsDigital ? 'Digital' : '';

    my $Shop = $hParams->{'Object'};

    my $OldTaxMatrix = $Shop->get('TaxMatrix'.$Postfix);
    my $NewTaxMatrix = $hParams->{'Values'}->{'TaxMatrix'.$Postfix};
    return if !defined $OldTaxMatrix || $NewTaxMatrix->id == $OldTaxMatrix->id;

    my %TaxAreas;
    $TaxAreas{$_->alias} = $_ foreach @{$NewTaxMatrix->get('TaxAreas'.$Postfix)};
    my %TaxClasses;
    $TaxClasses{$_->alias} = $_ foreach @{$NewTaxMatrix->get('TaxClasses'.$Postfix)};

    my $aOrders = $Shop->get('Orders');
    foreach my $Order (@$aOrders) {
        my $LineItemContainer;
        if ($Order->existsChild('LineItemContainer')) {
            $LineItemContainer = $Order->child('LineItemContainer');
            _ChangeToNewTaxMatrix($NewTaxMatrix, $LineItemContainer, \%TaxAreas, \%TaxClasses, $IsDigital);
        }

        foreach my $OrderDocument( @{$Order->get('Documents')} ) {
            $LineItemContainer = $OrderDocument->get('LineItemContainer');
            next unless defined $LineItemContainer;
            next unless $LineItemContainer->isChildOf($OrderDocument);
            _ChangeToNewTaxMatrix($NewTaxMatrix, $LineItemContainer, \%TaxAreas, \%TaxClasses, $IsDigital);
        }
    }

    my $aUsers = $Shop->child('Users')->children;
    my $aAnonymousUsers = $Shop->folder('AnonymousUsers')->children;
    push @$aUsers, @$aAnonymousUsers;
    foreach my $User ( @$aUsers ) {
        my $aBaskets = undef;
        $aBaskets = $User->child('Baskets')->children if( $User->existsChild('Baskets') );
        if( defined $aBaskets ) {
            foreach my $Basket (@$aBaskets) {
                my $LineItemContainer = $Basket->child('LineItemContainer');
                _ChangeToNewTaxMatrix($NewTaxMatrix, $LineItemContainer, \%TaxAreas, \%TaxClasses, $IsDigital);
            }
        }
    }
}

sub _ChangeToNewTaxMatrix {
    my ($NewTaxMatrix,$LineItemContainer, $hTaxAreas, $hTaxClasses,$IsDigital) = @_;

    my $Postfix = $IsDigital ? 'Digital' : '';

    my $LineItemClass = LoadClassByAlias('LineItem');
    my @LineItems = grep { $_->instanceOf($LineItemClass) } @{$LineItemContainer->children};
    Transaction( sub {
        foreach my $LineItem (@LineItems) {
            my $OldTaxClass = $LineItem->get('TaxClass'.$Postfix);
            next unless defined $OldTaxClass;
            my $NewTaxClass = $hTaxClasses->{$OldTaxClass->alias} // $NewTaxMatrix->get('DefaultTaxClass');
            $LineItem->set({'TaxClass'.$Postfix => $NewTaxClass } );
        }
        my $OldTaxArea = $LineItemContainer->get('TaxArea'.$Postfix);
        return unless defined $OldTaxArea;
        my $NewTaxArea = $hTaxAreas->{ $OldTaxArea->alias } // $NewTaxMatrix->get('DefaultTaxArea');
        $LineItemContainer->set({ 'TaxArea'.$Postfix  => $NewTaxArea } );
    } );
}

#========================================================================================
# function     OnFinishShopWizard
#----------------------------------------------------------------------------------------
# syntax       OnFinishShopWizard( $hParams );
# state        public
#----------------------------------------------------------------------------------------
# description  Sets the BCC email addresses for mail type CustomerOrderConfirmation
#               on completion of the shop wizard (Hook CustomerOrderConfirmation).
#----------------------------------------------------------------------------------------
# input        $hParams | form values which are supplied by Form ShopWizard of class Shop.
#               <ul>
#                   <li>Shop - the current Shop - object
#                   <li>Data - form values from Shop.ShopWizard - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnFinishShopWizard {
    my ($hValues) = @_;

    my $Shop = $hValues->{'Shop'};
    my $hForm = $hValues->{'Data'};

    my $ShopMailType = GetShopMailType( $Shop, 'CustomerOrderConfirmation' );
    my $BCC = $ShopMailType->get('BCC');
    $ShopMailType->set({ 'BCC' =>  $hForm->{'EMail'} }) unless defined $BCC;
    $ShopMailType->set({ 'IsActive' =>  1 });
    return;
}

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Creates the folders 'ProductTypes' and 'Products' when a new shop is
#               created.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    my $Cartridge = DE_EPAGES::Order::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);
}
#========================================================================================
# function     OnJSONAdvancedLinks
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnJSONAdvancedLinks( $hParams );
#----------------------------------------------------------------------------------------
# description  adds a link to to advanced links section in the link picker inside the
#               MBO
#               (Hook OBJ_JSONAdvancedLinks)
#----------------------------------------------------------------------------------------
# input        $hParams |
#               <ul>
#                   <li>Object - shop - object</li>
#                   <li>Values - attribute values - ref.hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnJSONAdvancedLinks {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    if($Shop->featureMaxValue('Basket')){
        my $SitePath = $hParams->{'Object'}->pathString;
        push @{$hParams->{'Links'}},{
            Alias=>Translate('Basket',undef,'DE_EPAGES::Order'),
            Link=>"?ChangeAction=CreateBasket&ObjectPath=$SitePath"
        };
    }
}

#========================================================================================
# function     OnSaveNumberFormats
#----------------------------------------------------------------------------------------
# syntax       OnSaveNumberFormats( $hParams );
# state        public
#----------------------------------------------------------------------------------------
# description  Evaluates the forms 'SaveNumberFormats_CustomerOrder' and
#               'SaveNumberFormats_OrderDocument' and returns its data.
#               If the feature 'OrderDocument' is missing, the form
#               'SaveNumberFormats_OrderDocument' will be omitted.
#               Form errors are not thrown directly but passed to the caller.
#----------------------------------------------------------------------------------------
# input        $hParams | form values which are supplied by Forms SaveNumberFormats_CustomerOrder
#                          and SaveNumberFormats_OrderDocument of class Shop.
#               <ul>
#                   <li>Shop - the current Shop - DE_EPAGES::Shop::API::Object::Shop
#                   <li>Form - form object to evaluate - DE_EPAGES::Presentation::API::Form
#                   <li>Data - form values to return to caller - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnSaveNumberFormats {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Shop'};
    my $Form = $hParams->{'Form'};
    my $hValues = $hParams->{'Values'};

    my $hFormValues = $Form->form($Shop, 'SaveNumberFormats_CustomerOrder', 1);

    my $hFormValues_2 = {};
    if($Shop->featureMaxValue('OrderDocument')){
        $hFormValues_2 = $Form->form($Shop, 'SaveNumberFormats_OrderDocument', 1);
    }

    %$hValues = ( %$hValues, %$hFormValues, %$hFormValues_2 );
    return;
}

#========================================================================================
# function     OnBeforePatchUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforePatchUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforePatchUnityShop.
#               Updated oder document attributes in case the shop name was changed.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object  - shop                              - object
#                 <li>Values  - new shop values                   - ref.hash
#               </ul>
#========================================================================================
sub OnBeforePatchUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};
    my $LanguageID = $hParams->{'LanguageID'};

    return unless defined $LanguageID;

    if (exists $hValues->{'Name'}) {
        ProcessOrderDocumentTextAttributesByLanguageID($Shop, 'UnityMBO-OrderDocumentText2', 1, $LanguageID);
    }

    return;
}

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforeCreateUnityShop.
#               Imports the new orders shop notification and if the shop
#               has unseen orders, activates the notification
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Shop - shop object | object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Object'};
    # import notifications
    my $Cartridge = DE_EPAGES::Order::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);

    return;
}

#========================================================================================
# function     OnIsUnityMigratable
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnIsUnityMigratable($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook OBJ_IsUnityMigratable.
#               Can add further restrictions for the Unity migration.
#               Checks for an order within the last 30 days.
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - shop object</li>
#                   <li>Values - attributes</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnIsUnityMigratable {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    # skip if another hook function already denied a Unity migration
    return unless $hParams->{'Values'}->{'IsUnityMigratable'};

    if (exists FEATURES_TO_EXCLUDE_FOR_UNITY_MIGRATION->{'MinOrderAmount'}
        && FEATURES_TO_EXCLUDE_FOR_UNITY_MIGRATION->{'MinOrderAmount'} == 0 ) {
        # check if the shop has a min order value
        my $MinOrderAmounts = $Shop->get('MinOrderAmounts');
        my $ExistsMinOrderAmount = 0;
        foreach my $MinOrderAmount (@$MinOrderAmounts) {
            if (defined $MinOrderAmount->{'Value'}) {
                $ExistsMinOrderAmount = 1;
                last;
            }
        }

        $hParams->{'Values'}->{'IsUnityMigratable'} = not $ExistsMinOrderAmount;
        return if $ExistsMinOrderAmount;
    }

    my $OrderWithinTheLast30Days = 0;

    # check for an order within the last 30 days
    my $LatestOrderCreationDate = GetLatestOrderCreationDate($Shop);
    if(defined $LatestOrderCreationDate) {
        my $CompareDate = AddDuration($LatestOrderCreationDate, 'days', MAX_DAYS_OF_LATEST_ORDER_FOR_UNITY_MIGRATION);
        my $Now = GetCurrentDateTime();

        if ($CompareDate > $Now) {
            $OrderWithinTheLast30Days = 1;
        }
    }

    $hParams->{'Values'}->{'IsUnityMigratable'} = $OrderWithinTheLast30Days;
}

1;
