#========================================================================================
# package      DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus
# state        public
# base         DE_EPAGES::Order::API::Object::ContainerPaymentMethod
#----------------------------------------------------------------------------------------
# description  Object interface for the PayPalPlus PaymentMethod.
#========================================================================================
package DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus;
use base DE_EPAGES::Order::API::Object::ContainerPaymentMethod;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestString );
use DE_EPAGES::PayPalPlus::API::Constants qw (
    INTERNAL_CARTRIDGE_NAME
    SUPPORTED_CURRENCIES
);
use DE_EPAGES::PayPalPlus::API::Requirement;

use strict;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = $PaymentMethod->featureName;
#----------------------------------------------------------------------------------------
# description  Returns the feature name 'PayPalPlus'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName {
    return INTERNAL_CARTRIDGE_NAME;
}

#========================================================================================
# function     usableAtStorefront
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Usable = $PaymentMethod->usableAtStorefront($hVars);
#----------------------------------------------------------------------------------------
# description  Tests if a payment method is visible in the storefront.
#               Overrides function <function DE_EPAGES::Payment::API::Object::PaymentMethod::usableAtStorefront>
#               to evaluate if payment method is configured
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       true if payment method is usable in the storefront | boolean
#========================================================================================
sub usableAtStorefront {
    my $self = shift;
    my ($hVars) = @_;

    my $IsUsableAtStorefront = 1;
    my $MerchantClientId = $self->get('MerchantClientId');
    my $ExperienceProfileId = $self->get('ExperienceProfileId');

    if( not $self->SUPER::usableAtStorefront($hVars) ) {
        $IsUsableAtStorefront = 0;
    }

    if( not isValid($MerchantClientId) ) {
        $IsUsableAtStorefront = 0;
    }

    if( not isValid($ExperienceProfileId) ) {
        $IsUsableAtStorefront = 0;
    }

    if( not $self->areMandatoryFieldsSet() or not $self->isMerchantAddressSet() ) {
        $IsUsableAtStorefront = 0;
    }

    return $IsUsableAtStorefront;
}

#========================================================================================
# function     canAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       if ($PaymentMethod->canAddToBasket($hVars)) {...
#----------------------------------------------------------------------------------------
# description  Tests if a payment method can be added to the basket with the given currency.
#               Overrides function <function DE_EPAGES::Order::API::Object::ContainerPaymentMethod::canAddToBasket>
#               Valid currencies are: EUR, GBP
#----------------------------------------------------------------------------------------
# input        $hVars | <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gross or net price    - int
#               </ul> | ref.hash
# return       true if payment method can be used for the given currency | boolean
#========================================================================================
sub canAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    my $AreCurrenciesSupported = $hVars->{'CurrencyID'} ~~ [(SUPPORTED_CURRENCIES)];
    my $CanBeAddedToBasket = $AreCurrenciesSupported && $self->_callSuperCanAddToBasket($hVars);

    return $CanBeAddedToBasket;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    # delete preset name so we can set in later for all languages.
    delete($hInfo->{'Name'});

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'IsInTestMode' => 0,
        'DisplayLogo' => 0
    };
}

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $PaymentMethod = DE_EPAGES::Payment::API::Object::PaymentMethod->insert($hInfo);
# example      my $Class = LoadClassByAlias('PaymentMethod');
#               my $PaymentMethod = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  first SUPER::instert is called, then we set the default payment method
#               name for all languages installed to the shop.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $PaymentMethod | new object | object
# hook         OBJ_InsertPaymentMethod | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>PaymentMethodID - payment method id    - int
#               </ul>
#========================================================================================
sub insert {
     my $class = shift;
     my ($hInfo) = @_;

     my $self = $class->SUPER::insert($hInfo);

     $self->_setDefaultNameForAllLanguages();

     return $self;
}

#========================================================================================
# function     areMandatoryFieldsSet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $AreMandatoryFieldsSet = $PaymentMethod->areMandatoryFieldsSet();
#----------------------------------------------------------------------------------------
# description  Determines if all necessary address fields are set as mandatory.
#----------------------------------------------------------------------------------------
# return       True if all necessary fields are set as mandatory,
#               otherwise False is returned. | boolean
#========================================================================================
sub areMandatoryFieldsSet {
    my $self = shift;
    my $Shop = $self->getSite();
    my $Requirement = DE_EPAGES::PayPalPlus::API::Requirement->new($Shop);
    return $Requirement->areMandatoryFieldsSet();
}

#========================================================================================
# function     isMerchantAddressSet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsMerchantAddressSet = $PaymentMethod->isMerchantAddressSet();
#----------------------------------------------------------------------------------------
# description  Determines if all necessary fields of the 
#               shop address contain a valid value.
#----------------------------------------------------------------------------------------
# return       True if the address information is set,
#               otherwise False is returned. | boolean
#========================================================================================
sub isMerchantAddressSet {
    my $self = shift;
    my $Shop = $self->getSite();
    my $Requirement = DE_EPAGES::PayPalPlus::API::Requirement->new($Shop);
    return $Requirement->isMerchantAddressSet();
}

#========================================================================================
# function     getTrackingId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTrackingId();
#----------------------------------------------------------------------------------------
# description  Gets the tracking id of the shop that is needed for the
#               oauth authentication with PayPal.
#               We use the GUID of a shop for the oauth tracking.
# return       string
#========================================================================================
sub getTrackingId {
    my $self = shift;

    my $Shop = $self->get('Shop');
    return $Shop->get('GUID');
}

#========================================================================================
# function     getOnboardingUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       getOnboardingUrl();
#----------------------------------------------------------------------------------------
# description  Gets the onboarding url of the shop that is needed for the
#               oauth authentication with PayPal.
#               In case the passed shop is a marked as 'internal-test-shop' sandbox
#               urls is returned; otherwise live url will be returned.
# return       string
#========================================================================================
sub getOnboardingUrl {
    my $self = shift;

    my $OnboardingUrl;

    eval {
        my $PaymentType = $self->get('PaymentType');

        $OnboardingUrl = $self->isInTestMode() ?
            $PaymentType->get('MerchantOnboardingURLSandbox') :
            $PaymentType->get('MerchantOnboardingURLLive');

        TestString('MerchantOnboardingUrl', $OnboardingUrl);
    };
    if(ExistsError()) {
        Error('PAYPALPLUS_PBO_SETUP_INCOMPLETE');
    };

    return $OnboardingUrl;
}

#========================================================================================
# function     getPartnerAccountId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPartnerAccountId();
#----------------------------------------------------------------------------------------
# description  Gets the partner account id of the shop that is needed for the
#               oauth authentication with PayPal.
#               In case the passed shop is a marked as 'internal-test-shop' sandbox
#               credentials are returned; otherwise live credentials will be returned.
# return       string
#========================================================================================
sub getPartnerAccountId {
    my $self = shift;

    my $PartnerAccountId;

    eval {
        my $PaymentType = $self->get('PaymentType');
        $PartnerAccountId = $PaymentType->get('ProviderPayPalAccountId');

        TestString('ProviderPayPalAccountId', $PartnerAccountId)
    };
    if(ExistsError()) {
        Error('PAYPALPLUS_PBO_SETUP_INCOMPLETE');
    };

    return $PartnerAccountId;
}

#========================================================================================
# function     getPartnerClientId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPartnerClientId();
#----------------------------------------------------------------------------------------
# description  Gets the partner client id of the shop that is needed for the
#               oauth authentication with PayPal.
#               In case the passed shop is a marked as 'internal-test-shop' sandbox
#               credentials are returned; otherwise live credentials will be returned.
# return       string
#========================================================================================
sub getPartnerClientId {
    my $self = shift;

    my $PartnerClientId;

    eval {
        my $PaymentType = $self->get('PaymentType');

        $PartnerClientId = $self->isInTestMode() ?
            $PaymentType->get('ProviderClientIdSandbox') :
            $PaymentType->get('ProviderClientId');

        TestString('PartnerClientId', $PartnerClientId)

    };
    if(ExistsError()) {
        Error('PAYPALPLUS_PBO_SETUP_INCOMPLETE')
    };

    return $PartnerClientId;
}

#========================================================================================
# function     getPartnerClientSecret
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPartnerClientSecret();
#----------------------------------------------------------------------------------------
# description  Gets the partner client secret of the shop that is needed for the
#               oauth authentication with PayPal.
#               In case the passed shop is a marked as 'internal-test-shop' sandbox
#               credentials are returned; otherwise live credentials will be returned.
# return       string
#========================================================================================
sub getPartnerClientSecret {
    my $self = shift;

    my $PartnerClientSecret;

    eval {
        my $PaymentType = $self->get('PaymentType');

        $PartnerClientSecret = $self->isInTestMode() ?
            $PaymentType->get('ProviderClientSecretSandbox') :
            $PaymentType->get('ProviderClientSecret');

        TestString('PartnerClientSecret', $PartnerClientSecret);
    };
    if(ExistsError()) {
        Error('PAYPALPLUS_PBO_SETUP_INCOMPLETE')
    };

    return $PartnerClientSecret;
}

#========================================================================================
# function     getPayPalPartnerAttributionId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPayPalPartnerAttributionId();
#----------------------------------------------------------------------------------------
# description  Gets the partner attribution id that is used to track transactions
#               for our revenue share with PayPal.
# return       string
#========================================================================================
sub getPayPalPartnerAttributionId {
    my $self = shift;
    my $PaymentType = $self->get('PaymentType');
    return $PaymentType->get('PayPalPartnerAttributionId');
}

#========================================================================================
# function     isInTestMode
# state        public
#----------------------------------------------------------------------------------------
# syntax       isInTestMode();
#----------------------------------------------------------------------------------------
# description  Determines if the payment method is used in test or live mode.
# return       1 if payment methods uses sandbox; 0 otherwise.
#========================================================================================
sub isInTestMode {
    my $self = shift;

    my $Shop = $self->get('Shop');
    return $Shop->get('IsInternalTestShop');
}

#========================================================================================
# function     getMerchantClientId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getMerchantClientId();
#----------------------------------------------------------------------------------------
# description  Gets the merchant client id that allows us to authorize our calls
#               against the PayPal API of the merchants paypal account.
# return       string
#========================================================================================
sub getMerchantClientId {
    my $self = shift;
    return $self->get('MerchantClientId');
}

#----------------------------------------------------------------------------------------
# private
#----------------------------------------------------------------------------------------

sub _setDefaultNameForAllLanguages {
    my $self = shift;

    my $PaymentTypePayPalPlus = $self->get('PaymentType');
    my $Shop = $self->get('Shop');
    my $hInstalledLanguages = $Shop->get('Languages');

    foreach my $Language (@{ $hInstalledLanguages }) {

        my $LanguageID = $Language->{'LanguageID'};
        my $LanguageDefaultName = $PaymentTypePayPalPlus->get('PaymentMethodDefaultName', $LanguageID);

        if(isValid($LanguageDefaultName)) {
            $self->set({'Name' => $LanguageDefaultName}, $LanguageID);
        }
    }
}

#----------------------------------------------------------------------------------------
# wrappers
#----------------------------------------------------------------------------------------

sub _callSuperCanAddToBasket {
    my $self = shift;
    my ($hVars) = @_;

    return $self->SUPER::canAddToBasket($hVars);
}

1;