#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::PaymentMethodPayPalPlus
# base         DE_EPAGES::Test::API::UnitTest::Test;
#----------------------------------------------------------------------------------------
# description  Unit test for PaymentMethodPayPalPlus
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::PaymentMethodPayPalPlus;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus;
use DE_EPAGES::PayPalPlus::API::Constants qw ( SUPPORTED_CURRENCIES );
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::Exception;
use strict;

use constant ICELAND_KRONA_CURRENCY_CODE => 'ISK';

sub test {
    my $self = shift;

    $self->_prepareMocks();

    $self->test_onboarding_helpers_live();
    $self->test_onboarding_helpers_sandbox();

    $self->test_setDefaultNameForAllLanguages();
    $self->test_setDefaultNameForAllLanguages_missing_default_name();

    $self->test_getters_credentials_live();
    $self->test_getters_credentials_live_missing_values();

    $self->test_getters_credentials_sandbox();
    $self->test_getters_credentials_sandbox_missing_values();
    
    $self->test_payment_can_be_added_to_basket_with_supported_currency();
    $self->test_payment_can_not_be_added_to_basket_with_unsupported_currency();

    done_testing();
}

sub test_onboarding_helpers_live {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('LiveShop'),
        'PaymentType' => $self->memory->get('PaymentType')
    });

    # WHEN + THEN
    is($SUT->getOnboardingUrl(), 'http://paypal.com/merchantonboarding',
        "$SubTestName - Test if correct onboarding url was returned.");
    is($SUT->getTrackingId(), 'live-shop-guid',
        "$SubTestName - Test if shop GUID as tracking id was returned.");

    return;
}

sub test_onboarding_helpers_sandbox {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('TestShop'),
        'PaymentType' => $self->memory->get('PaymentType')
    });

    # WHEN + THEN
    is($SUT->getOnboardingUrl(), 'http://sandbox.paypal.com/merchantonboarding',
        "$SubTestName - Test if correct onboarding url was returned.");
    is($SUT->getTrackingId(), 'test-shop-guid',
        "$SubTestName - Test if shop GUID as tracking id was returned.");

    return;
}

sub test_getters_credentials_live {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('LiveShop'),
        'PaymentType' => $self->memory->get('PaymentType'),
        'MerchantClientId' => 'merchant-client-id'
    });

    # WHEN + THEN
    is($SUT->getPartnerAccountId(), 'paypal-account-id',
        "$SubTestName - Test if correct account id was returned.");
    is($SUT->getPartnerClientId(), 'partner-client-id',
        "$SubTestName - Test if correct partner client id was returned.");
    is($SUT->getPartnerClientSecret(), 'partner-client-secret',
        "$SubTestName - Test if correct partner client secret was returned.");
    is($SUT->getMerchantClientId(), 'merchant-client-id',
        "$SubTestName - Test if correct merchant client id was returned.");
    is($SUT->getPayPalPartnerAttributionId(), 'attribution-id',
        "$SubTestName - Test if correct partner client secret was returned.");
    is($SUT->isInTestMode(), 0,
        "$SubTestName - Test if test mode is false.");

    return;
}

sub test_getters_credentials_sandbox {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('TestShop'),
        'PaymentType' => $self->memory->get('PaymentType'),
        'MerchantClientId' => 'merchant-client-id-sandbox'
    });

    # WHEN + THEN
    is($SUT->getPartnerAccountId(), 'paypal-account-id',
        "$SubTestName - Test if correct account id was returned.");
    is($SUT->getPartnerClientId(), 'partner-client-id-sandbox',
        "$SubTestName - Test if correct partner client id was returned.");
    is($SUT->getPartnerClientSecret(), 'partner-client-secret-sandbox',
        "$SubTestName - Test if correct partner client secret was returned.");
    is($SUT->getMerchantClientId(), 'merchant-client-id-sandbox',
        "$SubTestName - Test if correct merchant client id was returned.");
    is($SUT->getPayPalPartnerAttributionId(), 'attribution-id',
        "$SubTestName - Test if correct partner client secret was returned.");
    is($SUT->isInTestMode(), 1,
        "$SubTestName - Test if test mode is false.");

    return;
}

sub test_getters_credentials_live_missing_values {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('LiveShop'),
        'PaymentType' => $self->memory->get('PaymentTypeIncompleted'),
        'MerchantClientId' => undef
    });

    # WHEN + THEN
    throws_ok(sub {
        $SUT->getPartnerAccountId();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing account id throws an error.");
    throws_ok(sub {
        $SUT->getPartnerClientId();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing partner client id throws an error.");
    throws_ok(sub {
        $SUT->getPartnerClientSecret();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing partner client secret throws an error.");
    throws_ok(sub {
        $SUT->getOnboardingUrl();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing onboarding url throws an error.");

    return;
}

sub test_getters_credentials_sandbox_missing_values {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('TestShop'),
        'PaymentType' => $self->memory->get('PaymentTypeIncompleted'),
        'MerchantClientId' => undef
    });

    # WHEN + THEN
    throws_ok(sub {
        $SUT->getPartnerAccountId();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing account id throws an error.");
    throws_ok(sub {
        $SUT->getPartnerClientId();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing partner client id throws an error.");
    throws_ok(sub {
        $SUT->getPartnerClientSecret();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing partner client secret throws an error.");
    throws_ok(sub {
        $SUT->getOnboardingUrl();
    }, qr/PAYPALPLUS_PBO_SETUP_INCOMPLETE/,
        "$SubTestName - Test if missing onboarding url throws an error.");

    return;
}

sub test_setDefaultNameForAllLanguages {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('MultiLanguageShop'),
        'PaymentType' => $self->memory->get('MultiLanguagePaymentType'),
    });

    is($SUT->get('Name', 1), 'InitialName', "$SubTestName - Test if name DE is initialized properly.");
    is($SUT->get('Name', 2), 'InitialName', "$SubTestName - Test if name EN is initialized properly.");

    # WHEN
    $SUT->_setDefaultNameForAllLanguages(); ## no critic (ProhibitPrivateSubroutineUse)

    # THEN
    is($SUT->get('Name', 1), 'DefaultNameDE', "$SubTestName - Test if name DE was set to default name when present.");
    is($SUT->get('Name', 2), 'DefaultNameEN', "$SubTestName - Test if name EN was set to default name when present.");
}

sub test_setDefaultNameForAllLanguages_missing_default_name {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({
        'Shop' => $self->memory->get('MultiLanguageShop'),
        'PaymentType' => $self->memory->get('MissingLanguagePaymentType'),
    });

    is($SUT->get('Name', 1), 'InitialName', "$SubTestName - Test if name DE is initialized properly.");
    is($SUT->get('Name', 2), 'InitialName', "$SubTestName - Test if name EN is initialized properly.");

    # WHEN
    $SUT->_setDefaultNameForAllLanguages(); ## no critic (ProhibitPrivateSubroutineUse)

    # THEN
    is($SUT->get('Name', 1), 'DefaultNameDE', "$SubTestName - Test if name DE was set to default name when present.");
    is($SUT->get('Name', 2), 'InitialName', "$SubTestName - Test if name EN was not set to default name when not present.");
}

sub test_payment_can_be_added_to_basket_with_supported_currency {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({});

    # WHEN
    my $FirstSupportedCurrencyCode = (SUPPORTED_CURRENCIES)[0];
    my $hVars = { 'CurrencyID' => $FirstSupportedCurrencyCode };

    # THEN
    ok($SUT->canAddToBasket($hVars), "$SubTestName - $FirstSupportedCurrencyCode currency code can be used with PayPalPlus");
}

sub test_payment_can_not_be_added_to_basket_with_unsupported_currency {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    # GIVEN
    my $SUT = $self->_buildSUT({});

    # WHEN
    my $hVars = { 'CurrencyID' => ICELAND_KRONA_CURRENCY_CODE };

    # THEN
    ok(!$SUT->canAddToBasket($hVars), "$SubTestName -Iceland Krona currency code can not be used with PayPalPlus");
}

#===============================================
# mocks
#===============================================
sub _prepareMocks {
    my $self = shift;

    my $MultiLanguageShop = $self->mockFactory->build('DE_EPAGES::Payment::API::Object::PaymentType', {
        'Languages' => [{'LanguageID' => 1},{'LanguageID' => 2}],
    }, 1);

    my $TestShop = $self->mockFactory->buildShop({
        'GUID' => 'test-shop-guid',
        'IsInternalTestShop' => 1,
    });

    my $LiveShop = $self->mockFactory->buildShop({
        'GUID' => 'live-shop-guid',
        'IsInternalTestShop' => 0,
    });

    my $PaymentType = $self->mockFactory->buildPaymentType({
        'ProviderPayPalAccountId' => 'paypal-account-id',
        'ProviderClientId' => 'partner-client-id',
        'ProviderClientSecret' => 'partner-client-secret',
        'PayPalPartnerAttributionId' => 'attribution-id',
        'ProviderClientIdSandbox' => 'partner-client-id-sandbox',
        'ProviderClientSecretSandbox' => 'partner-client-secret-sandbox',
        'MerchantOnboardingURLSandbox' => 'http://sandbox.paypal.com/merchantonboarding',
        'MerchantOnboardingURLLive' => 'http://paypal.com/merchantonboarding'
    });

    my $PaymentTypeIncomplete = $self->mockFactory->buildPaymentType({
        'ProviderPayPalAccountId' => undef,
        'ProviderClientId' => undef,
        'ProviderClientSecret' => undef,
        'ProviderClientIdSandbox' => undef,
        'ProviderClientSecretSandbox' => undef,
        'PayPalPartnerAttributionId' => undef
    });

    my $PaymentTypeMultiLanguage = $self->mockFactory->build('DE_EPAGES::Payment::API::Object::PaymentType', {}, 1);
    $PaymentTypeMultiLanguage->addLanguageDependentAttribute('PaymentMethodDefaultName', {
        1 => 'DefaultNameDE',
        2 => 'DefaultNameEN',
    });

    my $PaymentTypeMissingLanguage = $self->mockFactory->build('DE_EPAGES::Payment::API::Object::PaymentType', {}, 1);
    $PaymentTypeMissingLanguage->addLanguageDependentAttribute('PaymentMethodDefaultName', {
        1 => 'DefaultNameDE',
        2 => undef,
    });

    $self->memory->add('MultiLanguagePaymentType', $PaymentTypeMultiLanguage->getMockObject());
    $self->memory->add('MissingLanguagePaymentType', $PaymentTypeMissingLanguage->getMockObject());
    $self->memory->add('MultiLanguageShop', $MultiLanguageShop->getMockObject());
    $self->memory->add('PaymentType', $PaymentType->getMockObject());
    $self->memory->add('PaymentTypeIncompleted', $PaymentTypeIncomplete->getMockObject());
    $self->memory->add('TestShop', $TestShop->getMockObject());
    $self->memory->add('LiveShop', $LiveShop->getMockObject());

    return;
}

sub _buildSUT {
    my $self = shift;
    my ($hAttributes) = @_;

    my $PackageName = 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus';
    my $DefaultLanguageId = 1;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new($PackageName, $hAttributes, $DefaultLanguageId);
    $Mockify->addMockWithReturnValueAndParameterCheck('_callSuperCanAddToBasket', 1, ['hashref']);
    $Mockify->addLanguageDependentAttribute('Name', {
        1 => 'InitialName',
        2 => 'InitialName',
    });

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();