#==============================================================================
# package      DE_EPAGES::SEO::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::SEO::UI::Shop;
use base qw ( DE_EPAGES::Presentation::UI::Object );

use strict;
use LWP::UserAgent;
use HTTP::Request;
use URI::Escape qw( uri_escape_utf8 );
use DE_EPAGES::Content::DAL::Content         qw ( DAL_GetContentObjectDerivedClasses );
use DE_EPAGES::Core::API::File               qw ( DeleteFile GetFileContent GetFileInfo ExistsFile fsplit );
use DE_EPAGES::Core::API::Log                qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools          qw ( GetCurrentDateTime AddDuration );
use DE_EPAGES::Object::API::Factory          qw ( LoadClassByAlias );
use DE_EPAGES::Search::API::SearchDefinition qw ( GetSearchDefinition );
use DE_EPAGES::SEO::API::Constants           qw ( SEOCOCKPIT_NUM_UNTESTED SEOCOCKPIT_NUM_WORST SITEMAP_XML_URLS );
use DE_EPAGES::SEO::API::SEO                 qw ( GetShopSEOQualityValues CalculateShopSEOQuality );
use DE_EPAGES::SEO::API::XMLSitemap          qw ( ExportXMLSitemap );
use DE_EPAGES::SEO::DAL::SEO                 qw ( DAL_GetMeanContentObjectSEOQuality );
use DE_EPAGES::Trigger::API::Trigger         qw ( TriggerHook );

#ERRORDEFINITION
sub ErrorDefinition () {
    return {
        'FileRenameFails' => 'Can\'t rename file (#Source -> #Target) because #Code.',
    }
}
#ERRORDEFINITION END

sub View {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $PathFileName = $Shop->get('SitemapPathFileName');
    $Servlet->vars('SitemapFileInfo', GetFileInfo($PathFileName));
}

sub ViewSEOCockpit {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $ShopID = $Shop->id;
    my $ContentObjectClass = LoadClassByAlias('ContentObject');

    my %LanguageAvg;
    my $AvgObjectQuality = 0;
    my @ShopLanguagIDs = @{ $Shop->languages };
    foreach my $LanguageID (@ShopLanguagIDs) {
        $LanguageAvg{$LanguageID} = DAL_GetMeanContentObjectSEOQuality($ShopID, $LanguageID);
        $AvgObjectQuality += $LanguageAvg{$LanguageID};
    }
    $AvgObjectQuality = $AvgObjectQuality / scalar(@ShopLanguagIDs);

    my $hValues = {
        'WorstObjects' => _GetWorstObjects($Servlet, $ShopID),
        'UntestedObjects' => _GetUntestedObjects($Servlet, $ShopID, $ContentObjectClass),
    };

    # get general data for the shop
    my $hShopValues = GetShopSEOQualityValues($Shop);

    # merge
    $hValues = {
        %$hValues,
        %$hShopValues,
    };

    # fill in data from other cartridges
    TriggerHook('SEOCockpit', {
        'Shop'          => $Shop,
        'Values'        => $hValues,
    });

    ## no critic qw(ProhibitMagicNumbers)

    # calculate final shop quality value
    my $ShopQuality = CalculateShopSEOQuality($hValues);
    $ShopQuality = 0 if not defined $ShopQuality;
    $hValues->{'AvgObjectQuality'} = $AvgObjectQuality;
    $hValues->{'AvgObjectQualityPerLanguage'} = \%LanguageAvg;
    $hValues->{'ShopQuality'} = $ShopQuality;
    my $ShopQualityFraction = 0.4 * $ShopQuality;
    $hValues->{'MeanSEOQuality'} = 0.6 * $AvgObjectQuality + $ShopQualityFraction;

    foreach my $LanguageID (@ShopLanguagIDs) {
        $hValues->{'MeanSEOQualityPerLanguage'}->{$LanguageID} = 0.6 * $LanguageAvg{$LanguageID} + $ShopQualityFraction;
    }
    $Servlet->vars('ResponseContent', $hValues);

    ## critic

    return $self->SUPER::View($Servlet);
}

sub ViewSEOCheckWidget {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->ViewSEOCockpit($Servlet);
}

sub _GetWorstObjects {
    my ($Servlet, $ShopID) = @_;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ContentObject'), 'MBO-SEOQuality-Worst');
    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'OrderBy' => 'AttributeValue',
        'OrderDesc' => 0,
        'Parameters' => {
            'Shop.ID'               => $ShopID,
            'IsVisible'             => 1,
        },
        'PageSize' => SEOCOCKPIT_NUM_WORST,
    );
    my $aWorstObjects = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options)->page;
    return $aWorstObjects;
}

sub _CanDoSEOCheck {
    my ($Shop, $ClassID) = @_;

    my $Class = $Shop->loadObject($ClassID, 'Object', {'AllowSystemSite' => 1});
    if($Class->existsAttribute('CanDoSEOCheck')) {
        my $DefaultValue = $Class->attribute('CanDoSEOCheck')->get('DefaultValue');
        return $DefaultValue if defined $DefaultValue;
    }
    # by default all content objects can do the SEO check. Those which cannot
    # need to set their classes' CanDoSEOCheck attribute to 0.
    return 1;
}

sub _GetUntestedObjects {
    my ($Servlet, $ShopID, $ContentObjectClass) = @_;

    my @ClassIDs = (
        $ContentObjectClass->id,
        @{DAL_GetContentObjectDerivedClasses($ShopID, $ContentObjectClass->id)}
    );
    my $Shop = $Servlet->object()->getSite();
    my @SEOClassIDs = grep { _CanDoSEOCheck($Shop, $_) } @ClassIDs;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('ContentObject'), 'MBO-SEOQuality-Untested');
    my %Options = (
        'Servlet' => $Servlet,
        'SearchDefinition' => $SearchDefinition,
        'OrderDesc' => 1,
        'OrderBy' => 'CreationDate',
        'Parameters' => {
            'Shop.ID'               => $ShopID,
            'IsVisible'             => 1,
            'CanDoClassIDs'         => \@SEOClassIDs,
            'FilterSEOQualityIDs'   => 1,
        },
        'PageSize' => SEOCOCKPIT_NUM_UNTESTED,
    );

    TriggerHook('SEOCockpitSearchUntested', {
        'Options'      => \%Options,
    });

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    my $Page = $Search->page;
    return $Page;
}

sub ViewSitemap {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_HandleViewSitemap($Servlet, 'SitemapPathFileName');
}

sub ViewMobileSitemap {
    my $self = shift;
    my ($Servlet) = @_;

    return $self->_HandleViewSitemap($Servlet, 'MobileSitemapPathFileName');
}

sub _HandleViewSitemap {
    my $self = shift;
    my($Servlet, $AttributeName) = @_;
    my $Shop = $Servlet->object;

    return $self->_ViewNoSitemap( $Servlet ) unless $self->_ExportAllowed($Shop);

    my $PathFileName = $Shop->get($AttributeName);

    my $CheckResult = $self->_checkSitemap($Shop, $Servlet, $PathFileName);
    if($CheckResult) {
        return $self->_ViewSitemap( $Servlet, $PathFileName ); # display sitemap
    } else {
        return $self->_ViewNoSitemap( $Servlet ); # display 404 error
    }
}

sub ExportSitemap {
    my $self = shift;
    my ($Servlet) = @_;

    ExportXMLSitemap($Servlet->object);
}

sub _checkSitemap {
    my $self = shift;
    my ($Shop, $Servlet, $PathFileName) = @_;

    my $FileExists = ExistsFile($PathFileName);
    my $CurrentTime = GetCurrentDateTime();

    # check if the file is currently being created
    my $TempFileName = "$PathFileName.tmp";
    if( ExistsFile($TempFileName) ) {
        my $TempFileTime = GetFileInfo($TempFileName)->{'LastChange'};
        if( AddDuration($TempFileTime, 'days', 1) < $CurrentTime ) {
            # temp file is too old, probably an error occured during export
            DeleteFile($TempFileName);
        }
    }

    # wait until the site map has been written by another process
    if( ExistsFile($TempFileName) ) {
        return $FileExists ? 1 : 0;
    }

    # check if file exists and is older than one day
    my $FileTime = $FileExists ? GetFileInfo($PathFileName)->{'LastChange'} : undef;
    my $LastLogin = $Shop->get('LastMerchantLogin');

    # check that the merchant has logged on after the file has been written
    my $HasLoggedOn = (defined $LastLogin && defined $FileTime && $LastLogin > $FileTime) ? 1 : 0;

    if( !$FileExists ) {
        ExportXMLSitemap($Shop);
    } elsif( $HasLoggedOn ) {
        # Mark the shop as dirty, the 24h scheduler then recreates the sitemap xmls
        $Shop->set( { 'IsSitemapsOutdated' => 1 } );
    }
    return 1;
}

sub RegisterSitemap {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    if ($self->_ExportAllowed($Shop)) {
        my $ProtocolAndServer;
        if ($Shop->get('HasSSLCertificate')) {
            $ProtocolAndServer = $Shop->get('ProtocolAndServerSSL');
        } else {
            $ProtocolAndServer = $Shop->get('ProtocolAndServer');
        }

        my @SitemapFilesPrivatePath = (
            $Shop->get('SitemapPathFileName')
        );
        if ($Shop->get('UseMobileSF')) {
            push(@SitemapFilesPrivatePath, $Shop->get('MobileSitemapPathFileName'));
        }

        my @SitemapUrls;
        my $AllSitemapsExist = 1;
        foreach my $SitemapFilePrivatePath ( @SitemapFilesPrivatePath ) {
            $AllSitemapsExist = 0 unless ExistsFile($SitemapFilePrivatePath);
            my (undef, $SitemapFile, $SitemapFileExt) = fsplit($SitemapFilePrivatePath);
            $SitemapFile .= $SitemapFileExt;

            my $SitemapURL = $ProtocolAndServer.'/'.$SitemapFile;
            $SitemapURL = uri_escape_utf8( $SitemapURL );
            $SitemapURL =~ s/%2F/\//ig;
            push(@SitemapUrls, $SitemapURL);
        }

        if(!$AllSitemapsExist) {
            ExportXMLSitemap($Shop);
        }

        my $AllRegisterSuccess = 1;
        my $ua = LWP::UserAgent->new;
        foreach my $SitemapURL ( @SitemapUrls ) {
            foreach my $URL (values %{SITEMAP_XML_URLS()}) {
                my $RegisterSuccess = _registerSearchEngine($ua, $URL . $SitemapURL);
                $Servlet->form->addFormError({
                    'Name'    => 'URL',
                    'Reason'  => 'RegistrationMiss',
                    'Form'    => 'RegisterSitemap',
                    'URL'     => $URL . $SitemapURL,
                }) unless $RegisterSuccess;
                $AllRegisterSuccess = $AllRegisterSuccess && $RegisterSuccess;
            }
        }
        if ($AllRegisterSuccess) {
            $Shop->set({ 'SubmittedSitemapDomain' => $ProtocolAndServer });
            $Servlet->vars('RegistrationSuccess', 1);
        }
    } else {
        $Servlet->params('Error', {
            'Reason' => 'PermissionFailed',
            'Name'   => 'PermissionFailed',
        });
    }
    $Servlet->form->executeFormError;
}

sub _registerSearchEngine {
    my ($ua, $URL) = @_;
    # register xml for search engine
    my $req = HTTP::Request->new(GET=>$URL);
    $req->content_type('application/x-www-form-urlencoded');
    LogDebug('Request', $req);
    my $res = $ua->request($req);
    LogDebug('Response', $res);
    return $res->is_success ? 1 : 0;
}

# returns an existing Sitemap file
sub _ViewSitemap{
    my $self = shift;
    my ($Servlet, $PathFileName) = @_;

    my $rContent= GetFileContent($PathFileName, {'binary'=>1});
    my $Response = $Servlet->response;
    $Response->content( $$rContent);
    $Response->header( 'Content-Type' => 'text/xml' );
    $Servlet->vars('ExitAfterEvent', 1);
    return;
}

sub _ViewNoSitemap {
    my $self = shift;
    my ($Servlet) = @_;

    my $Response = $Servlet->response;
    $Response->content('No Sitemap available.');
    $Response->code('404');
    $Response->header( 'Content-Type' => 'text/plain' );
    $Servlet->vars('ExitAfterEvent', 1);

    return;
}

sub _ExportAllowed {
    my $self = shift;
    my ($Shop) = @_;

    return 0 if( $Shop->get('ClosedByMerchant') || $Shop->get('ClosedOrClosedTempByProvider') || $Shop->get('LoginRequired') );
    return 1 if $Shop->get('DomainName');
    return 0;
}

1;
