#========================================================================================
# package      DE_EPAGES::Shop::API::Attributes::Shop_Ext
# state        private
# description  Implements the attribute "Features" of the "Shop" class.
#========================================================================================
package DE_EPAGES::Shop::API::Attributes::Shop_Ext;
use base DE_EPAGES::Object::API::BaseAttribute;

use strict;
use feature 'switch';
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Core::API::File      qw( GetFileInfo GetFileNames GetFileContentLines ExistsDirectory);
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime AddDuration GetLocalTimeZone );
use DE_EPAGES::Database::API::Connection qw(GetCurrentDBHandle);
use DE_EPAGES::Shop::API::Constants qw( MAXIMUM_FEATURE_VALUES_FOR_UNITY_MIGRATION );
use DE_EPAGES::Shop::API::FeatureMaxValueHash;
use DE_EPAGES::Shop::API::FeatureCurrentValueHash;
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw ( ReplaceTLEs );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DateTime::Duration;

sub getAttributes {
    my $self = shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my %Values;
    foreach my $Name (@$aNames) {
        given ($Name) {
            when('GrantServiceAccess') {
                my $GrantServiceAccessUntil = $Object->get('GrantServiceAccessUntil');
                if ( not defined $GrantServiceAccessUntil ) {
                    $Values{$Name} = 0;
                }
                else {
                    my $GrantServiceAccessUntilTemp = $GrantServiceAccessUntil->clone;
                    $GrantServiceAccessUntilTemp->set_time_zone('floating');
                    $GrantServiceAccessUntilTemp->set_time_zone(LoadRootObject()->get('TimeZone'));
                    $GrantServiceAccessUntilTemp = AddDuration($GrantServiceAccessUntilTemp, 'days', '1');
                    $GrantServiceAccessUntilTemp->set_time_zone(GetLocalTimeZone());
                    $Values{$Name} = $GrantServiceAccessUntilTemp > GetCurrentDateTime();
                }
            }
            when('FeatureMaxValue') {
                my %h;
                tie %h, 'DE_EPAGES::Shop::API::FeatureMaxValueHash', $Object;
                $Values{$Name} = \%h;
            }
            when('FeatureCurrentValue') {
                my %h;
                tie %h, 'DE_EPAGES::Shop::API::FeatureCurrentValueHash', $Object;
                $Values{$Name} = \%h;
            }
            when('FeatureValues') {
                $Values{$Name} = [
                    grep { $_->{'IsVisible'} or $_->{'MaxValue'} or $_->{'CurrentMaxValue'} }
                    map {
                        {
                            Feature => $_,
                            IsVisible => $Object->featureIsVisible($_),
                            MaxValue => $Object->featureMaxValue($_),
                            CurrentValue => $Object->featureCurrentValue($_),
                        }
                    }
                    @{ LoadRootObject()->folder('Features')->children }
                ]
            }
            when('MediaFiles') {
                my $Directory = $Object->get('PublicPath').'/MediaGallery';
                $Values{$Name} = ExistsDirectory($Directory)
                    ? [ sort { lc($a->{'FileName'}) cmp lc($b->{'FileName'}) }
                        map { GetFileInfo($_) }
                        @{GetFileNames($Directory)}
                      ]
                    : [];
                foreach my $PublicFile (@{$Values{$Name}}) {
                    $PublicFile->{'URLFileName'} = $PublicFile->{'FileName'} if $PublicFile->{'Extension'} eq '.url';
                    $PublicFile->{'FileName'} = GetFileContentLines($PublicFile->{'Name'},{'binary'=>1})->[0] if $PublicFile->{'Extension'} eq '.url';
                }
            }
            when( [qw(  ShopTypePromotionText ShopTypePromotionLink ShopTypePromotionLinkText
                        FeaturePromotionText FeaturePromotionLink
                        TrialShopText TrialShopLink MerchantWelcomeText
                        ClosedByProviderTextMBO ClosedByProviderTextSF)] )
            {
                $Values{$Name} = _GetShopTypeRefTexts($Object, $Name, $LanguageID);
            }
            when('UnassignedCountries') {
                my %ShopCountries = map { $_ => undef } @{$Object->countries};
                $Values{$Name} = [
                    grep { !exists $ShopCountries{ $_->{CountryID} } }
                    @{LoadRootObject()->get('Countries')}
                ];
            }
            when('NonStorefrontUserGroups') {
                my $Groups = $Object->child('Groups');
                $Values{$Name} = [
                    map { $Groups->child($_) }
                    grep { $Groups->existsChild($_) }
                    qw( Merchants ERPUsers )
                ];
            }
            when('StorefrontUserGroups') {
                my %NonStorefrontUserGroupIDs = map { $_->id => undef } @{$Object->get('NonStorefrontUserGroups')};
                $Values{$Name} = [
                    grep { !exists $NonStorefrontUserGroupIDs{ $_->id } }
                    @{$Object->child('Groups')->children}
                ];
            }
            when('ERPUserGroupID') {
                my $Groups = $Object->child('Groups');
                $Values{$Name} = $Groups->existsChild('ERPUsers')
                    ? $Groups->child('ERPUsers')->id
                    : $Groups->insertChild('Group', { 'Alias' => 'ERPUsers' })->id;
            }
            when('ClosedOrClosedTempByProvider') {
                $Values{$Name} = 0;
                $Values{$Name} = 1 if ($Object->get('ClosedByProvider') or $Object->get('ClosedByProviderTemporarily'));
            }
            when('RESTAPIURL') {
                my $WebServerScriptNamePart = $Object->get('WebServerScriptNamePart');
                if (defined $WebServerScriptNamePart) {
                    $Values{$Name} = $Object->get('ProtocolAndServerSSL') .
                                    '/rs/shops/' . $Object->get('WebServerScriptNamePart');
                }
            }
            when('CanShowShopSurvey') {
                $Values{$Name} = _CanShowShopSurvey($Object);
            }
            when('ActiveShopNotifications') {
                $Values{$Name} = [ grep { $_->get('IsActive') } @{$Object->folder('ShopNotifications')->children} ];
            }
            when('UnityDisplayUpsellBar') {
                my $ShopLanguageID = $Object->get('LanguageID');
                my $hInfoObject
                    = $Object->get(['ShopTypePromotionText',
                                    'ShopTypePromotionLink',
                                    'UnityMerchantClosedUpsellBar'],
                        $ShopLanguageID);
                if (   defined $hInfoObject->{'ShopTypePromotionText'}
                    || defined $hInfoObject->{'ShopTypePromotionLink'}) {
                    # text or link defined --> show upsell bar if not closed
                    unless (defined $hInfoObject->{'UnityMerchantClosedUpsellBar'}) {
                        # show upsell bar if it has not been closed so far
                        $Values{$Name} = 1;
                    } else {
                        # check if the upsell bar has been closed for more than four weeks
                        my $Duration = DateTime::Duration->new( weeks => 4 );
                        my $ShowAgainDate = $hInfoObject->{'UnityMerchantClosedUpsellBar'} + $Duration;
                        my $Now = GetCurrentDBHandle()->currentDateTime;
                        my $NowDate = $Now->clone->truncate( to => 'day' );

                        if ($ShowAgainDate < $NowDate) {
                            # show notification again if it has been closed for more than four weeks
                            $Values{$Name} = 1;
                        } else {
                            # notification closed
                            $Values{$Name} = 0;
                        }
                    }
                } else {
                    # do not show the upsell bar if neither the text or the link have been defined
                    $Values{$Name} = 0;
                }
            }
            when('IsUnityMigratable') {
                my $FeaturesPossibleForUnityMigration = 1;

                # check for the given feature restrictions
                my $FeatureCurrentValue = $Object->get('FeatureCurrentValue');
                # load feature restrictions from epages.conf
                my $MaximumFeatureValuesForUnityMigration = MAXIMUM_FEATURE_VALUES_FOR_UNITY_MIGRATION;
                my @RestrictedFeaturesForUnityMigration = keys %$MaximumFeatureValuesForUnityMigration;
                foreach my $Name (keys %$FeatureCurrentValue) {
                    next unless $Name ~~ @RestrictedFeaturesForUnityMigration;
                    # compare to currently used feature values
                    if ($FeatureCurrentValue->{$Name} > $MaximumFeatureValuesForUnityMigration->{$Name}) {
                        $FeaturesPossibleForUnityMigration = 0;
                        last;
                    }
                }

                # add possible restrictions from other cartridges
                my $hValues = { 'IsUnityMigratable' => 1 };
                if ($FeaturesPossibleForUnityMigration) {
                    TriggerHook('OBJ_IsUnityMigratable', { 'Object' => $Object, 'Values' => $hValues });
                }

                # set migration flag only if all conditions are fulfilled
                $Values{$Name} = $FeaturesPossibleForUnityMigration && $hValues->{'IsUnityMigratable'};
            }

            default {
                die "Invalid attribute: $Name";
            }
        }
    }

    return \%Values;
}

sub _CanShowShopSurvey {
    my ($Shop) = @_;

    my $hShopValues = $Shop->get(['CreationDate', 'ShopSurveyWaitForMerchant', 'InitialShopWizardStartup']);

    # test if the merchant has already reacted to the survey (Shop.ShopSurveyWaitForMerchant)
    return 0 if $hShopValues->{'ShopSurveyWaitForMerchant'} == 0;

    # test if the shop wizard was already running on the shop
    return 0 if $hShopValues->{'InitialShopWizardStartup'} == 1;

    # ask System attributes
    my $hSystemValues = $Shop->parent->parent->get(['ShopSurveyShopMinAge', 'ShopSurveyShopMaxAge', 'ShopSurveyIsActive']);

    # test if the survey is active on the system
    return 0 if $hSystemValues->{'ShopSurveyIsActive'} == 0;

    my $DateTime = GetCurrentDBHandle()->currentDateTime;

    # test the min age of the shop was reached. Do not diplay message for shops wich are younger than ShopSurveyShopMinAge days
    if ($hSystemValues->{'ShopSurveyShopMinAge'}) {
        my $DisplayDate = $hShopValues->{'CreationDate'} + DateTime::Duration->new('days' => $hSystemValues->{'ShopSurveyShopMinAge'});
        return 0 if $DateTime < $DisplayDate;
    }

    # test the max age of the shop was reached. Do not diplay message for shops wich are older than ShopSurveyShopMaxAge days
    if ($hSystemValues->{'ShopSurveyShopMaxAge'}) {
        my $DisplayDate = $hShopValues->{'CreationDate'} + DateTime::Duration->new('days' => $hSystemValues->{'ShopSurveyShopMaxAge'});
        return 0 if $DateTime > $DisplayDate;
    }

    return 1;
}

sub _GetShopTypeRefTexts {
    my ($Shop, $Name, $LanguageID) = @_;

    my $ShopTypeRef = $Shop->get('ShopTypeRef');
    return undef unless defined $ShopTypeRef;

    my $Value = ReplaceTLEs($ShopTypeRef->get($Name, $LanguageID), $Shop->tleHash);
    return $Value =~ /\S/ ? $Value : undef;
}

1;
