#========================================================================================
# package      DE_EPAGES::Shop::API::Object::ShopNotification
# base         DE_EPAGES::Object::API::Object::Object
# state        public
#----------------------------------------------------------------------------------------
# description  Class for shop notification
#========================================================================================
package DE_EPAGES::Shop::API::Object::ShopNotification;
use base DE_EPAGES::Object::API::Object::Object;

use strict;

use DE_EPAGES::Core::API::CSV qw ( ParseCSV );
use DE_EPAGES::Core::API::PerlTools qw (ElementOf);
use DE_EPAGES::Core::API::TestParameter qw( TestObject );
use DE_EPAGES::Shop::API::Constants qw(NOTIFICATION_POSITION_TOP);
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShopNotification->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShopNotification' }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification = DE_EPAGES::Shop::API::Object::ShopNotification->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShopNotification');
#               my $ShopNotification = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook
#               'OBJ_InsertShopNotification' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $ShopNotification | new object | object
# hook         OBJ_InsertShopNotification | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopNotificationID - shop notification id - int
#               </ul>
#========================================================================================
sub insert {
    my $Package = shift;
    my ($hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertShopNotification',
               {'Object' => $self, 'ShopNotificationID' => $self->id });
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook
#               'OBJ_DeleteShopNotification' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShopNotification | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopNotificationID - shop notification id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;
    TriggerHook('OBJ_DeleteShopNotification',
               {'Object' => $self, 'ShopNotificationID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->set($hValues);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShopNotification'
#               and 'OBJ_AfterUpdateShopNotification'.
#----------------------------------------------------------------------------------------
# input        $hValues | hash containing values to set | hash
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShopNotification | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopNotificationID - shop notification id - int
#                 <li>Values        - new values for object - ref.hash
#               </ul>
# hook         OBJ_AfterUpdateShopNotification | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopNotificationID - shop notification id - int
#                 <li>Values        - updated values       - ref.hash
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues) = @_;

    TriggerHook(    'OBJ_BeforeUpdateShopNotification',
        {'Object'=>$self, 'Values'=> $hValues, 'ShopNotificationID' => $self->id});
    $self->SUPER::set($hValues);

    TriggerHook('OBJ_AfterUpdateShopNotification',
        {'Object'=>$self, 'Values'=> $hValues, 'ShopNotificationID' => $self->id});

    return;
}

#========================================================================================
# function     addObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->addObject($Object);
#----------------------------------------------------------------------------------------
# description  This function add an object to the shop notification
#----------------------------------------------------------------------------------------
# input        $Object | object to set to the notification | object
#========================================================================================
sub addObject {
    my $self = shift;
    my ($Object) = @_;

    TestObject('Object',$Object);
    my $ShopNotificationString = $self->get('ShopNotificationObjectIDString');
    my $aShopNotificationIDs = [];
    $aShopNotificationIDs = ParseCSV($ShopNotificationString) 
                              if (defined $ShopNotificationString && $ShopNotificationString =~ /\S/);

    return if ElementOf($Object->id, @$aShopNotificationIDs);

    push @$aShopNotificationIDs,$Object->id;
    $self->set({'ShopNotificationObjectIDString' => _sjoin($aShopNotificationIDs) });

    # activate in case of a new object
    $self->activate();
}

#========================================================================================
# function     deleteObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->deleteObject($Object);
#----------------------------------------------------------------------------------------
# description  This function deletes an object from the shop notification
#----------------------------------------------------------------------------------------
# input        $Object | object to delete on the notification | object
#========================================================================================
sub deleteObject {
    my $self = shift;
    my ($Object) = @_;

    TestObject('Object',$Object);
    my $ObjectIDToRemove = $Object->id;

    my $ShopNotificationString = $self->get('ShopNotificationObjectIDString');
    my $aShopNotificationIDs = [];
    $aShopNotificationIDs = ParseCSV($ShopNotificationString) 
                              if (defined $ShopNotificationString && $ShopNotificationString =~ /\S/);

    return unless ElementOf($ObjectIDToRemove, @$aShopNotificationIDs);
    my $aNewIDs = [];
    @$aNewIDs = grep { $_ != $ObjectIDToRemove } @$aShopNotificationIDs;

    $self->set({'ShopNotificationObjectIDString' => _sjoin($aNewIDs) });
    # deactivate for empty list
    $self->deactivate() unless scalar @$aNewIDs;

}

#========================================================================================
# function     deleteAllObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->deleteAllObjects();
#----------------------------------------------------------------------------------------
# description  Delete all notification object ids
#========================================================================================
sub deleteAllObjects {
    my $self = shift;

    $self->set({'ShopNotificationObjectIDString' => undef });
    $self->deactivate();
}

#========================================================================================
# function     activate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->activate();
#----------------------------------------------------------------------------------------
# description  Set the notification as activated (IsActive=1)
#========================================================================================
sub activate {
    my $self = shift;

    $self->set({'IsActive' => 1});
    $self->setToFirstPosition if $self->get('IsPositioningAllowed');
}

#========================================================================================
# function     deactivate
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->deactivate();
#----------------------------------------------------------------------------------------
# description  Set the notification as deactivated (IsActive=0)
#========================================================================================
sub deactivate {
    my $self = shift;

    $self->set({'IsActive' => 0});
}

#========================================================================================
# function     setToFirstPosition
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopNotification->setToFirstPosition();
#----------------------------------------------------------------------------------------
# description  Changes a notification position to the 1st position.
#               Notifications with Position < NOTIFICATION_POSITION_TOP are the before
#========================================================================================
sub setToFirstPosition {
    my $self = shift;

    $self->set({ 'Position' => NOTIFICATION_POSITION_TOP });
    $self->parent->renumberChildren;
    return;
}

sub _sjoin {
    my ($aShopNotificationIDs) = @_;
    return undef unless scalar(@$aShopNotificationIDs);
    return scalar(@$aShopNotificationIDs) > 1 ? join(',', @$aShopNotificationIDs) : $aShopNotificationIDs->[0];
}

1;