#========================================================================================
# description      Count the number of shops that are eligible for the migration to
#                   Unity according to the given feature usage.
#========================================================================================

use strict;
use Getopt::Long;
use Time::HiRes qw (time);;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadObjectByPath );
use DE_EPAGES::Shop::API::Feature qw ( LoadFeatureByAlias );


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $PrintList;
    my $ShopType;

    GetOptions(
        'help'        => \$Help,
        'list'        => \$PrintList,
        'shoptype=s'  => \$ShopType,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            print "Unity-migratable shops: \n\n" if $PrintList;

            # construct SQL statement to get all non-Unity shops
            my (@WhereClauses, @Parameter);
            my $FeatureID = LoadFeatureByAlias('Unity')->id;
            push @WhereClauses, 'sf.featureid = ?';
            push @Parameter, $FeatureID;
            push @WhereClauses, 'sf.maxval = 0';

            # optionally restrict SQL statement to a given shoptype
            my $JoinClause = '';
            if (defined $ShopType) {
                my $ShopTypeRefID = LoadObjectByPath('/ShopTypeRefs/' . $ShopType)->id;
                $JoinClause = 'INNER JOIN shop s ON sf.shopid = s.shopid ';
                push @WhereClauses, 's.shoptyperefid = ?';
                push @Parameter, $ShopTypeRefID;
            }
            # join SQL statement
            ##no critic (ProhibitVariablesInSQLStatements)
            my $sql = 'SELECT sf.shopid FROM shopfeature sf '.
                $JoinClause.
                'WHERE ' . join(' AND ', @WhereClauses);

            # execute SQL
            my $dbi = GetCurrentDBHandle();
            my $aaRows = $dbi->execute($sql, \@Parameter, ['Integer']);
            my @ShopIDs = map { $_->[0] } @$aaRows;

            my $UnityMigratableShops = 0;
            foreach my $ShopID (@ShopIDs) {
                my $Shop = LoadObject($ShopID);
                # only consider shops with the correctly (restricted) feature set
                if ($Shop->get('IsUnityMigratable')) {
                    $UnityMigratableShops++;
                    print $Shop->alias . "\n" if $PrintList;
                }
            }
            print "\n" if ($PrintList);
            print "Count of Unity-migratable shops: " . $UnityMigratableShops . "\n\n";
        },
    );
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1;
}

sub usage {
    print <<END_USAGE;
Description:
    Count the number of shops that are eligible for the migration to
    Unity according to the given feature usage.

Usage:
    perl $0 [options] [flags] datafiles

options:
    -shoptype   alias of a specific shoptype to filter for (optional)
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -list       print a list of shop aliases additional to the count
    -help       show the command line options

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
