#========================================================================================
# package      DE_EPAGES::Shop::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete objects.
#========================================================================================
package DE_EPAGES::Shop::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Presentation::API::Object qw ( CollectJSONLayoutInfomation );
use DE_EPAGES::Shop::API::Constants qw ( EDITOR_JWT_LIFETIME );
use DE_EPAGES::Shop::API::JWT qw ( CreateShopJsonWebToken );
use DE_EPAGES::Shop::API::Widget qw (AddWidget);
use JSON;

sub JSONLayout {
    my $self = shift;
    my ($Servlet, @Parameter) = @_;

    my $Object = $Servlet->object;
    my $Shop = $Object->getSite();
    my $System = LoadRootObject();
    my @KnownPageTypes = $Servlet->form->value('KnownPageTypes') // ();
    my $hInfo = CollectJSONLayoutInfomation($Object, \@KnownPageTypes);

    if (exists $hInfo->{'TemplateTypes'}) {
        foreach my $Part (keys %{$hInfo->{'TemplateTypes'}}) {
            @{$hInfo->{'TemplateTypes'}{$Part}{'Variations'}} = grep {
                my $Feature = $System->loadObject($_->{'ID'}, 'TemplateType')->get('Feature');
                not defined $Feature or $Shop->featureMaxValue($Feature)
            } @{$hInfo->{'TemplateTypes'}{$Part}{'Variations'}};
        }
    }

    return $self->writeContentOutput($hInfo);
}


#========================================================================================
# function     JSONRemoveWidget
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONRemoveWidget( $Servlet );
#----------------------------------------------------------------------------------------
# description  Remove the current widget from the current active user
#               call action on the widget which should be deletet
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet  | object
#========================================================================================
sub JSONRemoveWidget {
    my $self = shift;
    my ($Servlet) = @_;

    my $Widget = $Servlet->object;
    Error(
        'CANNOT_REMOVE_PERMANENT_WIDGET',
        {
            'ID' => $Widget->id,
            'Alias' => $Widget->alias,
        }
    ) if $Widget->get('IsPermanent');

    $Widget->delete;
}

#========================================================================================
# function     JSONAddWidget
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONAddWidget( $Servlet );
#----------------------------------------------------------------------------------------
# description  add a widget of the given type to the current active user
#               send the id of the widget type as parameter WidgetTypeId
#               WidgetTypeId = 1234
#----------------------------------------------------------------------------------------
# input        $WidgetTypeId | id of the widget type which should be created  | string
#========================================================================================
sub JSONAddWidget {
    my $self = shift;
    my ($Servlet) = @_;

    my $User = $Servlet->object;
    my $WidgetTypeID = $Servlet->form->value('WidgetTypeId');

    my $WidgetType = LoadRootObject()->loadObject($WidgetTypeID, 'WidgetType');
    my $newWidget = AddWidget($User, $WidgetType);

    $self->writeContentOutput({'widgetId' => $newWidget->id});
}

#========================================================================================
# function     JSONSaveWidgetPositions
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONSaveWidgetPositions( $Servlet );
#----------------------------------------------------------------------------------------
# description  save the positions of the dashboard widgets
#               send a String with the comma seperated ObjectIDs as parameters in this form
#               Positions = 0815;1234;56787
#----------------------------------------------------------------------------------------
# input        $Positions | string of comma seperated ObjectIDs| string
#========================================================================================
sub JSONSaveWidgetPositions {
    my $self = shift;
    my ($Servlet) = @_;
    my $User = $Servlet->object;
    my $PositionsString = $Servlet->form->value('Positions');

    my @PositionsArray = grep { defined $_ } split(/[;,]/, $PositionsString);

    my $PositionIndex = 1;
    foreach my $WidgetID(@PositionsArray){

        my $Widget = $User->getSite()->loadObject($WidgetID, 'Widget');
        if($Widget->className eq 'Widget'){
            die 'Invalid WidgetID' unless $Widget->get('User')->id == $User->id;
            $Widget->set({ 'Position' => $PositionIndex });
            $PositionIndex++;
        }
    }
    $User->child('MBOWidgets')->renumberChildren;
    return;
}

#========================================================================================
# function     JSONCreateJWT
# state        public
#----------------------------------------------------------------------------------------
# syntax       JSONCreateJWT($Servlet);
#----------------------------------------------------------------------------------------
# description  Returns a newly created shop JSON web token,
#               using <function DE_EPAGES::Design::API::CreateShopJsonWebToken>.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
# return       $JWT | hash with JSON web token | hash
#========================================================================================
sub JSONCreateJWT {
    my $self = shift;
    my ($Servlet) = @_;

    # create a JWT for the editor with a lifetime of 3 minutes
    return $Servlet->writeContentOutput({'jwt' => CreateShopJsonWebToken(EDITOR_JWT_LIFETIME, $Servlet->object->getSite(), 'editor')});
}

#========================================================================================
# function     JSONMarkShopNotificationInactive
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONMarkShopNotificationInactive( $Servlet );
#----------------------------------------------------------------------------------------
# description  marks a ShopNotification as inactive
#========================================================================================
sub JSONMarkShopNotificationInactive {
    my $self = shift;
    my ($Servlet) = @_;
    my $ShopNotification = $Servlet->object;

    $ShopNotification->deactivate();

    return;
}

#========================================================================================
# function     JSONUnitySaveRegion
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONUnitySaveRegion( $Servlet );
#----------------------------------------------------------------------------------------
# description  Saves the alias and countries for a region and checks for errors
#========================================================================================
sub JSONUnitySaveRegion {
    my $self = shift;
    my ($Servlet) = @_;

    my $Region = $Servlet->object;

    my $Alias = $Servlet->form->value('Alias');
    my $aCountryIDs = from_json($Servlet->form->value('CountryIDs'));

    my @hErrors;

    push @hErrors, {
        'reason' => 'AliasMissing'
    } unless $Alias;

    push @hErrors, {
        'reason' => 'NoCountrySelected'
    } unless scalar @$aCountryIDs;

    my $hJSONResponse = {
        'success' => JSON::true
    };

    unless (scalar @hErrors) {
        $Region->saveCountries($aCountryIDs);
        $self->SUPER::Save($Servlet);
    } else {
        $hJSONResponse = {
            'success' => JSON::false,
            'error' => \@hErrors
        };
    }

    return $self->writeContentOutput($hJSONResponse);
}

#========================================================================================
# function     JSONCloseUpsellBar
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->JSONCloseUpsellBar( $Servlet );
#----------------------------------------------------------------------------------------
# description  set the upsell bar to closed
#========================================================================================
sub JSONCloseUpsellBar {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $Now = GetCurrentDBHandle()->currentDateTime;
    my $NowDate = $Now->clone->truncate( to => 'day' );

    $Shop->set({'UnityMerchantClosedUpsellBar' => $NowDate});

    return;
}

1;
