#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::Object::RemoteShop
# state        public
# base         DE_EPAGES::ShopConfiguration::API::Object::RemoteObject
#----------------------------------------------------------------------------------------
# description  Provides access from the site database to a shop in a store database.
#----------------------------------------------------------------------------------------
# example      my $RemoteShop = DE_EPAGES::ShopConfiguration::API::Object::RemoteShop->new(ShopRef => $ShopRef);
#               # same as $RemoteShop = $ShopRef->remoteShop
#               $RemoteShop->applyFeatureValues;
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::Object::RemoteShop;
use base DE_EPAGES::ShopConfiguration::API::Object::RemoteObject;

use strict;
use feature 'switch';

use Sort::Versions qw(versioncmp);
use DE_EPAGES::Core::API::TestParameter qw( TestHash TestArray TestInteger TestObject TestString );
use DE_EPAGES::Core::API::Error qw( Error HandleError );
use DE_EPAGES::Core::API::File qw( fsplit );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::Password qw ( CryptPassword );
use DE_EPAGES::Object::API::Factory qw( JoinObjectPath LoadRootObject );
use DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient;
use DE_EPAGES::ShopConfiguration::API::StoreCartridge qw( GetStoreCartridgeVersion );
use DE_EPAGES::XML::API::DateTimeFormatXML;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NotProviderStore' => 'The store #Store is not assigned to provider #Provider',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RemoteShop = DE_EPAGES::ShopConfiguration::API::Object::RemoteShop->new(
#                   ShopRef => $ShopRef,
#                   ShopID => $ShopID
#               );
#----------------------------------------------------------------------------------------
# description  Creates a RemoteShop object from a ShopRef object.
#----------------------------------------------------------------------------------------
# input        $ShopRef | shop reference object | object
# input        $ShopID | (optional) object id within the store database | integer
# return       $RemoteShop | remote shop object | object
#========================================================================================
sub new {
    my $class = shift;
    my %options = @_;
    TestObject('ShopRef', $options{'ShopRef'});

    return bless \%options, $class;
}

#========================================================================================
# function     shopRef
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopRef = $self->shopRef;
#----------------------------------------------------------------------------------------
# description  Returns the ShopRef object from the site database.
#----------------------------------------------------------------------------------------
# return       $ShopRef | shop reference | object
#========================================================================================
sub shopRef {
    return shift->{ShopRef};
}

#========================================================================================
# function     store
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Store = $self->store;
#----------------------------------------------------------------------------------------
# description  Returns the store database object where the shop data is stored.
#----------------------------------------------------------------------------------------
# return       $Store | Store object or undef if the shop is deleted | object
#========================================================================================
sub store {
    return shift->shopRef->get('Store');
}

#========================================================================================
# function     shopID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopID = $self->shopID;
#----------------------------------------------------------------------------------------
# description  Returns the object id of the shop within the store database.
#----------------------------------------------------------------------------------------
# return       $ShopID | shop object id | integer
#========================================================================================
sub shopID {
    my $self = shift;

    $self->{ShopID} = $self->shopRef->get('StoreObjectID') unless defined $self->{ShopID};

    return $self->{ShopID};
}

sub id {
    return shift->shopID;
}

#========================================================================================
# function     shopService
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopService = $self->shopService;
# syntax       $ShopService = $self->shopService( $Store );
#----------------------------------------------------------------------------------------
# description  Returns a web service client of the ShopService.
#----------------------------------------------------------------------------------------
# input        $Store | (optional) Store object | object
# return       $ShopService | web service client | object
#========================================================================================
sub shopService {
    my $self = shift;
    my ($Store) = @_;
    $Store //= $self->store;

    my $ShopServiceURI = $self->hasMetaService
      ? $self->metaService->getLatestWebServiceURI('ShopService')->result
      : 'urn://epages.de/WebService/ShopService/2004/08';

    my $soap = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri'   => $ShopServiceURI
    );

    return $soap;
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RemoteShop->create({
#                   Store => $Store,
#                   ShopAlias => $ShopAlias,
#                   ImportFiles => $aImportFiles,
#               });
#----------------------------------------------------------------------------------------
# description  Creates a shop in the specified store database and adds a shop reference
#               in the site database.
#----------------------------------------------------------------------------------------
# input        $hShop | shop info to create a shop
#               <ul>
#                   <li>IsClosed | shop is initialized closed by provider | boolean
#                   <li>Store | store database (default ShopRef.Store) | object
#                   <li>ShopAlias | unique shop alias for the store database (default ShopRef.Alias) | object
#                   <li>aImportFiles | list of file names to be imported (default ShopRef.ShopType.ShopImportFiles)- ref.array.string
#                   <li>PublicFilesDir | directory on the application server where image files
#                       for the new shop are located. These files are copied to the
#                       shop directory. (optional) | string</li>
#               </ul> | ref.hash
# return       $RemoteShop | remote shop | object
#========================================================================================
sub create {
    my $self = shift;
    my ($hOptions) = @_;
    $hOptions = {} unless defined $hOptions;
    TestHash('hOptions', $hOptions);

    my $ShopRef = $self->shopRef;

    my $Store = $hOptions->{Store};
    $Store = $ShopRef->get('Store') unless defined $Store;

    my $Provider = $ShopRef->get('Provider');
    Error('NotProviderStore', {
        Provider => $Provider->alias,
        Store => $Store->alias
    }) unless $Provider->hasStore($Store);

    my $ShopAlias = $hOptions->{ShopAlias};
    $ShopAlias = $ShopRef->alias           unless defined $ShopAlias;
    TestString('ShopAlias', $ShopAlias);

    my $aImportFiles = $hOptions->{ImportFiles};
    unless (defined $aImportFiles and scalar @$aImportFiles) {
        my $Path = LoadRootObject()->get('PrivatePath').'/ShopImport/';
        $aImportFiles = [ map { $Path . $_ } @{$ShopRef->get('ShopType')->get('ShopImportFiles')} ];
    }
    TestArray('ImportFiles', $aImportFiles);

    my $PublicFilesDir = $hOptions->{'PublicFilesDir'};
    unless( defined $PublicFilesDir ) {
        $PublicFilesDir = $ShopRef->get('ShopType')->get('PublicFilesDirectory');
    }

    # hidden feature: expand environment variables in PublicFilesDirectory
    if( defined $PublicFilesDir ) {
        $PublicFilesDir =~ s/\$ENV\{(\w+)\}/$ENV{$1}/ge;
    }

    my $IsClosed = $hOptions->{IsClosed} ? 1 : 0;

    # create shop in remote database by web service
    my $ShopID;

    eval {

        # temporary close shop to make sure that ShopRef and Shop both closed
        $ShopRef->set({
            IsClosed => 1,
            Store => $Store,
        });

        # create the Shop object
        GetLog()->debug( "insert Shop $ShopAlias" );

        my $RemoteShopTypeRef = $ShopRef->get('ShopType')->createRemoteShopTypeRef({'Store' => $Store });

        if( versioncmp( GetStoreCartridgeVersion( $Store, 'DE_EPAGES::Shop' ), '6.1.1' ) <= 0 ) {
            # up to ePages 6.0.7
            GetLog()->debug('using RemoteObject::insertChild');

            my $hShopInfo = {
                'Alias' => $ShopAlias,
                'ClosedByProvider' => 1,
                'ShopTypeRef' => $RemoteShopTypeRef->id,
            };
            my $FactoryService = $self->factoryService( $Store );
            my $RemoteShopClass = DE_EPAGES::ShopConfiguration::API::Object::RemoteObject->new(
                Store    => $Store,
                ObjectID => $FactoryService->classByAlias('Shop')->result,
            );
            my $RemoteShopsFolder = DE_EPAGES::ShopConfiguration::API::Object::RemoteObject->new(
                Store    => $Store,
                ObjectID => $FactoryService->objectByPath('/Shops')->result,
            );
            $ShopID = $RemoteShopsFolder->insertChild( $RemoteShopClass, $hShopInfo)->id;
        }
        else {

            # ePages 6.0.8 or higher
            GetLog()->debug('using ShopService::create');

            my $hShopInfo = {
                'Alias' => $ShopAlias,
                'ClosedByProvider' => 1,
                'ShopTypeRefID' => $RemoteShopTypeRef->id,
                'ProviderPath' => $Provider->get('Path'),
            };
            $ShopID = $self->shopService->create( $hShopInfo )->result;
        }

        $self->{'ShopID'} = $ShopID;
        GetLog()->debug( "ShopID=$ShopID" );

        # now the shop objects exists in the store database
        $ShopRef->set( {
            IsDeleted => 0,
            StoreObjectPath => '/' . JoinObjectPath([ 'Shops', $ShopAlias ]),
            StoreObjectID => $ShopID,
        } );

        # set the shop type and set feature values
        $self->applyFeatureValues;

        # import default data
        $self->importFiles( $aImportFiles );

        # copy images from demo shop to new shop
        if( $PublicFilesDir ) {
            $self->shopService->copyPublicFiles( $self->{'ShopID'},$PublicFilesDir);
        }

        # activate the shop reference
        GetLog()->debug( 'activate the shop reference ' . $ShopRef->alias ) unless $IsClosed;
        $ShopRef->set({ IsClosed => $IsClosed });
    };
    HandleError( OnError => sub {
        my ($Error) = @_;

        GetLog()->debug($Error->longMessage);

        given ($Error->code) {
            when (['UNITY_CONFIG_ERROR', 'UNITY_API_CALL_ERROR']) {
                Error($Error);
            }
            default {
                if (defined $ShopID) {
                    GetLog()->debug('delete shop after error');
                    $self->delete
                }
            }
        }

    });

    return;
}

#========================================================================================
# function     importFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->importFiles( $aImportFiles );
#----------------------------------------------------------------------------------------
# description  Imports data from XML files into the remote shop in the store database.
#----------------------------------------------------------------------------------------
# input        $aImportFiles | list of file names to be imported | ref.array.string
#========================================================================================
sub importFiles {
    my $self = shift;
    my ($aImportFiles) = @_;
    TestArray( 'aImportFiles', $aImportFiles );

    my $FactoryService = $self->factoryService;
    my $ObjectPath = $self->shopRef->get('StoreObjectPath');
    foreach my $FileName ( @$aImportFiles ) {
        GetLog()->debug( "importXML $FileName into $ObjectPath" );

        my $BaseName = (fsplit($FileName))[1];
        if( $BaseName =~ /^del_/ ) {
            $FactoryService->deleteXML(
                SOAP::Data->type('string'=>$FileName),
                SOAP::Data->type('string'=>$ObjectPath)
            );
        }
        else {
            $FactoryService->importXML(
                SOAP::Data->type('string'=>$FileName),
                SOAP::Data->type('string'=>$ObjectPath)
            );
        }
    }

    return;
}

#========================================================================================
# function     shopTransferService
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopTransferService = $self->shopTransferService;
# syntax       $ShopTransferService = $self->shopTransferService( $Store );
#----------------------------------------------------------------------------------------
# description  Returns a web service client of the ShopTransferService.
#----------------------------------------------------------------------------------------
# input        $Store | (optional) Store object | object
# return       $ShopTransferService | web service client | object
#========================================================================================
sub shopTransferService {
    my $self = shift;
    my ($Store) = @_;
    $Store //= $self->store;

    my $soap = DE_EPAGES::ShopConfiguration::API::StoreWebServiceClient->new(
        'store' => $Store,
        'uri' => 'urn://epages.de/WebService/ShopTransferService/2009/07',
    );

    return $soap;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->delete;
#----------------------------------------------------------------------------------------
# description  Deletes the remote shop
#========================================================================================
sub delete {
    my $self = shift;

    my $Store = $self->store;
    TestObject( 'Store', $Store );
    my $ShopID = $self->shopID;
    TestInteger( 'ShopID', $ShopID );

    # delete shop in remote database by web service
    GetLog()->debug("delete shop $ShopID using ObjectService");
    $self->objectService( $Store )->delete($ShopID);

    return;
}

#========================================================================================
# function     export
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->export;
#----------------------------------------------------------------------------------------
# description  Export the remote shop to the given path
#========================================================================================
sub export {
    my $self = shift;
    my ($ExportPath) = @_;

    my $Store = $self->store;
    TestObject( 'Store', $Store );
    my $ShopID = $self->shopID;
    TestInteger( 'ShopID', $ShopID );
    TestString( 'ExportPath', $ExportPath );

    GetLog()->debug("export shop $ShopID using ShopTransferService");
    $self->shopTransferService( $Store )->export($ShopID,$ExportPath);

    return;
}

#========================================================================================
# function     lastMerchantLogin
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LastMerchantLogin = $self->lastMerchantLogin();
#----------------------------------------------------------------------------------------
# description  Return the last login date and time by the Merchant in this Shop.
#----------------------------------------------------------------------------------------
# input        $LastMerchantLogin | the last login date and time | datetime
#========================================================================================
sub lastMerchantLogin {
    my $self = shift;

    my $LastMerchantLogin = $self->shopService->getLastMerchantLogin($self->shopID)->result;
    return unless $LastMerchantLogin;

    my $parser = DE_EPAGES::XML::API::DateTimeFormatXML->new();
    $LastMerchantLogin = $parser->parse_datetime($LastMerchantLogin);

    return $LastMerchantLogin;
}

#========================================================================================
# function     applyFeatureValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->applyFeatureValues;
#----------------------------------------------------------------------------------------
# description  Applies the currently allowed feature values to the shop. The
#               allowed values are determined by the function
#========================================================================================
sub applyFeatureValues {
    my $self = shift;

    my $ahFeatureValues = [
        map {
            {
                Feature  => $_->{FeatureRef}->alias,
                MaxValue => $_->{MaxValue},
                IsVisible=> $_->{IsVisible}
            }
        }
        @{ $self->shopRef->get('Features') }
    ];
    $self->shopService->setFeatureValues( $self->shopID, $ahFeatureValues );
}

#========================================================================================
# function     getShopFeatures
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahFeatureValues = $self->getShopFeatures;
#----------------------------------------------------------------------------------------
# description  get the current feature values from the shop. The
#               values are determined by the function
#               <function DE_EPAGES::Shop::API::WebService::ShopService::getFeatureValues>.
#----------------------------------------------------------------------------------------
# return       $ahFeatureValues | feature values | ref.array.hash
#========================================================================================
sub getShopFeatures {
    my $self = shift;

    return $self->shopService->getFeatureValues( $self->shopID )->result;
}

#========================================================================================
# function     getShopAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShopAddress = $self->getShopAddress;
#----------------------------------------------------------------------------------------
# description  get the address from the shop.
#----------------------------------------------------------------------------------------
# return       $hAddress | address values | ref.hash
#========================================================================================
sub getShopAddress {
    my $self = shift;
    my $result = $self->shopService->getShopAddress( $self->shopID )->result;
    return (defined $result and ref $result eq 'HASH') ? $result : undef;
}

#========================================================================================
# function     setShopAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setShopAddress(\@ShopAddress);
#----------------------------------------------------------------------------------------
# description  set the address of the shop.
#----------------------------------------------------------------------------------------
# input        $hAddress | address values | ref.hash
#========================================================================================
sub setShopAddress {
    my $self = shift;
    my ($hAddress) = @_;
    $self->shopService->setShopAddress( $self->shopID, $hAddress );
    return;
}

#========================================================================================
# function     getShopAttributeAliases
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShopAttributeAliases = $self->getShopAttributeAliases;
#----------------------------------------------------------------------------------------
# description  get the attributes aliases from the shop.
#----------------------------------------------------------------------------------------
# return       $hAttributeAliases | attribute aliases | ref.hash
#========================================================================================
sub getShopAttributeAliases {
    my $self = shift;
    my $result = $self->shopService->getShopAttributeAliases( $self->shopID )->result;
    return (defined $result and ref $result eq 'HASH') ? $result : undef;
}

#========================================================================================
# function     getShopMerchantEMails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aMerchantEMails = $self->getShopMerchantEMails;
#----------------------------------------------------------------------------------------
# description  get the merchant email addresses from the shop.
#----------------------------------------------------------------------------------------
# return       $aMerchantEMails | address values | ref.array.hash.string
#========================================================================================
sub getShopMerchantEMails {
    my $self = shift;
    return $self->shopService->getMerchantEMails( $self->shopID )->result;
}

#========================================================================================
# function     setShopMerchantEMails
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->setShopMerchantEMails(\@MerchantEMails);
#----------------------------------------------------------------------------------------
# description  set the merchant email addresses from the shop.
#----------------------------------------------------------------------------------------
# input        $aMerchantEMails | address values | ref.array.hash.string
#========================================================================================
sub setShopMerchantEMails {
    my $self = shift;
    my ($aMerchantEMails) = @_;
    $self->shopService->setMerchantEMails( $self->shopID, $aMerchantEMails );
    return;
}

#========================================================================================
# function     updateSuperMerchant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->updateSuperMerchant($hValues);
#----------------------------------------------------------------------------------------
# description  Sets login, email and password of the primary administrator.
#----------------------------------------------------------------------------------------
# input        $hValues | hash new values, all keys are optional:
#                <ul>
#                 <li>Alias | login | string</li>
#                 <li>Password | password (encrypted) | string</li>
#                 <li>EMail | email address | string</li>
#                 <li>Name | name | string</li>
#                 <li>LocaleID | locale id | string</li>
#                 <li>DeleteConfirmation | true if deletion should be confirmed | boolean</li>
#                 <li>IsLoginCookieAllowed | true if user automated login over cookies is enabled | boolean</li>
#                 <li>ShowFunctionHelp | true if function help is allowed | boolean</li>
#                </ul> | ref.hash.string
#========================================================================================
sub updateSuperMerchant {
    my $self = shift;
    my ($hValues) = @_;

    # encrypt the password; do not change the reference parameter $hValues
    my %Values = %$hValues;
    $Values{'Password'} = CryptPassword($Values{'Password'}) if defined $Values{'Password'};


    foreach my $i (qw( Alias Password EMail Name )){
        if(exists $Values{$i}) { $Values{$i} = SOAP::Data->type('string'=>$Values{$i}); }
    }

    $self->shopService->updateSuperMerchant( $self->shopID, \%Values );
    return;
}

#========================================================================================
# function     createSuperMerchantSession
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $self->createSuperMerchantSession();
#----------------------------------------------------------------------------------------
# description  Returns valid session and URL with session for the MBO logged in as supermerchant.
#               If the shop attribute "GrantServiceAccessUntil" is a date in the past
#               SessionId will be undef and Link to URL contains no session
#----------------------------------------------------------------------------------------
# input        $AlreadyGranted | true if access is allowed without merchant grant| boolean
# return       $hValues | hash with valid session and URL to MBO; keys:
#                       <ul>
#                       <li>SessionId | a valid SuperMerchant session | string</li>
#                       <li>WebUrlAdminSSL | Link to MBO without SessionId | string</li>
#                       <li>WebUrlAdminSSLWithSessionId | Link to MBO with session | string</li>
#                       </ul> | ref.hash
#========================================================================================
sub createSuperMerchantSession {
    my $self = shift;
    my ($AlreadyGranted) = @_;
    return $self->shopService->createSuperMerchantSession( $self->shopID, $AlreadyGranted )->result;
}

#========================================================================================
# function     getInfoSuperMerchant
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $self->getInfoSuperMerchant;
#----------------------------------------------------------------------------------------
# description  Returns login, email and other information of the primary administrator.
#----------------------------------------------------------------------------------------
# return       $hValues | merchant information hash:
#                 <ul>
#                   <li>Alias | login | string</li>
#                   <li>EMail | e-mail address | string</li>
#                   <li>LocaleID | locale id | string</li>
#                   <li>DeleteConfirmation | true if deletion should be confirmed | boolean</li>
#                   <li>IsLoginCookieAllowed | true if user automated login over cookies is enabled | boolean</li>
#                   <li>ShowFunctionHelp | true if function help is allowed | boolean</li>
#                 </ul> | ref.hash.string
#========================================================================================
sub getInfoSuperMerchant {
    my $self = shift;

    return $self->shopService->getInfoSuperMerchant( $self->shopID )->result;
}

#========================================================================================
# function     sendSuperMerchantNewPassword
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->sendSuperMerchantNewPassword($Alias);
#----------------------------------------------------------------------------------------
# description  Set password of the primary administrator, and sends him a email.
#----------------------------------------------------------------------------------------
# input        $Alias | super merchant login | string
#========================================================================================
sub sendSuperMerchantNewPassword {
    my $self = shift;
    my ($Alias) = @_;

    $self->shopService->sendSuperMerchantNewPassword( $self->shopID, $Alias );
    return;
}

#========================================================================================
# function     registerUnity
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->registerUnity;
#----------------------------------------------------------------------------------------
# description  Registers an Unity shop. Throws an error if somethings gone wrong.
#========================================================================================
sub registerUnity {
    my $self = shift;
    my $ShopRef = $self->shopRef;

    return unless $ShopRef->get('ShopType')->get('HasUnityEnabled');
    return unless $self->hasMetaService and $self->metaService->existsWebServiceMethod($self->shopService->uri, 'registerUnity')->result;

    my $UnityResult = $self->shopService->registerUnity($self->shopID)->result;

    if (defined ref $UnityResult and ref $UnityResult eq 'TError') {
        $ShopRef->set({'IsClosed' => 1});
        Error('UNITY_API_CALL_ERROR', {
            'Message' => $UnityResult->{'Message'}
        });
    } else {
        $ShopRef->set({'IsClosed' => 0});
    }
}

1;
