#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::Object::RemoteShopTypeRef
# state        public
# base         DE_EPAGES::ShopConfiguration::API::Object::RemoteObject
#----------------------------------------------------------------------------------------
# description  Provides access from the site database to a shoptyperef in a store database.
#----------------------------------------------------------------------------------------
# example      my $RemoteShopTypeRef = DE_EPAGES::ShopConfiguration::API::Object::RemoteShopTypeRef->new(ShopType => $ShopType);
#               # same as $RemoteShopTypeRef = $ShopType->RemoteShopTypeRef
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::Object::RemoteShopTypeRef;
use base DE_EPAGES::ShopConfiguration::API::Object::RemoteObject;

use strict;
use DE_EPAGES::Core::API::TestParameter qw( TestHash  TestObject  );
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( GetLog LogError );
use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem;
use DE_EPAGES::ShopConfiguration::API::StoreCartridge qw(GetStoreCartridgeVersion);

#ERRORDEFINITION
sub ErrorDefinition { return {
    'NotProviderStore' => 'The store #Store is not assigned to provider #Provider',
}}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RemoteShopTypeRef = DE_EPAGES::ShopConfiguration::API::Object::RemoteShopTypeRef->new(
#                   ShopType => $ShopType,
#               );
#----------------------------------------------------------------------------------------
# description  Creates a RemoteShopTypeRef object from a ShopType object.
#----------------------------------------------------------------------------------------
# input        $ShopType | shop type object | object
# return       $RemoteShopTypeRef | remote shop type ref object | object
#========================================================================================
sub new {
    my $class = shift;
    my %options = @_;
    TestObject('ShopType', $options{'ShopType'});

    return bless \%options, $class;
}

#========================================================================================
# function     shopType
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopType = $self->shopType;
#----------------------------------------------------------------------------------------
# description  Returns the ShopType object from the site database.
#----------------------------------------------------------------------------------------
# return       $ShopType | shop type | object
#========================================================================================
sub shopType {
    return shift->{ShopType};
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $RemoteShopTypeRef->create({
#                   Store => $Store,
#               });
#----------------------------------------------------------------------------------------
# description  Creates a shop type ref in the specified store database if it does not
#               yet exists. Otherwise returns the existing remote object.
#----------------------------------------------------------------------------------------
# input        $hShopTypeRef | shop type info to create a shop type ref
#               <ul>
#               </ul> | ref.hash
# return       $RemoteShopTypeRef | remote shop type ref | object
#========================================================================================
sub create {
    my $self = shift;
    my ($hOptions) = @_;
    TestHash('hOptions', $hOptions);

    my $Store = $hOptions->{'Store'};

    my $ShopType = $self->shopType;
    my $Provider = $ShopType->getSite();
    Error('NotProviderStore', {
        Provider => $Provider->alias,
        Store => $Store->alias
    }) unless $Provider->hasStore($Store);

    my $FactoryService     = $self->factoryService( $Store );
    my $ClassIDShopTypeRef = $FactoryService->classByAlias('ShopTypeRef')->result;

    my $RemoteClass = DE_EPAGES::ShopConfiguration::API::Object::RemoteObject->new(
                   'Store' => $Store,
                   'ObjectID' => $ClassIDShopTypeRef
               );

    my $RemoteSystem = DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem->new(
                   Store => $Store
               );
    my $ShopTypeRefsFolder = $RemoteSystem->folder('ShopTypeRefs');

    # create shop type ref in remote database by web service
    my $ShopTypeRefAlias = $ShopType->alias;
    my $ShopTypeRef;
    eval {
        if ( $ShopTypeRefsFolder->existsChild($ShopTypeRefAlias) ) {
            $ShopTypeRef = $ShopTypeRefsFolder->child($ShopTypeRefAlias)
        } else {
            # create the Shop type ref object
            GetLog()->debug( "insert Shop type ref $ShopTypeRefAlias" );
            my $hShopTypeRefInfo = {
                Alias => $ShopTypeRefAlias,
            };
            $ShopTypeRef = $ShopTypeRefsFolder->insertChild( $RemoteClass, $hShopTypeRefInfo);
            foreach my $hLanguage ( @{$ShopType->getSite()->get('Languages')} ) {
                my %Values;
                foreach my $Name ( qw( ShopTypePromotionText ShopTypePromotionLink
                                  ShopTypePromotionLinkText
                                  FeaturePromotionText FeaturePromotionLink
                                  TrialShopText TrialShopLink MerchantWelcomeText
                                  ClosedByProviderTextMBO ClosedByProviderTextSF )) {
                    $Values{ $Name } = $ShopType->get($Name, $hLanguage->{'LanguageID'});
                }

                # set some attributes only if store cartridge version > 6.2
                if (_RemoteStoreCartridgeVersion($Store) <= 6.200) {
                    delete $Values{'ClosedByProviderTextMBO'};
                    delete $Values{'ClosedByProviderTextSF'};
                }

                $ShopTypeRef->set( \%Values, $hLanguage->{'Code2'});
            }
        }
        GetLog()->debug( 'ShopTypeRefID= ' . $ShopTypeRef->id );
    };
    if( ExistsError() ) {
        my $Error = GetError();

        # ignore follow-up errors

        GetLog()->debug( 'delete shop type ref after error' );
        GetLog()->debug( $Error->longMessage );
        eval {
            GetLog()->debug( 'delete shop' );
            $ShopTypeRef->delete if defined $ShopTypeRef;
        };
        if (ExistsError()) {
            LogError( GetError() );
        }
        Error( $Error );
    }
    return $ShopTypeRef;
}

sub _update {
    my $self = shift;
    my ($Store, $hOptions) = @_;
    my $ShopTypeRefAlias = defined $hOptions->{'OldAlias'} ?  $hOptions->{'OldAlias'} : $self->shopType->alias;

    my $LanguageCode = defined $hOptions->{'LanguageID'} ? GetCodeByLanguageID($hOptions->{'LanguageID'}) : undef;

    my %Values = %$hOptions;
    delete $Values{'OldAlias'};
    delete $Values{'LanguageID'};

    return unless scalar(keys(%Values));

    my $RemoteSystem = DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem->new(Store => $Store);
    my $ShopTypeRefsFolder = $RemoteSystem->folder('ShopTypeRefs');
    if ($ShopTypeRefsFolder->existsChild( $ShopTypeRefAlias )) {
        my $ShopTypeRef = $ShopTypeRefsFolder->child($ShopTypeRefAlias);
        $ShopTypeRef->set( \%Values, $LanguageCode);
    } # else no update necessary
}


sub update {
    my $self = shift;
    my ($hOptions) = @_;
    TestHash('hOptions', $hOptions);

    if (exists $hOptions->{'Store'}) {
        my $Store = $hOptions->{'Store'};
        delete $hOptions->{'Store'};
        $self->_update($Store, $hOptions);
    } else {
        $self->_update($_, $hOptions) foreach
            grep { $_->get('Connection')->{'active'} }
            @{$self->shopType->get('Provider')->get('Stores')};
    }
    return;

}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->delete;
#----------------------------------------------------------------------------------------
# description  Deletes the shop type ref.
#========================================================================================
sub delete {
    my $self = shift;
    my ($hOptions) = @_;
    TestHash('hOptions', $hOptions);

    my $Store = $hOptions->{'Store'};

    if (defined $Store) {
        my $Alias = $hOptions->{'Alias'};
        my $RemoteSystem = DE_EPAGES::ShopConfiguration::API::Object::RemoteSystem->new(
                       Store => $Store
                   );
        my $ShopTypeRefsFolder = $RemoteSystem->existsChild('ShopTypeRefs') ? $RemoteSystem->child('ShopTypeRefs') : undef;
        $ShopTypeRefsFolder->child($Alias)->delete if defined $ShopTypeRefsFolder and $ShopTypeRefsFolder->existsChild( $Alias );
    };
    return;
}


# return the version of the remote store cartridge
sub _RemoteStoreCartridgeVersion {
    my $Store = shift;

    my $RemoteCoreCartridgeVersion = GetStoreCartridgeVersion($Store,'DE_EPAGES::Shop');
    my ($Main, $Sub, $Level, $Serial) = split(/\./, $RemoteCoreCartridgeVersion);
    $_ ||= 0  foreach ($Main, $Sub, $Level, $Serial);


    return "$Main.$Sub$Level$Serial";

}



1;
