#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::Object::ShopType
# state        public
# base         DE_EPAGES::ShopConfiguration::API::Object::FeatureCollection
# description  object interface for ShopType.
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::Object::ShopType;
use base DE_EPAGES::ShopConfiguration::API::Object::FeatureCollection;

use strict;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadObject );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::ShopConfiguration::API::Table::ShopTypeFeaturePack qw(
    ExistsShopTypeFeaturePack
    DeleteShopTypeFeaturePack
    InsertShopTypeFeaturePack
    GetPKeyShopTypeFeaturePacksByShopTypeID
);
use DE_EPAGES::ShopConfiguration::API::Object::RemoteShopTypeRef;

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ShopType->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ShopType' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ShopType->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopType = DE_EPAGES::ShopConfiguration::API::Object::ShopType->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShopType');
#               my $ShopType = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertShopType' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ShopType | new object | object
# hook         OBJ_InsertShopType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopTypeID    - shop tpye id         - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertShopType', {'Object' => $self, 'ShopTypeID' => $self->id });
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopType->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteShopType' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShopType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopTypeID    - shop tpye id         - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteShopType', {'Object' => $self, 'ShopTypeID' => $self->id });

    my $Alias= $self->alias;
    my $Stores =$self->getSite()->get('Stores');

    DeleteShopTypeFeaturePack($_) foreach @{GetPKeyShopTypeFeaturePacksByShopTypeID($self->id)};
    $self->SUPER::delete;

    foreach my $Store (@$Stores ) {
        next unless $Store->get('IsWebServiceAvailable');
        $self->deleteRemoteShopTypeRef({'Store' => $Store , 'Alias' => $Alias});
    }
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShopType->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateShopType' and 'OBJ_AfterUpdateShopType'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateShopType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopTypeID    - shop tpye id         - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateShopType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShopTypeID    - shop tpye id         - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateShopType', {'Object'=>$self, 'Values'=> $hValues, 'ShopTypeID' => $self->id, 'LanguageID' => $LanguageID});

    my %RemoteInfo;
    # set attribute for all texts for MBO by web service
    foreach my $Text (qw(MerchantWelcomeText TrialShopLink TrialShopText
                        FeaturePromotionLink FeaturePromotionText
                        ShopTypePromotionLink ShopTypePromotionText
                        ShopTypePromotionLinkText
                        ClosedByProviderTextMBO ClosedByProviderTextSF)) {
        if( exists $hValues->{$Text} ) {
            my $CurrentValue = $self->get($Text, $LanguageID);
            if ( (defined $CurrentValue and not defined $hValues->{$Text} ) or
                 (not defined $CurrentValue and defined $hValues->{$Text} ) or
                 (defined $CurrentValue and defined $hValues->{$Text} and $CurrentValue ne $hValues->{$Text} )
               ) {
                $RemoteInfo{$Text} = $hValues->{$Text};
            }
        }
    }

    if( exists $hValues->{'Alias'} ) {
        my $CurrentValue = $self->alias;
        if ($CurrentValue ne $hValues->{'Alias'} ) {
            $RemoteInfo{'OldAlias'} = $CurrentValue;
            $RemoteInfo{'Alias'} = $hValues->{'Alias'};
        }
    }
    if (scalar (keys %RemoteInfo) ) {
        $self->updateRemoteShopTypeRef({'LanguageID' => $LanguageID, %RemoteInfo });
    }
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShopType',  {'Object'=>$self, 'Values'=> $hValues, 'ShopTypeID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     exportXMLElements
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->exportXMLElements($Driver, $hAllAttributes);
#----------------------------------------------------------------------------------------
# description  Exports multilinguagal attribute value of this object to the xml SAX driver.
#----------------------------------------------------------------------------------------
# input        $Driver | export driver | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
#========================================================================================
sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;
    $self->SUPER::exportXMLElements($Driver, $allAttributes);
    $Driver->ShopTypeFeaturePacksByShopType();
}

#========================================================================================
# function     featurePacks
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aFeaturePacks = $ShopType->featurePacks;
# example      $ShopType->removeFeaturePack( $_ ) foreach @{ $ShopType->featurePacks };
#----------------------------------------------------------------------------------------
# description  Returns a list of feature packs that are available for this shop type.
#----------------------------------------------------------------------------------------
# return       $aFeaturePacks | list of feature pack objects | ref.array.object
#========================================================================================
sub featurePacks {
    my $self = shift;

    return [
        map { LoadObject( $_->{'FeaturePackID'} ) }
        @{ GetPKeyShopTypeFeaturePacksByShopTypeID( $self->id ) }
    ];
}

#========================================================================================
# function     addFeaturePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopType->addFeaturePack( $FeaturePack );
#----------------------------------------------------------------------------------------
# description  Makes a feature pack available for this shop type.
#----------------------------------------------------------------------------------------
# input        $FeaturePack | feature pack object | object
#========================================================================================
sub addFeaturePack {
    my $self = shift;
    my ($FeaturePack) = @_;

    InsertShopTypeFeaturePack( {
        ShopTypeID => $self->id,
        FeaturePackID => $FeaturePack->id
    } );
    return;
}

#========================================================================================
# function     removeFeaturePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopType->removeFeaturePack( $FeaturePack );
#----------------------------------------------------------------------------------------
# description  Removes a feature pack available for this shop type.
#----------------------------------------------------------------------------------------
# input        $FeaturePack | feature pack object | object
#========================================================================================
sub removeFeaturePack {
    my $self = shift;
    my ($FeaturePack) = @_;

    DeleteShopTypeFeaturePack( $self->id, $FeaturePack->id );
    return;
}

#========================================================================================
# function     hasFeaturePack
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $ShopType->hasFeaturePack( $FeaturePack );
# example      $ShopType->removeFeaturePack( $FeaturePack ) if $ShopType->hasFeaturePack( $FeaturePack );
#----------------------------------------------------------------------------------------
# description  Returns true if the feature pack is available for this shop type.
#----------------------------------------------------------------------------------------
# input        $FeaturePack | feature pack object | object
# return       $Exists | true if the feature pack is available | boolean
#========================================================================================
sub hasFeaturePack {
    my $self = shift;
    my ($FeaturePack) = @_;

    return ExistsShopTypeFeaturePack( $self->id, $FeaturePack->id );
}

#========================================================================================
# function     shops
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aShops = $ShopType->shops;
#----------------------------------------------------------------------------------------
# description  Returns all shops of this shop type.
#----------------------------------------------------------------------------------------
# input        $aShops | list of shop reference objects | ref.array.object
#========================================================================================
sub shops {
    my $self = shift;

    my $Class = LoadClassByAlias('ShopRef');
    return $Class->getObjects('ByShopTypeID', [$self->id]);
}

sub remoteShopTypeRef {
    my $self = shift;

    return DE_EPAGES::ShopConfiguration::API::Object::RemoteShopTypeRef->new(ShopType => $self);
}

sub createRemoteShopTypeRef {
    my $self = shift;
    my ($hOptions) = @_;

    return $self->remoteShopTypeRef->create($hOptions);
}

sub updateRemoteShopTypeRef {
    my $self = shift;
    my ($hOptions) = @_;

    return $self->remoteShopTypeRef->update($hOptions);
}

sub deleteRemoteShopTypeRef {
    my $self = shift;
    my ($hOptions) = @_;

    return $self->remoteShopTypeRef->delete($hOptions);
}

1;
