#========================================================================================
# package      DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService
# state        public
#----------------------------------------------------------------------------------------
# description  Provides web service access to create and delete shops
#----------------------------------------------------------------------------------------
# example      use DE_EPAGES::WebService::API::Client;
#               my $ShopConfigService = DE_EPAGES::WebService::API::Client
#                   ->uri('urn://epages.de/WebService/ShopConfigService/2005/03')
#                   ->proxy("http://localhost:80/epages/Site.soap");
#               $ShopConfigService->userinfo( "/Users/$Login:$Password" );
#               $ShopConfigService->delete({
#                   Alias => 'DemoShop'
#               });
#========================================================================================
package DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService;
use base DE_EPAGES::ShopConfiguration::API::WebService::BaseProviderService;

use strict;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::IDN qw( EncodeDomain DecodeDomain IsValidIDN );
use DE_EPAGES::Core::API::Log qw( GetLog LogDebug );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Domain qw( TestWebServerScriptNamePart );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject JoinObjectPath ExistsObjectByPath );
use DE_EPAGES::Object::API::Language qw( GetCodeByLanguageID );
use DE_EPAGES::ShopConfiguration::API::Constants qw ( SHOP_ADDRESS_PREFIX @WEBSERVICE_ADDRESS_ATTRIBUTES );
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw( RunOnASPoolDB );
use DE_EPAGES::WebService::Servlet::Soap qw( GetSoapServer );
use DE_EPAGES::WebService::API::WebService qw( FormatString );

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults_ShopRef' => 'The shop alias #ShopRefAlias does not exist for provider #ProviderAlias',
        'InActiveShopType'  => 'The provider #Provider does not have any active ShopType #ShopType',
        'NoDatabase'        => 'The provider #Provider does not have any active databases',
        'DatabaseNotActive' => 'The database #Database is not active',
        'InvalidDomainName' => 'The domain name #DomainName is not valid',
        'DomainNameInUse'   => 'The domain name #DomainName is already used by another site',
        'InvalidWebServerScriptNamePart' => 'The short url part \'#WebServerScriptNamePart\' is not valid',
        'InvalidShopAlias'  => 'The shop alias \'#ShopAlias\' is not valid',
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::ShopConfiguration::API::WebService::ShopConfigService->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    return $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/ShopConfigService/2005/03',
        @Options,
    ));
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopID = $ShopConfigService->create({
#                   ShopType => $ShopType,
#                   Database => $Database,
#                   Alias => $Alias,
#                   ShopAlias => $ShopAlias,
#                   IsClosed    => $IsClosed,
#                   DomainName  => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   WebServerScriptNamePart => $WebServerScriptNamePart,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   ImportFiles => $aImportFiles,
#                   PublicFilesDir => $PublicFilesDir,
#               })->result;
# example      my $ShopID = $ShopConfigService->create({
#                   ShopType => 'ECommerce100',
#                   Database => 'Store',
#                   Alias => 'DemoShop',
#                   ShopAlias => 'DemoShop',
#                   IsClosed    => 0,
#                   DomainName  => 'www.myshop.biz',
#                   HasSSLCertificate => 1,
#                   WebServerScriptNamePart => 'demoshop',
#                   IsTrialShop => 0,
#                   IsInternalTestShop => 0,
#                   ImportFiles => [
#                       '\\Server\Shared\Distributor\CommonData.xml',
#                       '\\Server\Shared\Distributor\ExampleProducts.xml',
#                       '\\Server\Shared\Distributor\Styles.xml',
#                   ],
#                   PublicFilesDir => '\\Server\Shared\Distributor\ShopImages'
#               })->result;
#----------------------------------------------------------------------------------------
# description  Creates a new shop within the given database on behalf of the current
#               provider. The new shop is given the feature set of the specified shop
#               type. Initial shop data are imported from import file(s). These import
#               files must be pre-configured corresponding to the shop's feature set.
#               Some example import files are located in the directory
#               %EPAGES_CARTRIDGES%/DE_EPAGES/DemoShop/Database/XML.
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                   <li>ShopType | shop type alias | string
#                   <li>Database | database identifier (store alias) | string
#                   <li>Alias | unique shop name for the provider | string
#                   <li>ShopAlias | unique shop name for the database | string
#                   <li>IsClosed | true if the new shop is closed by provider (optional, default: 0) | boolean
#                   <li>$DomainName | domain name to access the shop (optional) | string
#                   <li>HasSSLCertificate | true if the web server has an SSL certificate
#                       for the shop (optional) | string
#                   <li>WebServerScriptNamePart | script name part of the shop (optional) | string
#                   <li>IsTrialShop | true if the shop is in trial period (optional, default: 0) | boolean
#                   <li>IsInternalTestShop | true if the shop is used for test or support purposes (optional, default: 0) | boolean
#                   <li>aImportFiles | list of import files contianing initial shop date. These
#                       files must be accessible for the application servers that are
#                       responsible for the specified database | string
#                   <li>PublicFilesDir | directory on the application server where image files
#                       for the new shop are located. These files are copied to the
#                       shop directory. (optional) | string
#               </ul> | reference to hash
# return       $ShopID | object id of the shop within the database | integer
#========================================================================================
sub create {
    my $self = shift;
    my ($hOptions) = @_;
    my $Provider = $self->provider;

    #copy $hOptions for logging and add provider
    my %LogOptions = %$hOptions;
    $LogOptions{Provider} = $Provider->alias;

    #remove password from logging
    my $hPWOptions;
    $hPWOptions->{MerchantPassword} = delete $LogOptions{MerchantPassword};
    $hPWOptions->{MerchantLogin}    =        $LogOptions{MerchantLogin};
    #log sensitive data only for special category
    LogDebug('create', $hPWOptions, 'ShopConfigPasswords' );

    #do general logging without sesitive data
    LogDebug('create', \%LogOptions );

    # test domain name
    my $domainName = $hOptions->{'DomainName'};
    if (defined $domainName && $domainName ne '' ) {
        $domainName = DecodeDomain($domainName);
        unless( IsValidIDN( $domainName ) ) {
            Error( 'InvalidDomainName', { 'DomainName' => $domainName } );
        }

        # test that the new vanity domain name is not used by another site
        my $EncodedDomain = EncodeDomain( $domainName );
        my $hSiteByDomain = RunOnASPoolDB( sub { shift->getSiteByDomainName( $EncodedDomain ) } );
        if( defined $hSiteByDomain ) {
            Error( 'DomainNameInUse', { DomainName => $EncodedDomain } );
        }
    }

    # test short url
    my $WebServerScriptNamePart = $hOptions->{'WebServerScriptNamePart'};
    if( defined $WebServerScriptNamePart && $WebServerScriptNamePart ne '' ) {
        if(!TestWebServerScriptNamePart($WebServerScriptNamePart)) {
            Error( 'InvalidWebServerScriptNamePart', {
                'WebServerScriptNamePart' => $WebServerScriptNamePart
            });
        }
    }

    # test the shop alias
    my $ShopAlias =  $hOptions->{ShopAlias};
    if( !defined $ShopAlias ) {
        Error( 'InvalidShopAlias', { 'ShopAlias' => '(undefined)' });
    }
    if( $ShopAlias =~ /\s/ ) {
        Error( 'InvalidShopAlias', { 'ShopAlias' => $ShopAlias });
    }

    my $ShopTypeAlias = $hOptions->{ShopType};
    TestString('ShopType', $ShopTypeAlias);
    my $ShopType = $Provider->folder('ShopTypes')->child($ShopTypeAlias);
    unless( $ShopType->get('IsActive') ) {
        Error( 'InActiveShopType', { 'Provider' => $Provider->alias, 'ShopType' => $ShopType->alias } );
    }

    # use the default database unless Database is defined
    my $StoreAlias = $hOptions->{Database};
    my $Store;
    if( defined $StoreAlias ) {
        TestString('Database', $StoreAlias);
        $Store = LoadRootObject()->folder('Stores')->child($StoreAlias);
    }
    else {
        my $HasUnityEnabled = $ShopType->get('HasUnityEnabled');
        # use default store for unity shops
        $Store = $Provider->get('DefaultStoreUnity') if $HasUnityEnabled;
        # use default store for all other shops or if no unity default defined
        $Store = $Provider->get('DefaultStore') unless defined $Store;
        unless (defined $Store) {
            # try to find a usable database, if there is no explicit default
            my @ActiveStores = grep { $_->get('Connection')->{'active'} } @{$Provider->stores};
            if( 0 == scalar @ActiveStores ) {
                Error( 'NoDatabase', { 'Provider' => $Provider->alias } );
            }
            $Store = $ActiveStores[0];
        }
        GetLog()->debug("create Shop \'" . $ShopAlias . ($HasUnityEnabled ? "\' with" : "\' without")
            . ' feature Unity on store: ' . $Store->alias);
    }

    unless( $Store->get('Connection')->{'active'} ) {
        Error( 'DatabaseNotActive', { 'Database' => $Store->alias } );
    }

    my $ShopRef = $Provider->insertShop({
        ShopType => $ShopType,
        Store => $Store,
        Alias => $hOptions->{Alias},
        ShopAlias => $hOptions->{ShopAlias},
        ImportFiles => $hOptions->{ImportFiles},
        IsClosed => $hOptions->{'IsClosed'} ? 1 : 0,
        DomainName => $domainName,
        HasSSLCertificate => $hOptions->{'HasSSLCertificate'} ? 1 : 0,
        WebServerScriptNamePart => $hOptions->{'WebServerScriptNamePart'},
        IsTrialShop => $hOptions->{'IsTrialShop'} ? 1 : 0,
        IsInternalTestShop => $hOptions->{'IsInternalTestShop'} ? 1 : 0,
        PublicFilesDir => $hOptions->{'PublicFilesDir'},
    });

    return $ShopRef->get('StoreObjectID');
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopConfigService->delete({
#                   Alias => $Alias
#               });
# example      $ShopConfigService->delete({
#                   Alias => 'DemoShop'
#               });
#----------------------------------------------------------------------------------------
# description  Deletes a shop on behalf of the provider. The shop data is deleted
#               permanently from the shop database, but a reference to the deleted shop
#               remains in the site database for accounting and statistics purposes.
#               To remove this shop reference from the site database use
#               <function deleteShopRef>
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                 <li>Alias | unique shop name for the provider | string
#               </ul> | reference to hash
#========================================================================================
sub delete {
    my $self = shift;
    my ($hOptions) = @_;
    my $Provider = $self->provider;
    LogDebug('delete', [$hOptions,{'Provider',$Provider->alias}]);

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    return unless $Provider->existsShop( $ShopRefAlias );

    $Provider->shop( $ShopRefAlias )->deleteRemoteShop;
    return;
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $ShopConfigService->exists({
#                   Alias => $Alias
#               })->result;
# example      print "hurray" if $ShopConfigService->exists({
#                   Alias => 'DemoShop'
#               })->result;
#----------------------------------------------------------------------------------------
# description  Returns true if the provider has a shop with the given alias name.
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                   <li>Alias | unique shop name for the provider | string
#               </ul> | reference to hash
# return       $Exists | boolean value is tru if shop exists | soap data
#========================================================================================
sub exists {
    my $self = shift;
    my ($hOptions) = @_;
    my $Provider = $self->provider;
    LogDebug('exists', [$hOptions,{'Provider',$Provider->alias}]);

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    my $Exists = $Provider->existsShop( $ShopRefAlias ) ? 1 : 0;
    return SOAP::Data->name('Exists')->type( 'boolean' => $Exists );
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopConfigService->update({
#                   Alias => $Alias,
#                   ShopType => $ShopType,
#                   IsClosed => $IsClosed,
#                   DomainName => $DomainName,
#                   HasSSLCertificate => $HasSSLCertificate,
#                   WebServerScriptNamePart => $WebServerScriptNamePart,
#                   IsTrialShop => $IsTrialShop,
#                   IsInternalTestShop => $IsInternalTestShop,
#                   ImportFiles => $ImportFiles,
#               });
#----------------------------------------------------------------------------------------
# description  Updates a shop by changing the shop type, setting the status or importing
#               data from XML files.
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                   <li>Alias | unique shop name for the provider | string
#                   <li>ShopType | (optional) new shop type alias | string
#                   <li>IsClosed | (optional) true if closed by provider | boolean
#                   <li>MarkedForDelete | (optional) true if shop is marked for deletion | boolean
#                   <li>DomainName | (optional) domain name to access the shop | string
#                   <li>HasSSLCertificate | true if the web server has an SSL certificate
#                                    for the shop (optional) | string
#                   <li>WebServerScriptNamePart | script name part of the shop (optional) | string
#                   <li>IsTrialShop | true if the shop is in trial period (optional, default: 0) | boolean
#                   <li>IsInternalTestShop | true if the shop is used for test or support purposes (optional, default: 0) | boolean
#                   <li>ImportFiles | (optional) list of files names to import | ref.array.string
#               </ul> | reference to hash
#========================================================================================
sub update {
    my $self = shift;
    my ($hOptions) = @_;
    my $Provider = $self->provider;

    #copy $hOptions for logging and add provider
    my %LogOptions = %$hOptions;
    $LogOptions{Provider} = $Provider->alias;

    #remove password from logging
    my $hPWOptions;
    $hPWOptions->{MerchantPassword} = delete $LogOptions{MerchantPassword};
    $hPWOptions->{MerchantLogin}    =        $LogOptions{MerchantLogin};
    #log sensitive data only for special category
    LogDebug('update', $hPWOptions, 'ShopConfigPasswords' );

    #do general logging without sesitive data
    LogDebug('update', \%LogOptions );

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    Error( 'NoResults_ShopRef', {
        ShopRefAlias => $ShopRefAlias,
        ProviderAlias => $Provider->alias,
    } ) unless $Provider->existsShop( $ShopRefAlias );

    my $ShopRef = $Provider->shop( $ShopRefAlias );

    my %Values;
    if (defined $hOptions->{'DomainName'}) {
        if( $hOptions->{'DomainName'} ne '' ) {
            $Values{'DomainName'} = DecodeDomain( $hOptions->{'DomainName'} );

            unless( IsValidIDN( $Values{'DomainName'} ) ) {
                Error( 'InvalidDomainName', { 'DomainName' => $hOptions->{'DomainName'} } );
            }

            # test that the new vanity domain name is not used by another site
            my $EncodedDomain = EncodeDomain( $hOptions->{'DomainName'} );
            my $hSiteByDomain = RunOnASPoolDB( sub { shift->getSiteByDomainName( $EncodedDomain ) } );
            if( defined $hSiteByDomain
                && ($hSiteByDomain->{'StoreAlias'} ne $ShopRef->get('Store')->alias
                 || $hSiteByDomain->{'ObjectPath'} ne $ShopRef->get('StoreObjectPath')) )
            {
                Error( 'DomainNameInUse', { DomainName => $hOptions->{'DomainName'} } );
            }
        }
        else {
            $Values{'DomainName'} = undef;
        }
    }

    if( defined $hOptions->{'NewAlias'}) {
        TestString('Alias', $hOptions->{'NewAlias'});
        my $Path = JoinObjectPath([
            'Providers',
            $Provider->get('Alias'),
            'ShopRefs',
            $hOptions->{'NewAlias'}
        ]);

        #check if Object exists
        Error( 'ShopRefAlreadyExistsOnSiteDB', { 'ObjectPath' => $Path } ) if ExistsObjectByPath($Path);
    }

    if( defined $hOptions->{'ShopAlias'} ) {
        if ($ShopRef->remoteShop->get('Alias') ne $hOptions->{'ShopAlias'}){
            TestString('Alias', $hOptions->{'ShopAlias'});
            my $Path = JoinObjectPath(['Shops',$hOptions->{'ShopAlias'}]);

            Error( 'ShopAlreadyExistsOnStoreDB', { 'ShopAlias' => $hOptions->{'ShopAlias'} } ) if $ShopRef->remoteShop->factoryService->existsObjectByPath($Path)->result;
        }
    }

    if( defined $hOptions->{'ShopType'} ) {
        my $ShopTypeAlias = $hOptions->{'ShopType'};
        my $ShopType = $Provider->folder('ShopTypes')->child($ShopTypeAlias);
        $ShopRef->set({ 'ShopType' => $ShopType });
    }

    if( defined $hOptions->{'ImportFiles'} ) {
        $ShopRef->remoteShop->importFiles( $hOptions->{'ImportFiles'} );
    }

    if( defined $hOptions->{'MarkedForDelete'} && $hOptions->{'MarkedForDelete'}) {
        $ShopRef->set({'IsClosed' => 1});
        $ShopRef->set({'MarkedForDelOn' => GetCurrentDBHandle()->currentDateTime()});
    }
    elsif(( defined $hOptions->{'MarkedForDelete'} && !$hOptions->{'MarkedForDelete'})
     || (defined $hOptions->{'IsClosed'} && !$hOptions->{'IsClosed'}) ){
        $ShopRef->set({'MarkedForDelOn' => undef});
    }

    if( defined $hOptions->{'NewAlias'} ) {
        $ShopRef->set({'Alias' => $hOptions->{'NewAlias'}});
    }

    if( defined $hOptions->{'ShopAlias'} ) {
        if ($ShopRef->remoteShop->get('Alias') ne $hOptions->{'ShopAlias'}){
            $ShopRef->remoteShop->set({'Alias' => $hOptions->{'ShopAlias'}});
            $ShopRef->set({'StoreObjectAlias' => $hOptions->{'ShopAlias'}});
        }
    }

    # string attributes
    foreach my $Name (qw( WebServerScriptNamePart )) {
        next unless defined $hOptions->{$Name};
        $Values{$Name} = $hOptions->{$Name};
    }

    # boolean attributes
    foreach my $Name (qw( HasSSLCertificate IsClosed IsTrialShop IsInternalTestShop )) {
        next unless defined $hOptions->{$Name};
        $Values{$Name} = $hOptions->{$Name} ? 1 : 0;
    }

    $ShopRef->set( \%Values ) if scalar keys %Values;
    return;
}

#========================================================================================
# function     getInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShopInfo = $ShopConfigService->getInfo({
#                   Alias => $Alias
#               })->result;
#----------------------------------------------------------------------------------------
# description  Returns information about a shop from the site database.
#----------------------------------------------------------------------------------------
# input        $Alias | unique shop name for the provider | string
# input        $hOptions | with following parameters:
#               <ul>
#               </ul> | reference to hash
# return       $hShopInfo | shop data <ul>
#                         <li>Provider - provider alias - string</li>
#                         <li>Alias - unique shop name for the provider - string</li>
#                         <li>ShopType - shop type alias - string</li>
#                         <li>Database - store database identifier - string</li>
#                         <li>ObjectPath - object path of the shop in the store database - string</li>
#                         <li>IsClosed - true if closed by provider - boolean</li>
#                         <li>IsDeleted - true if deleted in store database - boolean</li>
#                         <li>DomainName - shop domain name - string</li>
#                         <li>HasSSLCertificate - true if the web server has an SSL certificate
#                                                 for the shop - string</li>
#                       </ul> | ref.hash.string
#========================================================================================
sub getInfo {
    my $self = shift;
    my ($hOptions) = @_;
    my $Provider = $self->provider;
    LogDebug('getInfo', [$hOptions,{'Provider',$Provider->alias}]);

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    Error( 'NoResults_ShopRef', {
        ShopRefAlias => $ShopRefAlias,
        ProviderAlias => $Provider->alias,
    } ) unless $Provider->existsShop( $ShopRefAlias );

    my $ShopRef = $Provider->shop( $ShopRefAlias );

    # register type names of complex types
    GetSoapServer()->serializer->maptype->{'TInfoShop'} = $self->getServiceUrn;

    return $self->getTInfoShop( $ShopRef );
}

#========================================================================================
# function     getAllInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahShopInfo = $ShopConfigService->getAllInfo()->result;
#----------------------------------------------------------------------------------------
# description  Returns information about all shops of the provider.
#----------------------------------------------------------------------------------------
# return       $hShopInfo | list of shop data. each element has the structure <ul>
#                         <li>Provider - provider alias - string</li>
#                         <li>Alias - unique shop name for the provider - string</li>
#                         <li>ShopType - shop type alias - string</li>
#                         <li>Database - store database identifier - string</li>
#                         <li>ObjectPath - object path of the shop in the store database - string</li>
#                         <li>IsClosed - true if closed by provider - boolean</li>
#                         <li>IsDeleted - true if deleted in store database - boolean</li>
#                         <li>DomainName - shop domain name - string</li>
#                         <li>HasSSLCertificate - true if the web server has an SSL certificate
#                                                 for the shop - string</li>
#                       </ul> | ref.array.hash.string
#========================================================================================
sub getAllInfo {
    my $self = shift;
    my $Provider = $self->provider;
    LogDebug('getAllInfo', [{'Provider',$Provider->alias}]);

    # register type names of complex types
    GetSoapServer()->serializer->maptype->{'TInfoShop'} = $self->getServiceUrn;

    my $aShopRefs = $Provider->get('ShopRefs');
    my @ShopInfos = map { $self->getTInfoShop($_) } @$aShopRefs;

    return SOAP::Data->name( 'ShopInfos' )->value( \@ShopInfos );
}

# create a TInfoShop structure from a ShopRef object without HasSSLCertificate
sub getTInfoShop {
    my $self = shift;
    my ($ShopRef) = @_;

    my $Provider = $ShopRef->get('Provider');
    my $ShopType = $ShopRef->get('ShopType');
    my $Store = $ShopRef->get('Store');

    return SOAP::Data->type( 'TInfoShop')->name('ShopInfo' => {
        'Provider'          => $self->soapString( $Provider->alias ),
        'Alias'             => $self->soapString( $ShopRef->alias ),
        'ShopType'          => $self->soapString( defined $ShopType ? $ShopType->alias : undef ),
        'Database'          => $self->soapString( defined $Store ? $Store->alias : undef ),
        'ObjectPath'        => $self->soapString( $ShopRef->get('StoreObjectPath') ),
        'IsClosed'          => $self->soapBool( $ShopRef->get('IsClosed') ),
        'IsDeleted'         => $self->soapBool( $ShopRef->get('IsDeleted') ),
        'DomainName'        => $self->soapString( $ShopRef->get('DomainName')),
    } );
}

#========================================================================================
# function     deleteShopRef
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShopConfigService->deleteShopRef({
#                   Alias => $Alias
#               });
# example      $ShopConfigService->delete({
#                   Alias => 'DemoShop'
#               });
#----------------------------------------------------------------------------------------
# description  Deletes a shop reference from the site database. Call this funtion
#               after the shop data has been deleted with <function delete>
#               or when the shop database is no longer accessible.
#               Do not call this function while the shop data still exists in the
#               shop database!
#----------------------------------------------------------------------------------------
# input        $hOptions | with following parameters:
#               <ul>
#                   <li>Alias | unique shop name for the provider | string
#               </ul> | reference to hash
#========================================================================================
sub deleteShopRef {
    my $self = shift;
    my ($hOptions) = @_;
    my $Provider = $self->provider;
    LogDebug('deleteShopRef', [$hOptions,{'Provider',$Provider->alias}]);

    my $ShopRefAlias = $hOptions->{Alias};
    TestString('Alias', $ShopRefAlias);

    return unless $Provider->existsShop( $ShopRefAlias );

    $Provider->shop( $ShopRefAlias )->delete;

    return;
}

#========================================================================================
# function     setInitialShopData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SimpleProvisioningService->setInitialShopData({
#                   Alias => $Alias,
#                   MerchantEMail => $MerchantEMail,
#                   ShopAddress_FirstName => $FirstName,
#                   ShopAddress_LastName => $LastName,
#                   ShopAddress_CountryID => $CountryID,
#                   ShopAddress_Street => $Street,
#                   ShopAddress_Zipcode => $Zipcode,
#                   ShopAddress_City => $City,
#                   ShopAddress_State => $State,
#                   Name => $Name,
#               });
# example       $SimpleProvisioningService->setInitialShopData({
#                   Alias => 'DemoShop',
#                   MerchantEMail => 'hab@ich.net',
#                   ShopAddress_FirstName => 'Max',
#                   ShopAddress_LastName => 'Mustermann',
#                   ShopAddress_CountryID => '276',
#                   ShopAddress_Street => 'Heinrich-Heine-Strae 1',
#                   ShopAddress_Zipcode => '07749',
#                   ShopAddress_City => 'Jena',
#                   ShopAddress_State => 'TH',
#                   Name => 'Milestones',
#               });
#----------------------------------------------------------------------------------------
# description  Sets the initial shop data (address and shop name) on the remote shop.
#----------------------------------------------------------------------------------------
# input        $hOptions | shop data <ul>
#                         <li>Alias | unique shop identifier | string</li>
#                         <li>MerchantEMail | merchant email address (optional) | string</li>
#                         <li>ShopAddress_FirstName | merchant first name | string</li>
#                         <li>ShopAddress_LastName | merchant last name | string</li>
#                         <li>ShopAddress_CountryID | merchant enumeric country code according to ISO 3166-1 | integer</li>
#                         <li>ShopAddress_Street | merchant street and house number | string</li>
#                         <li>ShopAddress_Zipcode | merchant zipcode | string</li>
#                         <li>ShopAddress_City | merchant city | string</li>
#                         <li>ShopAddress_State | merchant state code according to ISO 3166-2 | string</li>
#                         <li>Name | name of the shop | string</li>
#                       </ul> | ref.hash.string
#========================================================================================
sub setInitialShopData {
    my $self = shift;
    my ( $hOptions ) = @_;

    my $ShopRef = $self->provider->shop( $hOptions->{Alias} );
    my $RemoteShop = $ShopRef->remoteShop;
    my %ShopAddress;
    for my $Option (keys %$hOptions) {
        next unless $Option =~ m/^${\(SHOP_ADDRESS_PREFIX)}/;
        my $AddressAttribute = $Option;
        $AddressAttribute =~ s/^${\(SHOP_ADDRESS_PREFIX)}//;
        $ShopAddress{$AddressAttribute} = FormatString($hOptions->{$Option});
    }
    $ShopAddress{'EMail'} = FormatString($hOptions->{'MerchantEMail'}) if defined $hOptions->{'MerchantEMail'};
    $RemoteShop->setShopAddress( \%ShopAddress ) if %ShopAddress;
    my $LanguageCode = GetCodeByLanguageID( $RemoteShop->get('LanguageID') );
    $RemoteShop->set( {'Name' => FormatString($hOptions->{'Name'})}, $LanguageCode ) if defined $hOptions->{'Name'};

    return;
}

#========================================================================================
# function     getInitialShopData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SimpleProvisioningService->getInitialShopData({
#                   Alias => $Alias
#               });
# example       $SimpleProvisioningService->getInitialShopData({
#                   Alias => 'DemoShop'
#               });
#----------------------------------------------------------------------------------------
# description  Gets the initial shop data (address and shop name) of the remote shop.
#----------------------------------------------------------------------------------------
# input        $hOptions | shop data <ul>
#                         <li>Alias | unique shop identifier | string</li>
#                       </ul> | ref.hash.string
# $return       $AdditionalInfo | shop information <ul>
#                         <li>ShopAddress_EMail | merchant email address | string</li>
#                         <li>ShopAddress_FirstName | merchant first name | string</li>
#                         <li>ShopAddress_LastName | merchant last name | string</li>
#                         <li>ShopAddress_CountryID | merchant enumeric country code according to ISO 3166-1 | integer</li>
#                         <li>ShopAddress_Street | merchant street and house number | string</li>
#                         <li>ShopAddress_Zipcode | merchant zipcode | string</li>
#                         <li>ShopAddress_City | merchant city | string</li>
#                         <li>ShopAddress_State | merchant state code according to ISO 3166-2 | string</li>
#                         <li>Name | name of the shop | string</li>
#                       </ul> | ref.hash.string
#========================================================================================
sub getInitialShopData {
    my $self = shift;
    my ( $hOptions ) = @_;

    my $ShopRef = $self->provider->shop( $hOptions->{Alias} );
    my $RemoteShop = $ShopRef->remoteShop;
    my $ShopAddress = $RemoteShop->getShopAddress;
    my %AdditionalInfo;

    for my $Attribute (keys %$ShopAddress) {
        next unless $Attribute ~~ @WEBSERVICE_ADDRESS_ATTRIBUTES;
        $AdditionalInfo{SHOP_ADDRESS_PREFIX . $Attribute} = $self->soapString($ShopAddress->{$Attribute});
    }
    my $LanguageCode = GetCodeByLanguageID( $RemoteShop->get('LanguageID') );
    my $ShopName = $RemoteShop->get('Name', $LanguageCode);
    $AdditionalInfo{'Name'} = $self->soapString($ShopName) if defined $ShopName;

    return \%AdditionalInfo;
}

1;
