#========================================================================================
# package      DE_EPAGES::TaxUS::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::TaxUS::API::Cartridge;
use base DE_EPAGES::Design::API::DesignInstaller;

use strict;

use DE_EPAGES::Core::API::Log qw(GetLog);
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject Transaction );
use DE_EPAGES::Object::API::XML::AttributeImportHandler qw ( ImportTranslation );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw ( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::TaxUS::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::TaxUS.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/TaxUS',
        'Version'            => '7.18.0',
        'Patches'            => ['6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;
    $self->startInstall;
    $self->SUPER::install;

    Transaction(sub {
        my $XMLDir = $self->xmlDirectory;
        ImportObjects("$XMLDir/TaxMatrixUS.xml");
        ImportTranslation("$XMLDir/Translation.TaxMatrixUS");
        ImportObjects( $self->xmlDirectory .'/TaxTypes.xml' );
        _InsertTaxMatrixUS();
    });

    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;

    my $XMLDir = $self->xmlDirectory;
    DeleteObjects("$XMLDir/TaxMatrixUS.xml");
    DeleteObjects("$XMLDir/TaxTypes.xml");

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patchDBStructure_6_17_25
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_25;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.25' to version '6.17.26'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.26' | string
#========================================================================================
sub patchDBStructure_6_17_25 {
    my $self = shift;

    GetLog()->info( 'import patch dir Patch_6_17_25, PatchDBStructure' );
    $self->importPatchDir('Patch_6_17_25', 'PatchDBStructure');

     return '6.17.26';
}

#========================================================================================
# function     patch_6_17_30
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_30;
#----------------------------------------------------------------------------------------
# description  Patches cartridge in active store (database connection)
#               from version '6.17.30' to version '6.17.31'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.31' | string
#========================================================================================
sub patch_6_17_30 {
    my $self = shift;

    $self->importPatchDir('Patch_6_17_30', 'Patch');
    _InsertTaxMatrixUS();

    return '6.17.31';
}

sub _InsertTaxMatrixUS {

    my $System = LoadRootObject();
    my $SystemTaxMatrix = $System->child('TaxMatrixUS');

    Error('NoTaxMatrixUS' => {Path => $SystemTaxMatrix->get('Path')})
    unless $SystemTaxMatrix->instanceOf('TaxMatrixUS');

    foreach my $Shop (@{LoadRootObject->get('Shops')}){
        $SystemTaxMatrix->clone({
                'Parent'       => $Shop,
                'CreationDate' => GetCurrentDateTime()
            },1
        ) unless $Shop->existsChild('TaxMatrixUS');
    }

    return;
}





1;
